--[[
	To Do:
		-- fix draenor and dungeons! Note: Blizzard has disable coords for all player garrisons on draenor, and all dungeons/instances.
		-- Eversong Woods does not return coords for Azeroth, returns for outland. needs adjustments.
--]]
local addonName, addon = ...
local Masque = LibStub("Masque", true)

local Questra = {}

local HBD = LibStub:GetLibrary("HereBeDragons-2.0-Modified", true)
local CTL = assert(ChatThrottleLib, "Quest_Compass requires ChatThrottleLib.")
local FlyPaper = LibStub('LibFlyPaper-2.0')

local Quest_Compass = CreateFrame("Frame", "Quest_Compass", UIParent)
Quest_Compass.position = {x=0, y=0}
Q_C = Quest_Compass
_G["Q_C"] = Q_C
Q_C.notification = { click = function(...) end, tooltip = function(...)end, icon = function(...)end}
Q_C.navigation = { click = function(...) end, tooltip = function(...)end, icon = function(...)end}
local travelQueues = {
						flightMaster = {},
						userSelected = {},
						anchors = {},
						suggestions = {},
						rare = {},
						playerShared = {},
						target = {},
						creatureYell = {},
						portal = {},
					}

Q_C.frame = CreateFrame("Frame", addonName.."Frame", UIParent)
Q_C.frame:RegisterEvent("ADDON_LOADED")
Q_C.frame:SetSize(100, 100)
Q_C.frame:SetScript("OnEvent", function(_, event, arg1)
	if (event == "ADDON_LOADED") and arg1 == addonName then
		Q_C:Create()
	end
end)

Q_C.TexturePath = "Interface\\AddOns\\"..addonName.."\\artwork\\"

do --Base Frame Creation
	function Q_C:Create()
		self:EnableSettings()

		WorldMapFrame:SetClampedToScreen(true)
		self.frame:SetMovable(true)
		self.frame:SetUserPlaced(true)
		self.frame:SetSize(100, 100)

		self:CreateCoordinatesDisplay()
		self:CreatePasteFrame()
		self:CreateSaveButton()
		self:CreatePersistButton()
		self:CreateCompass()
		self:CreateArrow()
		self:CreateOverlay()

		self:CreateNotification()
		self:ActivateScripts()

		self.distText = self.frame:CreateFontString(addonName.."DistanceText", "ARTWORK", "GameFontNormal")
		self.distText:SetPoint("Center", self.Notify, "Center", 0, 2)
		self.distText:SetDrawLayer("OVERLAY", 2)

		self.Notify.Icon:SetDrawLayer("BACKGROUND", 2)
		self.Notify:SetFrameStrata("BACKGROUND")

		self.timeLeftbg = CreateFrame("Frame", self.pin, self, "BackdropTemplate")
		self.timeLeftbg:SetScale(1)
		self.timeLeftbg:SetFrameStrata("HIGH", 1)

		self.timeLeft = self.timeLeftbg:CreateFontString(addonName.."PercentText", "ARTWORK", "GameFontNormalSmall")
		self.timeLeft:SetPoint("BottomLeft", self.pin, "Bottom", 13, -4)
		self.timeLeftbg:SetAllPoints(self.timeLeft)

		self.timeLeftbg:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true,
			tileEdge = true,
			tileSize = 8,
			edgeSize = 4,
			insets = { left = 0, right = 0, top = 0, bottom = 0 },
		})

		self.percent = self.pin:CreateFontString(addonName.."PercentText", "ARTWORK", "GameFontNormalSmall")
		self.percent:SetPoint("BottomLeft", self.pin, "BottomLeft", -3, -3)

		ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", self.FilterForCoords)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", self.FilterForCoords)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", self.FilterForCoords)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", self.FilterForCoords)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", self.RareYellFilter)
		--ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", self.FilterForCoords)

		self:Layout()
		self.frame.id = "Q_C_Frame"
		self.frame.sets = self:GetSets()
		FlyPaper:RegisterFrame(addonName, addon, self.frame)
		FlyPaper:ActivateForAddon(addonName)
	end

	function Q_C:EnableSettings()
		if not Quest_CompassDB then
			Quest_CompassDB = CopyTable(self.defaults)
		end

		if not Quest_CompassAltDB then
			Quest_CompassAltDB = CopyTable(self.defaults)
		end

		local db = Quest_CompassDB
		local alt =  Quest_CompassAltDB

		db.version =  db.version and db.version + 1 or 1
		self.sets = Quest_CompassDB

		self.travelQueues =  db.travelQueues or {
			flightMaster = {},
			userSelected = {},
			anchors = {},
			suggestions = {},
			rare = {},
			playerShared = {},
			target = {},
			creatureYell = {},
		}
		db.travelQueues = self.travelQueues

		for i, b in pairs(travelQueues) do
			db.travelQueues[i] = db.travelQueues[i] or b
		end
	end

	function Q_C:CreateCoordinatesDisplay()
		self.coordText = CreateFrame("EditBox", addonName.."EditBox", self.frame, "InputBoxTemplate")
		self.coordText:SetFontObject("GameFontHighlight")
		self.coordText:SetHighlightColor(.5,1,.5)
		self.coordText:SetClampedToScreen(true)
		self.coordText:SetPoint("Bottom", 0, 0)
		self.coordText:SetJustifyH("CENTER")
		self.coordText.Left:SetPoint("Left")
		self.coordText:SetAutoFocus(false)
		self.coordText:SetMovable(true)
		self.coordText:SetSize(98, 20)

		self.coordText.button = CreateFrame("Button", nil, self.coordText)
		self.coordText.button:SetAllPoints(self.coordText)
		self.coordText.button:RegisterForClicks("AnyUp")

		self.content = CreateFrame("ScrollFrame", nil, self.frame, "HybridScrollFrameTemplate, BackdropTemplate")
		self.content:SetSize(200,350)
		self.content:SetPoint("Center", UIParent)
		self.content.sticker = CreateFrame("Frame", nil, self.content)
		self.content.sticker:SetSize(100, 100)
		self.content:SetScrollChild(self.content.sticker)
	end

	function Q_C:CreatePasteFrame()
		self.Paste = CreateFrame("EditBox", addonName.."PasteBox", self.content.sticker, "InputBoxScriptTemplate")
		self.Paste:SetPoint("TopLeft", 0, 0)
		self.Paste:SetFontObject("GameFontNormalLarge")
		self.content:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true,
			tileEdge = true,
			tileSize = 32,
			edgeSize = 16,
			insets = { left = -10, right = -10, top = -10, bottom = -10 },
		})

		self.content:SetBackdropColor(0,0,0)
		self.Paste:SetMultiLine(true)
		self.Paste:SetSize(205, 50)
		self.Paste:SetAutoFocus(false)
		self.content:Hide()

		local content = self.content
		function self.Paste:Show()
			content:Show()
		end

		function self.Paste:Hide()
			content:Hide()
		end
	end

	function Q_C:CreateSaveButton()
		self.saveButton = CreateFrame("Button", addonName.."SaveButton", self.frame)
		self.saveButton:SetHighlightTexture(self.TexturePath.."LeftArrow-Highlight")
		self.saveButton:SetPushedTexture(self.TexturePath.."LeftArrow-Pushed")
		self.saveButton:SetNormalTexture(self.TexturePath.."LeftArrow")
		self.saveButton:SetPoint("Right", self.coordText, "Left", 2, 0)
		self.saveButton:GetHighlightTexture():SetBlendMode("ADD")
		self.saveButton:RegisterForClicks("AnyUp")
		self.saveButton:SetSize(10, 20)
	end

	function Q_C:CreatePersistButton()
		self.persistButton = CreateFrame("Button", addonName.."PersistButton", self.frame)
		self.persistButton:SetHighlightTexture(self.TexturePath.."LeftArrow-Highlight")
		self.persistButton:SetPushedTexture(self.TexturePath.."LeftArrow-Pushed")
		self.persistButton:SetNormalTexture(self.TexturePath.."LeftArrow")
		self.persistButton:SetPoint("Left", self.coordText, "Right", -3, 0)
		self.persistButton:GetHighlightTexture():SetTexCoord(1,0,0,1)
		self.persistButton:GetHighlightTexture():SetBlendMode("ADD")
		self.persistButton:GetNormalTexture():SetTexCoord(1,0,0,1)
		self.persistButton:GetPushedTexture():SetTexCoord(1,0,0,1)
		self.persistButton:SetSize(10, 20)
	end

	function Q_C:CreateCompass()
		self.pin = CreateFrame("Button", addonName.."Pin", self.coordText)
			--[[
				parenting this to the coordText
				seems to avoid taint issues,
				once we add the group finder
				button and item button
			--]]
		local pin = self.pin
		pin:SetPoint("Center", self.frame, 0, 50)
		pin:SetSize(60, 60)

		self.pin:SetFrameStrata("MEDIUM")
		pin:SetNormalTexture(2909091)
		pin.normalTexture = pin:GetNormalTexture()
		pin.normalTexture:SetDrawLayer("BACKGROUND", -1)
		pin.normalTexture:SetAllPoints(pin)

		pin:SetPushedTexture(2909091)
		pin.pushedTexture = pin:GetPushedTexture()
		pin.pushedTexture:SetDrawLayer("BACKGROUND", -1)
		pin.pushedTexture:SetAllPoints(pin)

		pin:SetHighlightTexture(166862)
		pin.Highlight = pin:GetHighlightTexture()

		pin.Faction = pin:CreateTexture(addonName.."PinFactionIcon", "ARTWORK")
		pin.Faction:SetPoint("Center")

		pin.Texture = pin:CreateTexture(addonName.."PinTexture", "BACKGROUND")

		pin.border = pin:CreateTexture(addonName.."PinBoder", "ARTWORK")
		pin.border:SetPoint("Center")
		pin.border:SetSize(105, 105)
		pin.border:SetTexture(self.TexturePath.."UI-MINIMAP-BORDER")

		pin.bg = pin:CreateTexture(addonName.."PinBoder", "LOW")
		pin.bg:SetPoint("Center", 0, 0)
		pin.bg:SetSize(95, 110)
		pin.bg:SetTexture(self.TexturePath.."UI-MINIMAP-BORDER-background")
		if Masque then
			local icon =  pin.border
			local iconTexture = pin.Texture

			self.masqueGroup = Masque:Group(addonName)
			self.masqueGroup:AddButton(pin, {
				--FloatingBG = {...},
				Icon = pin.normalTexture,
				--Cooldown = {...},
				--Flash = {...},
				Pushed = pin.pushedTexture,
				--Normal = pin.border,
				--Disabled = {...},
				--Checked = {...},
				Border = pin.border,
				--AutoCastable = {...},
				Highlight = pin.Highlight,
				--HotKey = {...},
				--Count = Q_C.distText,
				--Name = {...},
				--Duration = {...},
				--Shine = {...},
			})
			self.icon = icon
			self.iconTexture = iconTexture
		end
	end

	function Q_C:CreateArrow()
		self.arrowHandle = CreateFrame("Frame", nil, self)
		self.arrowHandle:SetAllPoints(self.pin)
		self.arrowHandle:SetFrameStrata("HIGH")

		self.arrow = self.arrowHandle:CreateTexture(addonName.."PinArrow", "OVERLAY")
		self.arrow:SetTexture(self.TexturePath.."WorldMapArrow")
		self.arrow:SetAllPoints(self.pin.normalTexture)--"Center", self.pin)
	end

	function Q_C:CreateOverlay()
		-- Frame movement and Options menu Handler
		self.optionMenuOverlay = CreateFrame("Frame", addonName.."OptionsDisplay", self.frame)
		self.optionMenuOverlay.overlay = self.optionMenuOverlay:CreateTexture(addonName.."OptionsDisplayOverlay", 'OVERLAY')
		self.optionMenuOverlay.overlay:SetAllPoints(self.optionMenuOverlay)
		self.optionMenuOverlay.overlay:SetColorTexture(0,0,1, .25)
		self.optionMenuOverlay:SetAllPoints(self.frame)
		self.optionMenuOverlay:EnableMouse(true)
		self.optionMenuOverlay:SetFrameLevel(50)
		self.optionMenuOverlay:Hide()
	end

	function Q_C:CreateNotification()
		--notification stuffs
		self.Notify = CreateFrame("Button", addonName.."NotificationIcon", self)
		--self.Notify:SetFrameStrata("MEDIUM")
		self.Notify:RegisterForClicks("AnyUp")

		local x,y = self.pin:GetSize()
		self.Notify:ClearAllPoints()
		self.Notify:SetPoint("TopLeft", self.coordText, "BottomLeft", 0, 0)
		self.Notify:SetPoint("TopRight", self.coordText, "BottomRight", 0, 0)
		--self.Notify:SetAlpha(.75)
		self.Notify:SetHeight(15)

		self.Notify.Icon = self.Notify:CreateTexture(nil, "OVERLAY")
		self.Notify.Icon:SetAllPoints(self.Notify)
		self.Notify.Icon:SetTexture(self.TexturePath.."notifyBorder")

		self.Notify:SetNormalTexture(self.TexturePath.."notifyIcon")
		self.Notify.normalTexture = self.Notify:GetNormalTexture()
		self.Notify.normalTexture:SetDrawLayer("BACKGROUND", -1)
		self.Notify.normalTexture:ClearAllPoints()
		self.Notify.normalTexture:SetAllPoints(self.Notify)

		self.Notify:SetPushedTexture(self.TexturePath.."notifyPushed")
		self.Notify.pushedTexture = self.Notify:GetPushedTexture()
		self.Notify.pushedTexture:SetDrawLayer("BACKGROUND", -1)
		self.Notify.pushedTexture:ClearAllPoints()
		self.Notify.pushedTexture:SetAllPoints(self.Notify.normalTexture)

		self.Notify:SetHighlightTexture(self.TexturePath.."notifyHighlight")
		self.Notify.Highlight = self.Notify:GetHighlightTexture()
		self.Notify.Highlight:ClearAllPoints()
		self.Notify.Highlight:SetAllPoints(self.Notify.normalTexture)

		self.Notify.qty = self.Notify:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		self.Notify.qty:SetPoint("Center", self.Notify.Icon, 0, -7)
		self.Notify.qty:SetSize(12, 12)
	end


local events  = {
	"CORPSE_IN_INSTANCE",
	"CORPSE_IN_RANGE",
	"CORPSE_OUT_OF_RANGE",
	"PLAYER_ALIVE",
	"PLAYER_DEAD",

	"ITEM_INTERACTION_CLOSE",
	"ITEM_INTERACTION_OPEN",

	"USER_WAYPOINT_UPDATED",

	"ZONE_CHANGED",
	"ZONE_CHANGED_INDOORS",
	"ZONE_CHANGED_NEW_AREA",

	"NAVIGATION_FRAME_CREATED",
	"NAVIGATION_FRAME_DESTROYED",



	"QUEST_ACCEPTED",
	"QUEST_AUTOCOMPLETE",
	"QUEST_COMPLETE",

	"QUEST_POI_UPDATE",
	"QUEST_REMOVED",
	"QUEST_TURNED_IN",
	"QUEST_WATCH_LIST_CHANGED",
	"QUEST_WATCH_UPDATE",
	"QUESTLINE_UPDATE",


	"WAYPOINT_UPDATE",
	"WORLD_QUEST_COMPLETED_BY_SPELL",

	"QUEST_ACCEPT_CONFIRM",
	"QUEST_FINISHED",

	"QUEST_PROGRESS",
	
	"SUPER_TRACKING_CHANGED",
	"UNIT_TARGET",
	
	"LFG_QUEUE_STATUS_UPDATE",
	"LFG_LIST_ACTIVE_ENTRY_UPDATE",
	"LFG_UPDATE",
}

	function Q_C:ActivateScripts()
		--All scripts needed for above frames. sequenced in oder of frame creation.

		self.Notify:SetScript("OnClick", function(...)
			if self.notification.click then
				self.notification.click(...)
			end
		end)
		--self.Notify:SetScript("OnEnter", function(...) if self.Notify.tooltip then self.Notify.tooltip(...) end end)
		self.Notify:SetScript("OnLeave", function() GameTooltip:Hide() end)
		self.Notify:SetScript("OnMouseWheel", function(notify, delta) Q_C:NotifyMouseWheel(notify, delta) end)

		self.coordText:SetScript("OnEditFocusLost", function() self.coordText.hasFocus = nil self.coordText:HighlightText(0,0) end)
		self.coordText:SetScript("OnEditFocusGained", function() self.coordText.hasFocus = true self.coordText:SetText("") end)
		self.coordText:SetScript("OnEscapePressed", function() self.coordText:ClearFocus() end)
		self.coordText:SetScript("OnEnterPressed",	function() self:OnEnterPressed() end)

		self.coordText.button:SetScript("OnClick", function(_,btn)
			if btn == "RightButton" then
				self.Paste:Show()
				self.Paste:SetFocus(true)
			else
				self.coordText:SetFocus(true)
			end
		end)

		self.Paste:SetScript("OnEditFocusLost", function() self.Paste:HighlightText(0,0) self.Paste:Hide() end)
		self.Paste:SetScript("OnEditFocusGained", function() self.Paste:SetText("") self.Paste:Show() end)
		self.Paste:SetScript("OnEscapePressed", function() self.Paste:ClearFocus() end)
		self.Paste:SetScript("OnEnterPressed",	function() self:PasteWaypoints(self.Paste:GetText()) self.Paste:ClearFocus() end)
		self.Paste:SetScript("OnMouseDown", function() self.Paste:SetFocus(true) end)

		self.saveButton:SetScript("OnClick", function(_,btn)
			if btn == "LeftButton" then
				self:SavePrompt()
			else
				self:SetAnchor()
			end
		end)

		self.saveButton:SetScript("OnEnter", function()
			GameTooltip:SetOwner(self.saveButton, "ANCHOR_LEFT")
			GameTooltip:SetText("Left Click to Save Current Location")
			GameTooltip:AddLine("Right Click to Set Anchor Waypoint")
			GameTooltip:Show()
		end)
		self.saveButton:SetScript("OnLeave", function() GameTooltip:Hide() end)

		self.persistButton:SetScript("OnClick", function()
			Quest_CompassDB.persist = not Quest_CompassDB.persist
			GameTooltip:Hide()
			self.persistButton:GetScript("OnEnter")()
		end)
		self.persistButton:SetScript("OnEnter", function()
			GameTooltip:SetOwner(self.persistButton, "ANCHOR_RIGHT")
			GameTooltip:SetText("Store waypoint between sessions.")
			if Quest_CompassDB.persist == true then
				GameTooltip:AddLine("Enabled")
			else
				GameTooltip:AddLine("Disabled")
			end
			GameTooltip:Show()
		end)

		self.persistButton:SetScript("OnLeave", function() GameTooltip:Hide() end)

		self.pin:SetScript("OnEnter", function() self:PinOnEnter(self.pin) end)
		self.pin:SetScript("OnLeave", function() self:PinOnLeave(self.pin) end)
		self.pin:RegisterForClicks("AnyUp")
		self.pin:SetScript("OnClick", function(_,btn)
			self:PinOnClick(btn)
		end)
		self.pin:SetScript("OnMouseWheel", function(_,delta)
			self:ScrollWay(delta)
		end)

		self.optionMenuOverlay:SetScript("OnMouseDown",	function(_,btn)  Q_C:OnUpdate(true) self:OnMouseDown(btn) end)
		self.optionMenuOverlay:SetScript("OnMouseUp",	function() self:OnMouseUp()  Q_C:OnUpdate(true) end)
		self.optionMenuOverlay:SetScript("OnEnter",	function()  Q_C:OnUpdate(true) self:OnEnter() end)
		self.optionMenuOverlay:SetScript("OnMouseWheel",	function(_, delta) self:OnMouseWheel(delta) Q_C:OnUpdate(true) end)

		self.Notify:SetScript("OnMouseWheel", function(_,delta)
			self:ScrollNotify(delta)
		end)

		WorldMapFrame:HookScript("OnShow", function()
			self:CreateMapOverlay()
		end)

		self.frame:SetScript("OnUpdate", function() self:OnUpdate() end)
		self.frame:UnregisterAllEvents()
		
		
		for i, event in pairs(events) do
			self.frame:RegisterEvent(event)
		
		end
		
		self.frame:SetScript("OnEvent", function(_, event) self:OnUpdate(event) end)
		
	end

	local Test = false
	local sent = {}
	function Q_C:SendToChat(message)
		if (not sent[message]) or IsAltKeyDown() then
			if Test then
				print(message)
			else
				CTL:SendChatMessage("NORMAL", "Quest_Compass", message, "CHANNEL", "COMMON", 1)
			end
		end
	end
end

local notifyIndex = 0
do -- functions only called by scripts
	function Q_C:NotifyMouseWheel(notify, delta)
		notifyIndex = notifyIndex + delta
	end

	function Q_C:OnEnterPressed(text)
		local coordString = text or self.coordText:GetText()

		local t = string.gsub(coordString, ",", "")

		local x, y = string.split(" ", t)

		x, y = tonumber(x), tonumber(y)

		if not (x and y) then
			self.coordText:ClearFocus()
			return
		end

		x, y = x/100, y/100

		if x and y then
			self:SetWaypoint(x, y, self:GetViewedMapID())
			self.coordText:ClearFocus()
		end
	end

	function Q_C:PasteWaypoints(info)
		local _lines = {string.split("%/", info)}

		local slash =  (_lines[1] and string.trim(_lines[1]) ~= "") and {string.split(" ", string.trim(_lines[1]))} or (_lines[2] and string.trim(_lines[2]) ~= "") and {string.split(" ", string.trim(_lines[2]))}

		for i, text in pairs(_lines) do
			if text and string.trim(text) ~= "" then
				Cmd = string.trim(string.gsub(text, "^"..slash[1], ""))
				SlashCmdList["QUESTCOMPASSWAY"](Cmd)
			end
		end
	end

	function Q_C:SavePrompt()
		if not self.saveFrame then
			self.saveFrame = CreateFrame("Frame", addonName.."SaveFrame", UIParent, "BasicFrameTemplateWithInset")
			self.saveFrame:SetSize(500, 70)
			self.saveFrame:SetPoint("Center")
			self.saveFrame.TitleText:SetText("Quest Compass: Save Waypoint")

			self.saveFrame.title = CreateFrame("EditBox", addonName.."SaveFrameTitleEditBox", self.saveFrame, "InputBoxInstructionsTemplate")
			local titleString = self.saveFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			titleString:SetPoint("TopLeft",10,-25)
			titleString:SetText("Title:")
			self.saveFrame.title:SetPoint("Left", titleString, "Right", 8, 0)
			self.saveFrame.title:SetAutoFocus(false)
			self.saveFrame.title:SetSize(100, 30)

			local tooltipString = self.saveFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			tooltipString:SetText("Information:")
			tooltipString:SetPoint("Left", self.saveFrame.title, "Right", 8, 0)
			self.saveFrame.tooltip = CreateFrame("EditBox", addonName.."SaveFrameTitleEditBox", self.saveFrame, "InputBoxInstructionsTemplate")
			self.saveFrame.tooltip:SetPoint("Left", tooltipString, "Right", 8, 0)
			self.saveFrame.tooltip:SetPoint("Right", self.saveFrame,-5, 0)

			self.saveFrame.tooltip:SetSize(100, 30)
			self.saveFrame.tooltip:SetAutoFocus(false)

			self.saveFrame.title:SetScript("OnEnterPressed", function()
				if self.saveFrame.title:GetText() ~= "" then
					self.saveFrame.tooltip:SetFocus()
				end
			end)

			self.saveFrame.tooltip:SetScript("OnEnterPressed", function()
				self:GetSets().savedLocations = self:GetSets().savedLocations or {}
				local playerX, playerY, playerMapID = self:GetPlayerPosition()
				local inf = {name = self.saveFrame.title:GetText(), tooltip = self.saveFrame.tooltip:GetText(), position = {mapID = playerMapID, x = playerX , y = playerY}}
				tinsert(self:GetSets().savedLocations, inf)
				self.saveFrame.tooltip:ClearFocus()
				self.saveFrame:Hide()
				--save the waypoint
			end)

			local save = CreateFrame("Button", nil, self.saveFrame, "GameMenuButtonTemplate")
			save:SetPoint("BottomRight", -2, 6)
			save:SetText("Save Waypoint")
			save:SetScript("OnClick", function()
				self:GetSets().savedLocations = self:GetSets().savedLocations or {}
				local playerX, playerY, playerMapID = self:GetPlayerPosition()
				local worldID = HBD.GetWorldMapIDFromMapID[vID]
				local _pX, _pY = HBD:TranslateZoneCoordinates(_pX, _pY, playerMapID, worldID, true)
				local inf = {name = self.saveFrame.title:GetText(), tooltip = self.saveFrame.tooltip:GetText(), mapID = worldID, x = _pX , y = _pY}
				tinsert(self:GetSets().savedLocations, inf)
				self.saveFrame.tooltip:ClearFocus()
				self.saveFrame:Hide()
			end)

			local cancel = CreateFrame("Button", nil, self.saveFrame, "GameMenuButtonTemplate")
			cancel:SetPoint("Right", save, "Left", -2, 0)
			cancel:SetText("Cancel")
			cancel:SetWidth(75)
			cancel:SetScript("OnClick", function() self.saveFrame:Hide() end)
		end

		self.saveFrame.title:SetText("")
		self.saveFrame.tooltip:SetText("")
		self.saveFrame:Show()
		self.saveFrame.title:SetFocus(true)
	end

	local trigger

	function Q_C:triggerUpdate()
		--must travel 15yards before anchor will set.
		if trigger then
			local x, y, mapID, _, t = unpack(trigger)
			local px, py, pID = self:GetPlayerPosition()
			local range = self.range(pID, px, py, mapID, x, y)
			if range > 15 then
				self:SetWaypoint(x, y, mapID, _, "anchor")
				trigger = nil
			end
		end
	end

	function Q_C:SetAnchor()
		--need to make delay optional

		--if not trigger then
			local x , y, mapID = self:GetPlayerPosition()
			trigger = {x, y, mapID, nil, "anchor"}
		--end

		-- self.Anchored = true
		-- local x , y, mapID = self:GetPlayerPosition()
		-- self.waypointType = "anchor"
		-- self:SetWaypoint(x, y, mapID, nil, "anchor")
	end

	function Q_C:PinOnEnter(pin)

	end

	function Q_C:PinOnLeave(self)
		GameTooltip:Hide()
	end

	function Q_C:PinOnClick(btn)
		--local way = self.waypointType

		if self.navigation.click then
			local click = self.navigation.click
			if click then
				click(self, btn)
			end
			if 	MouseIsOver(self.pin) == true then
				--self:PinOnEnter(self.pin)
			end
		end
	end

	local wayStep = 1
	Q_C.wayStep = wayStep
	function Q_C:ScrollWay(delta)
		Q_C.wayStep = Q_C.wayStep - delta
		Q_C.delta = delta
	end

	local notifyStep = 1
	function Q_C:GetWayValue(kind, delta, index)
		kind = kind or self.CurrentNotifyInfo

		local wayInfo = kind and Q_C.travelQueues[kind]
		if not wayInfo then return end

		if wayInfo[2] and delta == 1 then
			local two = CopyTable(wayInfo[2])
			tremove(wayInfo, 2) -- remove from first place in list
			tinsert(wayInfo, two) -- add back to end of list.
		elseif wayInfo[2] then
			local last = CopyTable(wayInfo[#wayInfo])
			tremove(wayInfo, #wayInfo) -- remove from last place in list
			tinsert(wayInfo, 2, last) -- add back to start of list.
		end

		return (index and wayInfo) and wayInfo[index] or wayInfo
	end

	function Q_C:ScrollNotify(delta)
		Q_C:GetWayValue(_, delta)
	end

	local justify = {Left = 1, Right = -1}
	function Q_C:GetCoordButton(parent, justifyH, offset)
		local b = parent:CreateTexture(nil, "ARTWORK")
		b:SetTexture(Q_C.TexturePath.."coordBG")

		local text = parent:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
		text:SetJustifyH(justifyH)

		local tX, bX = (15 + (offset or 0)) * justify[justifyH], 15 * justify[justifyH]
		if justifyH == "Right" then
			b:SetTexCoord(1, 0, 0, 1)
			tX, bX = bX, tX
		end

		text:SetPoint("TopLeft", b, tX, 0)
		text:SetPoint("BottomRight", b, bX, 0)

		return text, b
	end

	function Q_C:CreateMapOverlay()
		self.coordsBar = CreateFrame("Frame", nil, WorldMapFrame, addonName.."NavBarTemplate")
		self.coordsBar:SetPoint("Right", WorldMapFrame.NavBar, "Right", 2, 0)
		self.coordsBar:SetPoint("Top", WorldMapFrame, "Bottom", 0, 0)
		self.coordsBar:SetPoint("Left", WorldMapFrame, "Left", 5, 0)
		self.coordsBar:SetScript("OnShow", nil)
		self.coordsBar:SetFrameStrata("HIGH")
		self.coordsBar:EnableMouse(true)
		self.coordsBar:SetHeight(40)

		local playerCoordsBG
		self.playerCoords, playerCoordsBG = Q_C:GetCoordButton(self.coordsBar, "Left", 10)
		local playerText, playerTextBG = Q_C:GetCoordButton(self.coordsBar, "Left")
		playerTextBG:SetPoint("BottomRight" , self.coordsBar, "BottomLeft", 100, 3)
		playerTextBG:SetPoint("TopLeft" , self.coordsBar, 3, -5)
		playerText:SetText("Player")
		playerCoordsBG:SetPoint("BottomRight" , playerTextBG, "BottomRight", 150, 0)
		playerCoordsBG:SetPoint("TopLeft" , playerTextBG, "TopRight", -20, 0)

		local cursorCoordsBG
		self.cursorCoords, cursorCoordsBG = Q_C:GetCoordButton(self.coordsBar, "Right", 10)
		local CursorText, CursorTextBG = Q_C:GetCoordButton(self.coordsBar, "Right")
		CursorTextBG:SetPoint("BottomLeft" , self.coordsBar, "BottomRight", -100, 3)
		CursorTextBG:SetPoint("TopRight" , self.coordsBar, -3, -5)
		CursorText:SetText("Cursor")
		cursorCoordsBG:SetPoint("BottomLeft" , CursorTextBG, "BottomLeft", -150, 0)
		cursorCoordsBG:SetPoint("TopRight" , CursorTextBG, "TopLeft", 20, 0)

		self.MapOverlayContainer = CreateFrame("ScrollFrame", nil, WorldMapFrame)
		self.MapOverlayContainer:SetAllPoints(WorldMapFrame)
		self.MapOverlayContainer:SetFrameStrata("HIGH")
		self.MapOverlayContainer:SetFrameLevel(700)

		self.MapOverlayFrame = CreateFrame("Frame", nil, self.MapOverlayContainer)
		self.MapOverlayFrame:SetAllPoints(self.MapOverlayContainer)
		self.MapOverlayFrame:SetFrameStrata("HIGH")
		self.MapOverlayFrame:SetFrameLevel(700)
		self.MapOverlayFrame:EnableMouse(true)
		self.MapOverlayFrame:SetScript("OnMouseUp", function()
			self:ApplyWayPoint()
		end)

		self.waypointsFrame = CreateFrame("Frame", nil, self.MapOverlayFrame)
		self.waypointsFrame:SetAllPoints(self.MapOverlayFrame)
		self.waypointsFrame:SetFrameStrata("HIGH")
		self.waypointsFrame:SetFrameLevel(700)

		self.waypointLocationHandler = CreateFrame("Frame", nil, UIParent)
		self.playerLocationHandler = CreateFrame("Frame", nil, UIParent)

		self.waypointLine = self.MapOverlayFrame:CreateLine(nil, "ARTWORK")
		self.waypointLine:SetColorTexture(1,1,1,1)
		self.waypointLine:SetThickness(9)

		self.waypointMarker = self.MapOverlayFrame:CreateTexture(nil, "HIGH")
		self.waypointMarker:SetTexture("Interface\\BUTTONS\\UI-StopButton")
		self.waypointMarker:SetSize(25, 25)

		function self:CreateMapOverlay()
			QuestScrollFrame:SetPoint("Bottom", self.coordsBar)
			--QuestScrollFrame.Background:SetPoint("Bottom", 0, 0)
			QuestMapFrame.VerticalSeparator:SetPoint("Bottom", self.coordsBar, 0, 0)
			QuestMapFrame:SetPoint("Bottom", self.coordsBar, 0, 0)
			QuestMapFrame.DetailsFrame:SetPoint("Bottom", self.coordsBar, 0, 0)
			WorldMapFrame.BorderFrame:SetPoint("Bottom", self.coordsBar, 0 ,-8)
		end
	end

	function Q_C:ShowMover()
		if MouseIsOver(self.coordText) then
			if IsShiftKeyDown() and self.optionMenuOverlay:IsVisible() ~= true then
				self.optionMenuOverlay:Show()
			elseif (IsShiftKeyDown() == nil) and (self.optionMenuOverlay:IsVisible() == true) then
				if self.mouseIsDown then
					self:OnMouseUp()
				end
				self.optionMenuOverlay:Hide()
			end
		end

		local canvas = self:GetCanvas()
		if self.MapOverlayFrame then
			self.MapOverlayFrame:SetFrameStrata("HIGH")
			self.MapOverlayFrame:SetFrameLevel(canvas:GetFrameLevel()+200)

		if MouseIsOver(canvas) and IsShiftKeyDown() then
			self.MapOverlayFrame:EnableMouse(true)
		else
			self.MapOverlayFrame:EnableMouse(false)
		end

		local vID = self:GetViewedMapID()
		local canvas = self:GetCanvas()

		if canvas ~= self.MapOverlayContainer.lastCanvas then
			local level = canvas:GetFrameLevel()
			self.MapOverlayContainer:SetFrameStrata("HIGH")
			self.MapOverlayContainer:SetFrameLevel(level + 50)
			self.MapOverlayContainer:SetParent(canvas)
			self.MapOverlayContainer:SetAllPoints(canvas)
			self.MapOverlayContainer.lastCanvas = canvas
		end
			Q_C:SetNodes(canvas, vID)
		end
	end

	function Q_C:OnEnter()
		if IsShiftKeyDown() then
			self.optionMenuOverlay:Show()
		else
			if self.optionMenuOverlay:IsShown() then
				self.optionMenuOverlay:Hide()
			end
		end
	end

	local GetRelPos = function(self)
		local width, height = GetScreenWidth()/self:GetScale(), GetScreenHeight()/self:GetScale()
		local x, y = self:GetCenter()
		local xOffset, yOffset
		local Hori = (x > width/2) and 'RIGHT' or 'LEFT'
		if Hori == 'RIGHT' then
			xOffset = self:GetRight() - width
		else
			xOffset = self:GetLeft()
		end
		local Vert = (y > height/2) and 'TOP' or 'BOTTOM'
		if Vert == 'TOP' then
			yOffset = self:GetTop() - height
		else
			yOffset = self:GetBottom()
		end
		return Vert, Hori, xOffset, yOffset
	end

	local function SelectProperSide(self)
		local width = GetScreenWidth()

		local Vert, Hori, xOffset, yOffset = GetRelPos(self)
		local vert, hori

		if Vert == "TOP" then
			vert = "BOTTOM"
		elseif Vert == "BOTTOM" then
			vert = "TOP"
		end

		if Hori == "LEFT" then
			hori = "RIGHT"
		elseif Hori == "RIGHT" then
			hori = "LEFT"
		end

		return Vert..Hori, vert..hori
	end
	Q_C.SelectProperSide = SelectProperSide

	function Q_C:OnMouseDown(btn)
		if btn == "RightButton" then
			ToggleDropDownMenu(1, 1, self:CreateDropDownMenu(), self.coordText:GetName(), 0, -5)

			DropDownList1:ClearAllPoints()
			local p1, p2 = SelectProperSide(self.coordText)
			DropDownList1:SetPoint(p1, self.coordText, p2)
			return
		end

		self.mouseIsDown = true
		self.frame:SetMovable(true)
		self.frame:StartMoving()
		self.coordText:ClearFocus()
	end

	function Q_C:OnMouseUp()
		self.frame:StopMovingOrSizing()
		self.coordText:ClearFocus()
		--self:SavePosition()
		self.frame:Stick()
		self.mouseIsDown = nil
	end

	function Q_C:OnMouseWheel(delta)
		self:Rescale(self:GetSets("scale", 1) + (delta/100))
	end

	function Q_C:StoreRareInfo(guid, x, y, mapID, name, classification)
		Q_C:GetSets().rareLocations = Q_C:GetSets().rareLocations or {}
		for i, b in pairs(Q_C:GetSets().rareLocations) do
			if b.guid == guid then
				return
			end
		end
		tinsert(Q_C:GetSets().rareLocations, {position = {x = x, y = y, mapID = mapID}, name = name, classification = classification, guid = guid})
	end

	local function IsSaved(guid)
		for i, b in pairs(Q_C:GetSets().rareLocations) do
			if b.guid == guid then
				return true
			end
		end
	end

	function Q_C:GetPlayerPosition()
		if IsInInstance() then
			local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceID, instanceGroupSize, LfgDungeonID = GetInstanceInfo()

			local info = Quest_CompassDB.instanceLocations and Quest_CompassDB.instanceLocations[name]
			if info then
				return info.position.x, info.position.y, info.position.mapID
			end
		end
		local playerMapID = self:GetPlayerMapID()
		if playerMapID then
			local px, py, pID = HBD:GetPlayerZonePosition(true)
			if not px then
				--player position doesn't seem to work on draenor or in dungeons
				playerX, playerY = 0, 0

				--if HBD.GetWorldMapIDFromMapID[playerMapID] ~= 572 then --not on draenor
					--placeholder for now, want to find a way to get position while in a dungeon
				--end
				return playerX, playerY
			else
				return px, py, pID
			end
		end
		return 0, 0
	end

	function Q_C:GetCursorPosition()
		local canvas = self:GetCanvas()
		if self:GetMap():IsShown() and canvas and MouseIsOver(canvas) then
			local curX, curY = GetCursorPosition()
			local scale = canvas:GetEffectiveScale()
			curX = ((curX / scale) - canvas:GetLeft()) / canvas:GetWidth()
			curY = (canvas:GetTop() - (curY / scale)) / canvas:GetHeight()
			return curX, curY
		end
		return 0, 0
	end
end




local lastEvent
do -- updates: these run constantly...be careful of too much at once
	function Q_C:OnUpdate(isEvent)
		--	self:PrintViewed()
		self:ShowMover()

		local playerX, playerY, playerMapID = self:GetPlayerPosition()


			self:UpdateAll(isEvent)

		
		self:UpdateTooltip(waypointX, waypointY, waypointID)

		if playerX and playerY then
			local cursorX, cursorY = self:GetCursorPosition()
			self:UpdateCoordText(playerX*100, playerY*100, cursorX*100, cursorY*100 )
		end
		

		if self.notification.tooltip and self.Notify:IsShown() and MouseIsOver(self.Notify) then
			local info = (type(self.notification.tooltip) == "function") and self.notification.tooltip(self)
			GameTooltip:SetOwner(self.Notify, "ANCHOR_RIGHT")

			if not info then return end
			
			GameTooltip:SetText(info[1])
			tremove(info, 1)
			
			for i, text in pairs(info) do
				GameTooltip:AddLine(text)
			end

			local p1, p2 = self.SelectProperSide(self.Notify)
			GameTooltip:ClearAllPoints()
			GameTooltip:SetPoint(p1, self.Notify, p2)
			GameTooltip:Show()
		end

--//..\\

		local t = GetTime()
		for l, r in pairs(Q_C.travelQueues.playerShared) do
			-- if math.abs(t - r[7][5]) > 45 then
				-- tremove(Q_C.travelQueues.playerShared, l)
			-- end
		end

		for l, r in pairs(Q_C.travelQueues.creatureYell) do
			--if math.abs(t - r[7][5]) > 45 then
			--	tremove(Q_C.travelQueues.creatureYell, l)
			--end
		end

		self:triggerUpdate()
	end


	local lastdestX, lastdestY, lastdestMapID, lastcompass

	local active = {}
	local translateActive = {}
	local lastX, lastY, lastID
	function Q_C:UpdateAll(isEvent)
			local pX, pY, pID = self:GetPlayerPosition()

	
		if not isEvent then
			if (not UnitIsGhost("player")) and ((lastX == pX) or (lastY == pY) or (lastID == pID)) then
			--	return
			end
		
		end
	lastX, lastY, lastID = pX, pY, pID
	
		if isEvent then
			local way = Q_C.way

			local info = way[Q_C.wayStep]

			local notify = info and way[Q_C.wayStep].notify or way[#way].notify

			wipe(active)

			-- local displayed
			-- local high, low = 0, #way
			for i, info in pairs(way) do
				local compass = info.compass
				if compass then
					local shouldShow, kind = compass.shouldShow and compass.shouldShow(self, pID)
					if shouldShow then
						info.index = i
						tinsert(active, info)
					end
				end
			end

			if Q_C.wayStep > #active then
				Q_C.wayStep = 1
			end
			if Q_C.wayStep < 1 then
				Q_C.wayStep = (#active)
			end
			if Q_C.snapTo then
				for i, b in pairs(active) do
					if b.index == Q_C.snapTo then
						Q_C.wayStep = i
						Q_C.snapTo = nil
					end
				end
			end

			if #active < 1 then
				tinsert(active, way[#way])
				Q_C.wayStep = 1
			end
		end
		local index, info = Q_C.wayStep, active[Q_C.wayStep]
		local i = index
		if info then
			local compass = info.compass
			if compass then
				local shouldShow, kind, questID = compass.shouldShow(self, pID)

				if compass.shouldShow and shouldShow then
					local destX, destY, destMapID = unpack(shouldShow)

					if isEvent then

						if self.navigation.tooltip ~= compass.tooltip then
							self.navigation.tooltip = compass.tooltip
						elseif self.navigation.tooltip == compass.tooltip then

						else
							self.navigation.tooltip = function() end
						end

						if self.navigation.click ~= compass.click then
							self.navigation.click = compass.click
						elseif self.navigation.click == compass.click then

						else
							self.navigation.click = function() end
						end

						if self.navigation.icon ~= compass.icon then
							self.navigation.icon = compass.icon
						elseif self.navigation.icon == compass.icon then

						else
							self.navigation.icon = function() end
						end

						if self.Get ~= info.get then
							self.Get = info.get
						elseif self.Get == info.get then

						else
							self.Get = function() end
						end

						self.Get = self.Get or function() end
					end
					
					self:SetCompassDisplay(pX, pY, pID, destX, destY, destMapID, compass,isEvent) --x, y, destX, destY, mapID, destMapID
					
					--Thank you Blizzard!
					
					if isEvent and isEvent ~= lastEvent then
					lastEvent = isEvent
						--print(kind)
						if kind == "quest" then
							C_SuperTrack.SetSuperTrackedUserWaypoint(false)
							C_Map.ClearUserWaypoint()
							C_SuperTrack.SetSuperTrackedQuestID(questID)
							
							
							if C_SuperTrack.IsSuperTrackingQuest() ~= true then
								C_Map.SetUserWaypoint(UiMapPoint.CreateFromCoordinates(destMapID, destX, destY))
								C_SuperTrack.SetSuperTrackedUserWaypoint(true);
							end
							
							self.super = true
						elseif kind == "corpse" and C_SuperTrack.IsSuperTrackingCorpse() == true then
							C_SuperTrack.SetSuperTrackedUserWaypoint(false)
							C_Map.ClearUserWaypoint()
							self.super = true
						elseif destMapID and destX and destY then
							C_Map.SetUserWaypoint(UiMapPoint.CreateFromCoordinates(destMapID, destX, destY))
							C_SuperTrack.SetSuperTrackedUserWaypoint(true);
							self.super = true
						else
							C_SuperTrack.SetSuperTrackedUserWaypoint(false);
							self.super = nil
						end
					end
				end
			else
				self:HideAll()
			end
			notify = info.notify or notify
		end

		if not isEvent then
			return
		end

		if notify then
			local shouldShow = notify.shouldShow and notify.shouldShow(self, pID)
			if shouldShow then
				local _, kind = notify.shouldShow(self, pID)
				self.CurrentNotifyInfo = kind

				if self.notification.tooltip ~= notify.tooltip then
					self.notification.tooltip = notify.tooltip
				elseif self.notification.tooltip == notify.tooltip then

				else
					self.notification.tooltip = function() end
				end

				if self.notification.click ~= notify.click then
					self.notification.click = notify.click
				elseif self.notification.click == notify.click then

				else
					self.notification.click = function() end
				end

				if self.notification.icon ~= notify.icon then
					self.notification.icon = notify.icon
				elseif self.notification.icon == notify.icon then

				else
					self.notification.icon = {0,0,0,1}
				end

			end
		end

		local pR, pG, pB, pA = self.Notify.normalTexture:GetVertexColor()

		local r, g, b, a -- type(self.notification.icon) == "function" and self.notification.icon() or type(self.notification.icon) == "table" and unpack(self.notification.icon)

		if type(self.notification.icon) == "function" then
			r, g, b, a = self.notification.icon()
		elseif type(self.notification.icon) == "table" then
			r, g, b, a = unpack(self.notification.icon)
		end

		if (pR ~= r) or (pG ~= g) or (pB ~= b) or (pA ~= a) then
			self.Notify.normalTexture:SetVertexColor(r, g, b, a)
		end

		if isEvent and way then
			for i, info in pairs(way) do
				local update = info.update
				if update then
					update(self)
				end
			end
		end
	end

	local last

	function Q_C:UpdateTooltip()
		if self.navigation.tooltip and self.pin:IsShown() and MouseIsOver(self.pin) and (not MouseIsOver(self.Notify)) then
			local info = self.navigation.tooltip(self)

			if (last == info) and GameTooltip:IsShown() then
				return
			else
				last = nil
			end
				if not info then
					return
				end

			last = info
			GameTooltip:SetOwner(self.pin, "ANCHOR_RIGHT")

			if info and type(info[1]) == "table" then
				GameTooltip:SetText(unpack(info[1]))
			elseif info then
				GameTooltip:SetText(info[1])
			end
			tremove(info, 1)
			for i, text in pairs(info) do
				if type(text) == "function" then
					text(self)
				elseif type(text) == "table" then
					GameTooltip:AddLine(unpack(text))
				else
					GameTooltip:AddLine(text)
				end
			end

			if GameTooltip.ItemTooltip and GameTooltip.ItemTooltip:IsShown() then
				local p1, p2 = self.SelectProperSide(self.pin)
				GameTooltip:ClearAllPoints()

				GameTooltip_CalculatePadding(GameTooltip)

				GameTooltip:SetPoint(p1, self.pin, p2)
			else
				local p1, p2 = self.SelectProperSide(self.pin)
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint(p1, self.pin, p2)
			end
			GameTooltip:Show()
		end
	end

	function Q_C:UpdateCoordText(playerX, playerY, cursorX, cursorY)
		if not self.coordText then
			return
		end

		local playerString, cursorString = "", ""
		if playerX ~= 0 and playerY ~= 0 then
			playerString = format("%.2f, %.2f", playerX or 0, playerY or 0)
		end
		if cursorX ~= 0 and cursorY ~= 0 then
			cursorString = format("%.2f, %.2f", cursorX or 0, cursorY or 0)
		end
		if not self.coordText.hasFocus then
			self.coordText:SetText(playerString)
		end
		if self.playerCoords then
			self.playerCoords:SetText(playerString)
			self.cursorCoords:SetText(cursorString)
		end
	end
end

do -- layout and appearance functions
	function Q_C:Layout()
		self:Rescale()
		--self:Reposition()
		self.coordText:ClearAllPoints()
		self.pin:ClearAllPoints()
		self.Notify:ClearAllPoints()
		self.distText:SetPoint("Center", self.Notify, "Center", 0, 2)

		local flip = self:GetSets().display.flip or nil

		local top    = flip and "Top"     or "Bottom"
		local bottom = flip and "Bottom"  or "Top"
		local insets = flip and {0,1,1,0} or {0,1,0,1}

		local one    = flip and 1         or 0
		local zero   = flip and 0         or 1

		local num    = flip and -1.5      or 1.5
		local three  = flip and -3        or 0

		self.coordText:SetPoint(top)
		self.pin:SetPoint("Center", self.frame, 0, num)
		self.Notify:SetPoint(bottom.."Left", self.coordText, top.."Left", 0, three)
		self.Notify:SetPoint(bottom.."Right", self.coordText, top.."Right", 0, three)


		self.Notify.Icon:SetTexCoord(0, 1, one, zero)
		self.Notify.normalTexture:SetTexCoord(0, 1, one, zero)
		self.Notify.pushedTexture:SetTexCoord(0, 1, one, zero)
		self.Notify.Highlight:SetTexCoord(0, 1, one, zero)
	end

	function Q_C:Rescale(value)
		if not self then return end
		if value then
			self:GetSets().scale = value
		end
		local scale = self:GetSets("scale", value or 1)
		self.frame:SetScale(scale)
		if value then
			self:Reposition()
		end
	end

	function Q_C:Reposition()
		local x, y = self:GetSets().x, self:GetSets().y
		self:ClearAllPoints()
		local scale = self.frame:GetEffectiveScale()
		self.frame:SetPoint("Center", UIParent, "Center", x/scale, y/scale)
	end

	function Q_C:SavePosition()
		local scale = self.frame:GetEffectiveScale()
		local x, y = self.frame:GetCenter()
		local pX, pY = UIParent:GetCenter()

		local uScale = UIParent:GetEffectiveScale()

		local snapDist = 20

		x, y = x*scale, y*scale

		local uiX = (x >= ((pX * uScale) - snapDist) and x <= ((pX * uScale) + snapDist)) and uiX or pX * uScale
		local uiY = (y >= ((pY * uScale) - snapDist) and y <= ((pY * uScale) + snapDist)) and uiY or pY * uScale

		x, y = x - uiX, y - uiY

		self:GetSets().x, self:GetSets().y = x, y
		self:Reposition()
	end
end

do -- Waypoints
	if not TomTom then
		--if TomTom is not installed/enabled, steal its waypoint sharing!!!
		--sure, call me a thief. /shrug
		TomTom = {}

		function TomTom:AddWaypoint(uiMapID, x, y, options)
			options = options or {}

			-- Default values
			local zoneName = HBD:GetLocalizedMap(uiMapID)

			self.WaypointTitle = options.title
			self.waypointType = "suggestions"

			if x > 1 then
				x = x/100
				y = y/100
			end

			local skip

			for q, z in pairs(Quest_CompassAltDB) do
				if (math.floor(z.position.x*100) == math.floor(x*100)) and
				 (math.floor(z.position.y*100) == math.floor(y*100)) and
				 (z.name == options.title) and (z.position.mapID == uiMapID) then
					skip = true
				end
			end
			if not skip then
				local saving = {
					position = {
						x = x,
						y = y,
						mapID = uiMapID,
					},
					name = options.title,
				}

				tinsert(Quest_CompassAltDB, saving)
			end

			return uiMapID
		end
	end

	function Q_C:ApplyWayPoint()
		local canvas = self:GetCanvas()
		local cursorX, cursorY = 0, 0

		if self:GetMap():IsShown() and canvas and MouseIsOver(canvas) then
			local curX, curY = GetCursorPosition()
			local scale = canvas:GetEffectiveScale()
			cursorX = ((curX / scale) - canvas:GetLeft()) / canvas:GetWidth()
			cursorY = (canvas:GetTop() - (curY / scale)) / canvas:GetHeight()
		end

		self.waypointType = "userSelected"
		self.WaypointTitle = "Waypoint"
		self:SetWaypoint(cursorX, cursorY, self:GetViewedMapID())
	end

	local index = {
		corpse = 1,
		anchor = 2,
		target = 3,
		playerShared = 4,
		portal = 5,
		creatureYell = 6,
		flightMaster = 7,
		rare = 8,
		userSelected = 9,
		suggestions = 10,
		worldQuest = 10,
		quest = 12,
		basic = 13,
	}

	local last

	function Q_C:SetWaypoint(x, y, mapID, t, extra, node,message)
		mapID = tonumber(mapID)
		if mapID and (mapID > -2147483649) and (mapID < 2147483648) then
			local info = C_Map.GetMapInfo(mapID or self:GetViewedMapID())

			if extra and last ~= extra then
				if index[extra] then
					Q_C.snapTo = index[extra]
					Q_C.lastQuestID = nil
				end
			end
			last = extra

			if extra == "quest" then
				--Q_C.travelQueues.quest = Q_C.travelQueues.quest or {}
				--tinsert(Q_C.travelQueues.creatureYell, {info and info.name, x , y, mapID, t, node, node.guid})
			elseif extra == "creatureYell" then
				Q_C.travelQueues.creatureYell = Q_C.travelQueues.creatureYell or {}
				tinsert(Q_C.travelQueues.creatureYell, {info and info.name, x , y, mapID, t, extra, node, message})
			elseif extra == "target" then
				tinsert(Q_C.travelQueues.target, 1, {info and info.name, x , y, mapID, t, node, node.guid})
			elseif extra == "playerShared" then
				tinsert(Q_C.travelQueues.playerShared, {info and info.name, x , y, mapID, t, extra, node})
			elseif extra == "rare" then
			--print(info and info.name, x , y, mapID, t, extra, node.info.classification)
				tinsert(Q_C.travelQueues.rare, 1, {info and info.name, x , y, mapID, t, extra, node.info.classification})
			elseif extra == "suggestions" then
				tinsert(Q_C.travelQueues.suggestions, 1, {info and info.name, x , y, mapID, t})
			elseif extra == "portal" then
				tinsert(Q_C.travelQueues.portal, 1, {info and info.name, x , y, mapID, t})
			elseif extra == "anchor" then
				tinsert(Q_C.travelQueues.anchors, 1, {info and info.name, x , y, mapID})
			elseif extra == "flightMaster" then
				tinsert(Q_C.travelQueues.flightMaster, 1, {info and info.name, x, y, mapID or self:GetViewedMapID(), t})
			else
				tinsert(Q_C.travelQueues.userSelected, 1, {info and info.name, x, y, mapID or self:GetViewedMapID()})
				Q_C.snapTo = index.userSelected
				Q_C.lastQuestID = nil
			end
		end
	end

	function Q_C:RestorePersistent()
		-- if self.sets.persist and self.sets.persistWaypoint then
			-- local point = self.sets.persistWaypoint
			-- self.waypointType = point.waypointType
			-- self.classification = point.classification
			-- self.WaypointTitle = point.WaypointTitle
			-- self.waypointTooltip = point.waypointTooltip
			-- self:SetWaypoint(point.x, point.y, point.mapID)
		-- end

	end
end

do --Portal Recommendations
	local function digForZone(mapID)
		local parent = HBD.GetParentMapID[mapID]

		if parent then
			local mapInfo = C_Map.GetMapInfo(parent)
			if (mapInfo.mapType == Enum.UIMapType.Continent) or mapInfo.mapType == Enum.UIMapType.World then
				return mapID
			else
				return digForZone(parent)
			end
		else
			return mapID
		end
	end

	Q_C.digForZone = digForZone

	local function digForContinent(mapID)
		local parent = HBD.GetParentMapID[mapID]

		if parent then
			local mapInfo = C_Map.GetMapInfo(parent)
			if  mapInfo.mapType == Enum.UIMapType.World then
				return mapID
			else
				return digForContinent(parent)
			end
		else
			return mapID
		end
	end

	local continentPortals = {
		[13] = {
			[245] = {
				["y"] = 0.79655785617416,
				["x"] = 0.56321089764031,
				["destination"] = {
					["id"] = 85,
					["x"] = 0.4764620014373,
					["y"] = 0.39140493181188,
				},
				["tooltip"] = "Portal to Orgrimmar",
			},
			[110] = {
				{
					["y"] = 0.18680549839493,
					["x"] = 0.58536847841333,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},

			[90] = {
				{
					["y"] = 0.17035557881295,
					["x"] = 0.85221381072501,
					["destination"] = {
						["id"] = 100,
						["x"] = 0.89162719838972,
						["y"] = 0.49560798548094,
					},
					["tooltip"] = "Portal to HellFire Peninsula, Outland",
				}, -- [1]
			},
			[241] = {
				{
					["y"] = 0.53492878438869,
					["x"] = 0.73593721339996,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.50135588842975,
						["y"] = 0.37890044201114,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
		},
		[1355] = {
			[1355] = {
				{
					["y"] = 0.62772743942331,
					["x"] = 0.4727348470357,
					["destination"] = {
						["id"] = 1163,
						["x"] = 0.70333319769965,
						["y"] = 0.68583374023438,
					},
					["tooltip"] = "Portal to Zuldazar",
				}, -- [1]
			},
		},
		[946] = {
			[100] = {
				{
					["y"] = 0.49464972688152,
					["x"] = 0.89232424936608,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
			[624] = {
				{
					["y"] = 0.51649872833174,
					["x"] = 0.60789520263672,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
			[111] = {
				{
					["y"] = 0.48866477166798,
					["x"] = 0.56823639354067,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
		},
		[948] = {
			[207] = {
				{
					["y"] = 0.53103871944398,
					["x"] = 0.50934902393179,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
		},
		[875] = {
			[1163] = {
				{
					["y"] = 0.85333353678385,
					["x"] = 0.62999979654948,
					["destination"] = {
						["id"] = 1355,
						["x"] = 0.47193758468472,
						["y"] = 0.6262505365728,
					},
					["tooltip"] = "Portal to Nazjatar",
				}, -- [1]
				{
					["y"] = 0.85416666666667,
					["x"] = 0.73611111111111,
					["destination"] = {
						["id"] = 81,
						["x"] = 0.41537530710609,
						["y"] = 0.44943381928406,
					},
					["tooltip"] = "Portal to Silithus",
				}, -- [2]
				{
					["y"] = 0.775,
					["x"] = 0.73666653103299,
					["destination"] = {
						["id"] = 88,
						["x"] = 0.22214766105373,
						["y"] = 0.1687186758094,
					},
					["tooltip"] = "Portal to Thunder Bluff",
				}, -- [3]
				{
					["y"] = 0.70166727701823,
					["x"] = 0.73611111111111,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [4]
				{
					["y"] = 0.62250061035156,
					["x"] = 0.73944430881076,
					["destination"] = {
						["id"] = 110,
						["x"] = 0.58264465141624,
						["y"] = 0.19238329952114,
					},
					["tooltip"] = "Portal to Silvermoon City",
				}, -- [5]
			},
			[862] = {
				{
					x = 0.58454345381237,
					y = 0.62980799772618,
					destination = {
						id = 896,
						x = 0.20607681440632,
						y = 0.43693684881455,
					},
					tooltip = "Banshee's Wail to Drustvar",
				},
				{
					x = 0.58454345381237,
					y = 0.62980799772618,
					destination = {
						id = 942,
						x = 0.51058783027949,
						y = 0.31070487254826,
					},
					tooltip = "Banshee's Wail to Stormsong Valley",
				},
				{
					x = 0.58454345381237,
					y = 0.62980799772618,
					destination = {
						id = 895,
						x = 0.88203434638142,
						y = 0.51159193712012,
					},
					tooltip = "Banshee's Wail to Tiragarde Sound",
				},
			},
			[1165] = {
				{
					x = 0.41825685012782,
					y = 0.87604148782861,
					destination = {
						id = 1462,
						x = 0.75731763870321,
						y = 0.21320900152281,
					},
					tooltip = "Greasy Eel to Mechagon",
				},
			},
		},
		[876] = {
			[896] = {
				{
					x = 0.20602456086601,
					y = 0.43340743614875,
					destination = {
						id = 862,
						x = 0.58402363790205,
						y = 0.62495129448905,
					},
					tooltip = "Banshee's Wail to Zuldazar",
				},
			},
			[895] = {
				{
					x = 0.87841985717018,
					y = 0.51185500686031,
					destination = {
						id = 862,
						x = 0.58402363790205,
						y = 0.62495129448905,
					},
					tooltip = "Banshee's Wail to Zuldazar",
				},
			},
			[942] = {
				{
					x = 0.51955369973581,
					y = 0.24462557170856,
					destination = {
						id = 862,
						x = 0.58402363790205,
						y = 0.62495129448905,
					},
					tooltip = "Banshee's Wail to Zuldazar",
				},
			},
			[1462] = {
				{
					x = 0.75500752005348,
					y = 0.22668496550168,
					destination = {
						id = 1165,
						x = 0.41751074550264,
						y = 0.87433673840999,
					},
					tooltip = "Greasy Eel to Zuldazar",
				},
			},
		},
		[113] = {
			[125] = {
				{
					["y"] = 0.25476156859977,
					["x"] = 0.55301410416057,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},

		},
		[619] = {
			[627] = {
				{
					["y"] = 0.2395178315967,
					["x"] = 0.55211635002278,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
			[630] = {
				{
					["y"] = 0.41296159312871,
					["x"] = 0.46663798048915,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
		},
		[424] = {
			[371] = {
				{
					["y"] = 0.14021435795428,
					["x"] = 0.28511899099554,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
			[1530] = {
				{
					x = 0.63762695780944,
					y = 0.098700038554703,
					destination = {
						id = 85,
						x = 0.57103620194035,
						y = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
		},
		[12] = {
			[85] = {
				{
					["y"] = 0.91711090955257,
					["x"] = 0.56224005120374,
					["destination"] = {
						["id"] = 125,
						["x"] = 0.55915854190679,
						["y"] = 0.467831174332,
					},
					["tooltip"] = "Portal to Dalaran, Crystalsong Forest",
				}, -- [1]
				{
					["y"] = 0.9226301536912,
					["x"] = 0.57460081297161,
					["destination"] = {
						["id"] = 371,
						["x"] = 0.2855772268634,
						["y"] = 0.13982759669818,
					},
					["tooltip"] = "Portal to Jade Forest",
				}, -- [2]
				{
					["y"] = 0.91348896151172,
					["x"] = 0.5858117364355,
					["destination"] = {
						["id"] = 1163,
						["x"] = 0.70333319769965,
						["y"] = 0.68583374023438,
					},
					["tooltip"] = "Portal of Zuldazar",
				}, -- [3]
				{
					["y"] = 0.89537901076584,
					["x"] = 0.58885869565217,
					["destination"] = {
						["id"] = 630,
						["x"] = 0.46817071748393,
						["y"] = 0.41362162170844,
					},
					["tooltip"] = "Portal to Azsuna",
				}, -- [4]
				{
					["y"] = 0.87839019431731,
					["x"] = 0.58322460025153,
					["destination"] = {
						["id"] = 624,
						["x"] = 0.56951995849609,
						["y"] = 0.4973742659755,
					},
					["tooltip"] = "Portal to Warspear, Ashran",
				}, -- [5]
				{
					["y"] = 0.87347457309522,
					["x"] = 0.5712661134567,
					["destination"] = {
						["id"] = 111,
						["x"] = 0.53003551136364,
						["y"] = 0.49210974806648,
					},
					["tooltip"] = "Portal to Shattrath",
				}, -- [6]
				{
					["y"] = 0.88218457597122,
					["x"] = 0.55982527847646,
					["destination"] = {
						["id"] = 110,
						["x"] = 0.58264465141624,
						["y"] = 0.19238329952114,
					},
					["tooltip"] = "Portal to Silvermoon City",
				}, -- [7]
				{
					["y"] = 0.92538982839592,
					["x"] = 0.56413717211642,
					["destination"] = {
						["id"] = 74,
						["x"] = 0.5460492553501,
						["y"] = 0.28302714070312,
					},
					["tooltip"] = "Portal to Cavern of Times",
				}, -- [8]
				{
					["y"] = 0.55594681212568,
					["x"] = 0.50745008758534,
					["destination"] = {
						["id"] = 90,
						["x"] = 0.84585550612658,
						["y"] = 0.16332547124834,
					},
					["tooltip"] = "Portal to Undercity",
				}, -- [9]

				{
					["y"] = 0.38545449836978,
					["x"] = 0.48876521514553,
					tooltip = "Portal to Uldum",
					["destination"] = {
						["id"] = 1527,
						["x"] = 0.54895175722124,
						["y"] = 0.34247656465072,
					},
				}, -- [13]
				{
					["y"] = 0.38545449836978,
					["x"] = 0.48876521514553,
					["destination"] = {
						["id"] = 249,
						["x"] = 0.54895175722124,
						["y"] = 0.34247656465072,
					},
					["tooltip"] = "Portal to Uldum",
				}, -- [14]
				{
					["y"] = 0.36544725152611,
					["x"] = 0.49221473005749,
					["destination"] = {
						["id"] = 204,
						["x"] = 0.51370135161043,
						["y"] = 0.60947867366724,
					},
					["tooltip"] = "Portal to Vashj'ir",
				}, -- [15]
				{
					["y"] = 0.36311887139568,
					["x"] = 0.50808255479698,
					["destination"] = {
						["id"] = 207,
						["x"] = 0.5059176514649,
						["y"] = 0.52942107099127,
					},
					["tooltip"] = "Portal to Deepholm",
				}, -- [16]
				{
					["y"] = 0.38278114574241,
					["x"] = 0.5111870620733,
					["destination"] = {
						["id"] = 198,
						["x"] = 0.63486551497068,
						["y"] = 0.2337236134106,
					},
					["tooltip"] = "Portal to Hyjal",
				}, -- [17]
				{
					["y"] = 0.39450947374272,
					["x"] = 0.50227581521739,
					["destination"] = {
						["id"] = 241,
						["x"] = 0.73631666027462,
						["y"] = 0.53393293284393,
					},
					["tooltip"] = "Portal to Twilight Highlands",
				}, -- [18]
				{
					["y"] = 0.39287103860618,
					["x"] = 0.47387486525332,
					["destination"] = {
						["id"] = 245,
						["x"] = 0.44771604383759,
						["y"] = 0.62372974957755,
					},
					["tooltip"] = "Portal to Tol Barad",
				}, -- [19]
			},
			[198] = {
				{
					["y"] = 0.24446088576159,
					["x"] = 0.63479490345888,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},

			[81] = {
				{
					["y"] = 0.4520574119515,
					["x"] = 0.41601595868837,
					["destination"] = {
						["id"] = 1163,
						["x"] = 0.68277757432726,
						["y"] = 0.64583333333333,
					},
					["tooltip"] = "Portal to Zuldazar",
				}, -- [1]
				{
					["y"] = 0.44514740906467,
					["x"] = 0.43181062653252,
					["destination"] = {
						id = 1473,
						x = 0.50225006103516,
						y = 0.35937156668864,
					},
					["tooltip"] = "Titan Translocator",
				}, -- [2]
			},
			[74] = {
				{
					["y"] = 0.267044943523,
					["x"] = 0.58198599758397,
					["destination"] = {
						["id"] = 85,
						["x"] = 0.57103620194035,
						["y"] = 0.89813858019974,
					},
					["tooltip"] = "Portal to Orgrimmar",
				}, -- [1]
			},
			[1473] = {
				{
					["x"] = 0.50100006103516,
					["y"] = 0.30574598989233,
					["destination"] = {
						id = 81,
						x = 0.41537530710609,
						y = 0.44943381928406,
					},
					["tooltip"] = "Titan Translocator",
				}, -- [1]
			}
		},
	}
	Q_C.continentPortals = continentPortals

	Q_C.AllPortals = {}

	for world, continents in pairs(continentPortals) do
		for zone, portals in pairs(continents) do
			--print(#portals)
			if zone then
				Q_C.AllPortals[zone] = {}
				if portals.tooltip then
					portals.position = {x = portals.x, y = portals.y, mapID = zone}
					portals.name = portals.tooltip
					portals.tooltip = "   "..format("%.2f, %.2f", portals.x * 100 or 0, portals.y * 100 or 0)
					tinsert(Q_C.AllPortals[zone], portals)
				else
					for index, portal in pairs(portals) do
						if type(portal) == "table" then
							portal.position = {x = portal.x, y = portal.y, mapID = zone}
							portal.name = portal.tooltip

							portal.tooltip = "   "..format("%.2f, %.2f", portal.x * 100 or 0, portal.y * 100 or 0)
							tinsert(Q_C.AllPortals[zone], portal)

						end
					end
				end
			end
		end
	end

	-- for cont, zones in pairs(continentPortals) do
		-- for index, portals in pairs(zones) do
			-- local portals = CopyTable(portals)

			-- for l, a in pairs(portals) do
			-- print(a)
			-- if a then
				-- a.position = {x = a.x, y = a.y, mapID = index}
				-- a.name = a.tooltip
				-- end
			-- end

			-- Q_C.AllPortals[index] = portals
		-- end
	-- end
	-- print(#Q_C.AllPortals)

	local portals = {
		Horde = continentPortals,
		Alliance = {},
	}

	local continentPortals = portals[UnitFactionGroup("player")]

	local paths = {}

	--zone To zone to zone...
	local function reTry(id, path)
		for continentID, contPortals in pairs(continentPortals) do
			for zoneID, zonePortals in pairs(contPortals) do
				if zoneID == id then
					for _, portal in pairs(zonePortals) do
						if type(portal) == "table" and portal.destination and portal.destination.id then
							local path = CopyTable(path)

							if tContains(path, portal.destination.id) ~= true then

								tinsert(path, portal.destination.id)

								tinsert(paths, path)

								reTry(portal.destination.id, path)
							end
						end
					end
				end
			end
		end
	end

	--convert portals into "portal paths", so we can easily check if a portal will take us to our destination.
	for continentID, contPortals in pairs(continentPortals) do
		for zoneID, zonePortals in pairs(contPortals) do
			for _, portal in pairs(zonePortals) do
				if type(portal) == "table" and  portal.destination and portal.destination.id then
					local path = {}
					tinsert(path, zoneID)
					tinsert(paths, path)

					path = CopyTable(path)
					tinsert(path, portal.destination.id)

					tinsert(paths, path)

					reTry(portal.destination.id, path)
				end
			end
		end
	end

	for i, b in pairs(paths) do
		if #b < 2 then
			tremove(paths, i)
		end
	end

	local function GetPortalFromMapID(id, nextID)
		local nextCont = digForContinent(nextID)
		for continentID, contPortals in pairs(continentPortals) do
			for zoneID, zonePortals in pairs(contPortals) do
				if zoneID == id then
					for zonePortal, portal in pairs(zonePortals) do
						portal.id = zoneID
						local portaldestCont = portal.destination and digForContinent(portal.destination.id)

						if (portal.destination and (portal.destination.id == nextID)) then
							return portal
						end
					end
				end
			end
		end
	end

	local lastOrigon, lastDestination
	local lastOrigonContinent, lastDestinationContinent
	local lastPortal, lastDestinationPortal
	function Q_C:GetPossibleWaypoint(destination, destx, desty)

		if self.sets.portals.enable ~= true then
			return destx, desty, destination
		end

		local origon = Q_C:GetPlayerMapID()

		if (not origon) or origon == destination then
			return destx, desty, destination
		end

		if origon == lastOrigon and destination == lastDestination and lastPortal then
			return unpack(lastPortal)
		end

		local origonConinent = digForContinent(origon)
		local destinationConinent = digForContinent(destination)

		lastOrigon, lastDestination = origon, destination

		local _pX, _pY, playerMapID = HBD:GetPlayerZonePosition(true)
		local selected, destinationPortal
		local options = {}
		local backup = {}
		do --any portals?
			for i, possiblePath in pairs(paths) do
				local possibleOrigon = possiblePath[1]
				local possibleDestination = possiblePath[#possiblePath]

				local possibleOrigonCont = digForContinent(possibleOrigon)
				local possibleDestinationCont = digForContinent(possibleDestination)

				if ((destination == possibleDestination)
				or (destination == possibleDestinationCont)) and
				   ((origon == possibleOrigon)
				or (origon == possibleOrigonCont)) then
					tinsert(options , possiblePath) -- direct portals
				end

				if ((destination == possibleDestination)
				or ( destination == possibleDestinationCont)
				or ( possibleDestinationCont == destinationConinent)) and

				   ((origon == possibleOrigon)
				or (origon == possibleOrigonCont)
				or (possibleOrigonCont == origonConinent)) then
					tinsert(backup , possiblePath) -- indirect portals
				end

			end

			local possible = {x = _pX, y = _pY, id = playerMapID}

			local dx, dy, did = Questra:TranlsateCoords(destx, desty, destination, playerMapID)

			local waypointDistance = ((playerMapID == did) and (destinationConinent ~= origonConinent)) and Questra:GetDistanceBetween(dx, dy, did, _pX, _pY, playerMapID) or math.huge

			local lastDist = (waypointDistance > 0 ) and waypointDistance or math.huge

			local index = 0

			local portals = {}

			for _, path  in pairs(options) do
				local pos = {}

				for i, id in pairs(path) do
					local nextID = path[i+1]
					local portal = nextID and GetPortalFromMapID(id, nextID)
					if portal then
						tinsert(pos, portal)
						tinsert(pos, portal.destination)
					end
				end
				tinsert(pos, 1, possible)
				tinsert(pos, {x = destx, y = desty, id = destination})
				tinsert(portals, pos)
			end

			if #portals<1 then
				for _, path  in pairs(backup) do
					local pos = {}
					for i, id in pairs(path) do
						local nextID = path[i+1]
						local portal = nextID and GetPortalFromMapID(id, nextID)
						if portal then
							tinsert(pos, portal)
							tinsert(pos, portal.destination)
						end
					end
					tinsert(pos, 1, possible)
					tinsert(pos, {x = destx, y = desty, id = destination})
					tinsert(portals, pos)
				end
			end

			for _, pos in pairs(portals) do
				local dist = 0
				local index = 1

				local qty = math.huge

				for i = 1, #pos/2 do
					local location = pos[index]
					local nextLocation = pos[index+1]
					index = index + 2

					local x, y, id =  location.x, location.y, location.id

					local nextLocation = pos[i+1]
					if nextLocation then
						local nextX, nextY, nextId =  Questra:TranlsateCoords(nextLocation.x, nextLocation.y, nextLocation.id, id)
						if (id == nextId) then
							local d = Questra:GetDistanceBetween(x, y, id, nextX, nextY, nextId)
							if d > waypointDistance then
								dist = math.huge
							end
							dist = dist + d
						end
					end
				end

				if (dist > 0 and dist < lastDist) then
					lastDist = dist
					selected = pos[2]
					destinationPortal = pos[#pos]
				end
			end

			if selected then
				local x, y, id = Questra:TranlsateCoords(selected.x, selected.y, selected.id, playerMapID)
				lastPortal = {selected.x, selected.y, selected.id, selected.tooltip}
				lastDestinationPortal = {destinationPortal.x, destinationPortal.y, destinationPortal.id, selected.tooltip}
				return x, y, id, selected.tooltip
			else
				lastPortal = nil
				lastDestinationPortal = nil
			end
		end

	end
end

do -- events, kinda
	local matches = {
			"%d%d.%d%d.%s%d%d",			--5
			"%d%d.%s%d%d.%d%d",			--6
			"%d+.+%d+",					--1 --catches decimals
			"%d%d.%s%d%d",				--2
			"%d%d.%a%d%d",				--3
			"%d%d.%d%%d",				--4
		"%d%d.%d%d.%s%d%d.%d%d",
		"%d%d.%s%a%s%d%d",
		"%d%d.%s%d%d",
		"%d%d.%a%d%d",
		"%d%d.%d%d",
		"%d%d%s%d%d",
		"%d%d.%d%d.%s%d%d",
		"%d%d.%s%d%d.%d%d",
		"%d+.%s%d+.%d+",
		"%d+.%d+%s%d+.%d+",
	}

	local function matchMade(message)
		message = string.gsub(message, "%a+", " ") --convert all letters to spaces
		message = string.gsub(message, ",", " ")
		message = string.gsub(message, [[\]], " ")
		message = string.gsub(message, [[:]], " ")
		message = string.gsub(message, [[;]], " ")
		message = string.gsub(message, [[']], " ")
		message = string.gsub(message, [["]], " ")
		message = string.gsub(message, [[?]], " ")
		message = string.gsub(message, [[!]], " ")
		message = string.gsub(message, [[*]], " ")
		message = string.gsub(message, [[/]], " ")
		message = string.gsub(message, "-", " ")
		message = string.gsub(message, "%s+", " ")
		for i, b in pairs(matches) do
			local co = strmatch(message, b)
			if co then
				local x, y = string.split(" ", co)
				if tonumber(x) and tonumber(y) then
					return x, y
				end
			end
		end
	end

	local sharedThisSession = {}

	local lastMessage

	local mult = 5614300 --* (math.abs(zRight - zLeft)/.0223559218274385)

	local mults = {
		[88] = .017300009990335,
		[85] = .0186868484984595,
		[627] = .012915138537737,
		[7] = .016895413395758,
		[66] = .012451072035246,
		[1462] = 0.019328068697401,
	}

	local curID

	-- local edit = CreateFrame("EditBox", addonName.."EditBoxHelper", UIParent, "InputBoxTemplate")
	-- Questra.editor = edit
	-- edit:SetFontObject("GameFontHighlight")
	-- edit:SetSize(200, 100)
	-- edit:SetPoint("Center", 30, 0)
	-- edit:SetAutoFocus(false)
	-- edit:SetMultiLine("true")
	-- edit:SetScript("OnMouseDown", function(n, btn) PrintViewed(btn) end)
	-- edit:SetText(.017300009990335)
	-- edit:SetScript("OnEnterPressed", function()
		-- Quest_CompassDB.mults[curID] = tonumber(edit:GetText())
		-- edit:ClearFocus()
	-- end)

	Q_C.range = function(oID, oX, oY, dID, dX, dY, is)
		local owID = HBD.GetWorldMapIDFromMapID[oID]
		local dwID = HBD.GetWorldMapIDFromMapID[dID]
		
		local distance = (is and Q_C.super) and C_Navigation.GetDistance()
		
		if distance and distance > 0 then
			return distance
		end
		
		if owID == dwID then --origin and destination on same world
		
			local owX, owY = Questra:TranslateCoords(oX, oY, oID, owID, true)
			local dwX, dwY = Questra:TranslateCoords(dX, dY, dID, dwID, true)

			if not (owX or owY or dw or dwY) then return math.huge end

			Quest_CompassDB.mults = Quest_CompassDB.mults or mults

			local mult = Quest_CompassDB.mults[oID] or .017300009990335

			local test = ((math.sqrt(math.pow(owX - dwX, 2) + math.pow(owY - dwY, 2)) * 100000) / 100) / (45)

			curID = oID
			local val = tonumber(format("%.2f", ((math.sqrt(math.pow(owX - dwX, 2) + math.pow(owY - dwY, 2)) * 100000) / 100) / (mult)))

			if edit and not edit:HasFocus() then
				edit:SetText(test)
			end

			return val
		end
		return math.huge
	end
	local lastMessage, lastPlayer, lastServer, lastCoords, lastCreature
	local player = UnitName("player")

	function Q_C:FilterForCoords(chatFrame, message, playerServer, channel, k)
	if k == "2. Trade - City" then
		return
	end
		if message ~= lastMessage then
			lastMessage = message
			local name, server = string.split("-", playerServer)
			if (player ~= name) then
				local x, y = matchMade(message)
				if x and y then
					x, y = tonumber(x)/100, tonumber(y)/100
					if (x and y) and (x <= 1) and (y <= 1)then
						if not sharedThisSession[message] then
							sharedThisSession[message] = true
							lastMessage = message
							local _, _, pID = Q_C:GetPlayerPosition()
							Q_C:SetWaypoint(x, y, pID, name, "playerShared", message)
						end
					end
				end
			end
		end
	end

	local rareTimers = {}
	Q_C.rareTimers = rareTimers
	local rareTimed = {}
	Q_C.rareTimed = rareTimed
	Q_C.RareYellFilter = function(chatFrame, event, message, name)
		local info
		for i, inf in pairs(Quest_CompassDB.rareLocations) do
			if inf and inf.name == name then
				info = inf
				break
			end
		end
		if info and message ~= lastMessage then
			local x, y = info.position.x, info.position.y
			if x and y then
				x, y = x, y
				if x and y then
					--if not rareTimed[message] then
					--	rareTimed[message] = true

						if message and lastMessage ~= message then
							lastMessage = message

							local _, _, pID = Q_C:GetPlayerPosition()
							Q_C:SetWaypoint(x, y, pID  , name    , "creatureYell", info.classification, message)
						end
					--end
					lastMessage  = message
					lastPlayer   = name
				end
			end
		end
	end
end

do -- Nodes: flight points, saved way points, rare creatures, and ones added by other addons
	local nodes = {
		name = {
			flight = {},
			saved = {},
			added = {},
			rare = {},
			portal = {},
		},
		index = {
			flight = {},
			saved = {},
			added = {},
			rare = {},
			portal = {},
		},
	}

	--flight paths
	local flightHandler = CreateFromMixins(FlightPointDataProviderMixin)

	local enemy = {
		Horde = "Alliance",
		Alliance = "Horde",
	}

	local GetNodeTypes = {
		flight = {
			ShouldShow = function(nodeInfo)
				local factionGroup = UnitFactionGroup("player")
				if not string.find(nodeInfo.atlasName, enemy[factionGroup]) then
					return true
				end
			end,
			isEnabled = function() return Q_C:GetSets().types.flight end,
			Get = function(mapID)
				return C_TaxiMap.GetTaxiNodesForMap(mapID)
			end,
			kind = "flightMaster",
			translate = function(x, y, mapID) return x, y, mapID end,
			texture = function(node)
				node:SetNormalTexture(123034)
				node:GetNormalTexture():SetBlendMode("ADD")
			end
		},
		saved = {
			ShouldShow = function(nodeInfo)
				--local factionGroup = UnitFactionGroup("player")
				return true
			end,
			Get = function(mapID)
				return Q_C:GetSets().savedLocations
			end,
			kind = "userSelected",
			isEnabled = function() return Q_C:GetSets().types.savedLocations end,
			translate = function(x,y,mapID)

				local vID = Q_C:GetViewedMapID()
				if vID == mapID then
					return x, y, mapID
				end
				local _pX, _pY = HBD:TranslateZoneCoordinates(x, y, mapID, vID, true)
				return _pX, _pY, vID
			end,
			texture = function(node)
				node:SetNormalTexture("Interface\\BUTTONS\\UI-StopButton")
			end,
			clear = function(node, btn)
				if btn == "RightButton" then
					node:SetAlpha(0)
					node:EnableMouse(false)
					tremove(Q_C:GetSets().savedLocations, node.index)
					return true
				end
			end,
		},
		rare = {
			ShouldShow = function(nodeInfo)
				--local factionGroup = UnitFactionGroup("player")
				return true
			end,
			Get = function(mapID)
				return Q_C:GetSets().rareLocations
			end,
			kind = "rare",
			isEnabled = function() return Q_C:GetSets().types.rareLocations end,
			translate = function(x, y, mapID)

				local vID = Q_C:GetViewedMapID()
				if vID == mapID then
					return x, y, mapID
				end
				local _pX, _pY = HBD:TranslateZoneCoordinates(x, y, mapID, vID, true)
				return _pX, _pY, vID
			end,
			texture = function(node, classification)
				local icon
				if ( classification == "rare" or classification == "worldboss" ) then
					icon = ("nameplates-icon-elite-gold");
				elseif ( classification == "rareelite" ) then
					icon = ("nameplates-icon-elite-silver");
				end

				node:SetNormalAtlas(icon)
			end,
			clear = function(node, btn)
				if btn == "RightButton" then
					node:SetAlpha(0)
					node:EnableMouse(false)
					tremove(Q_C:GetSets().rareLocations, node.index)
					return true
				end
			end,
		},
		added = {
			ShouldShow = function(nodeInfo)
				return true
			end,
			Get = function(mapID)
				return Quest_CompassAltDB
			end,
			kind = "suggestions",
			isEnabled = function() return Q_C:GetSets().types.added end,
			translate = function(x, y, mapID)
				local vID = Q_C:GetViewedMapID()
				if vID == mapID then
					return x, y, mapID
				end
				local _pX, _pY = HBD:TranslateZoneCoordinates(x, y, mapID, vID, true)
				return _pX, _pY, vID
			end,
			texture = function(node, classification)
				node:SetNormalTexture("Interface\\BUTTONS\\UI-StopButton")
				node:GetNormalTexture():SetBlendMode("ADD")
			end,
			clear = function(node, btn)
				if btn == "RightButton" then
					node:SetAlpha(0)
					node:EnableMouse(false)
					tremove(Quest_CompassAltDB, node.index)
					return true
				end
			end,
		},
		portal = {
			ShouldShow = function(nodeInfo)
				return true
			end,
			Get = function(mapID)
				return Q_C.AllPortals[mapID] or {}
			end,
			kind = "portal",
			isEnabled = function() return true end,
			translate = function(x, y, mapID)
				local vID = Q_C:GetViewedMapID()
				if vID == mapID then
					return x, y, mapID
				end
				local _pX, _pY = HBD:TranslateZoneCoordinates(x, y, mapID, vID, true)
				return _pX, _pY, vID
			end,
			texture = function(node, classification)
				node:SetNormalTexture("Interface\\AddOns\\Quest_Compass\\artwork\\notifySwirl")
				node:GetNormalTexture():SetBlendMode("ADD")
			end,
			clear = function(node, btn)
				-- if btn == "RightButton" then
					-- node:SetAlpha(0)
					-- node:EnableMouse(false)
					-- tremove(Quest_CompassAltDB, node.index)
					-- return true
				-- end
			end,
		},
	}

	Q_C.nodesTypes = GetNodeTypes

	function GetNodes(nodeType, byIndex)
		if byIndex then
			return nodes.index[nodeType]
		else
			return nodes.name[nodeType]
		end
	end

	function EnableNodes(nodeType, mapID)
		if nodes.name[nodeType] and not nodes.name[nodeType][mapID] then
			nodes.name[nodeType][mapID] = nodes.name[nodeType][mapID] or {}
			nodes.index[nodeType][mapID] = nodes.index[nodeType][mapID] or {}
		end
		return nodes.name[nodeType] and nodes.name[nodeType][mapID], nodes.name[nodeType] and nodes.index[nodeType][mapID]
	end

	function HideNodes(mapID, kind)
		for nodeType, mapInfo in pairs(nodes.index) do
			if (kind and kind == nodeType) or not kind then
				for id, nodes in pairs(mapInfo) do
					for index, node in pairs(nodes) do
						if mapID == id then
							node:Hide()
						end
					end
				end
			end
		end
	end

	function ShowNodes(mapID)
		for nodeType, mapInfo in pairs(nodes.index) do
			for id, nodes in pairs(mapInfo) do
				if GetNodeTypes[nodeType].isEnabled() == true then
					for index, node in pairs(nodes) do
						if mapID == id then
							node:Show()
						end
					end
				end
			end
		end
	end

	function Q_C:GetNode(nodeType, name, mapID, info)
		local nodesByName, nodesByIndex = EnableNodes(nodeType, mapID)

		local node = nodesByName[name]

		if node then
			-- if node already exists, return it.
			return node
		end

		--if the node does not exist, create it.
		node = CreateFrame("Button", nil, self.waypointsFrame)
		node:RegisterForClicks("AnyUp")
		tinsert(nodesByIndex, node)
		nodesByName[name] = node

		node.info = info
		node.index = #nodesByIndex
		node:SetScript("OnEnter", function()
			GameTooltip:SetOwner(node, "ANCHOR_RIGHT")
			GameTooltip:SetText(node.info.name)
			GameTooltip:AddLine(node.info.tooltip)
			GameTooltip:Show()
		end)
		node:SetScript("OnLeave", function() GameTooltip:Hide() end)

		local nodeFunc = GetNodeTypes[nodeType]
		node:SetScript("OnClick", function(_, btn)
			if nodeFunc.clear then
				local ret = nodeFunc.clear(node, btn)
				if ret then
					return
				end
			end

			self.classification = node.info.classification
			self.waypointType = node.info.type
			self.WaypointTitle = node.info.name
			self.waypointTooltip = node.info.tooltip or node.info.name

			local x, y, id = info.x or info.position.x, info.y or info.position.y, info.mapID or info.position.mapID

			self.rareTimed[node.info.name] = true
			tinsert(self.rareTimers, 1, { node.info.tooltip , node.info.name, math.huge, node.info.classification, x, y})

					local px, py, pID = Q_C:GetPlayerPosition()
					local viewedid = Q_C:GetViewedMapID()

			self:SetWaypoint(x, y, id or info.position.mapID or viewedid or pID, node.info.name, node.info.type, node)
		end)

		nodeFunc.texture(node, info.classification)

		node.sizeManager = CreateFrame("Frame")
		node.sizeManager:SetAllPoints(node)
		node.sizeManager:SetScript("OnSizeChanged", function() node:SetSize(25/node.canvas:GetScale(),25/node.canvas:GetScale()) end)

		return node
	end

	local lastMap = {}
	local lastCount = {}
	function Q_C:SetNodes(canvas, mapID)
		if (not IsShiftKeyDown()) and mapID then
			return HideNodes(mapID)
		elseif mapID then
			ShowNodes(mapID)
		end

		if WorldMapFrame:IsVisible() then
			for nodeType, nodeFunc in pairs(GetNodeTypes) do
				local nodes = nodeFunc.Get(mapID)
								--	print(nodeType, #nodes)
				if lastCount[nodeType] ~= #nodes then
					lastMap[nodeType] = nil
				end
				if mapID ~= lastMap[nodeType] then
					if lastMap[nodeType] then
						HideNodes(lastMap, nodeType)
					end
					for i, nodeInfo in ipairs(nodes) do
						if nodeFunc.ShouldShow(nodeInfo) == true then

							local node = Q_C:GetNode(nodeType, nodeInfo.name or nodeInfo.tooltip, mapID, nodeInfo)
							node.info = nodeInfo
							node.info.type = nodeFunc.kind
							node.position = node.position or nodeInfo.position or nodeInfo
							local x, y, id = node.position.x, node.position.y, node.position.mapID or mapID
							local x, y, id = nodeFunc.translate(x, y, id)
							if (x and x < 1) and (y and y < 1) then
								local W, H = (canvas:GetWidth()), ((canvas:GetHeight()))
								local _wX = ((x)) * (W)
								local _wY = H -((y) * (H))
								node:SetSize(70, 70)
								node.canvas = canvas
								node:SetPoint("Center", canvas, "BottomLeft", _wX, _wY)
								node:Hide()
							end
						end
					end
					lastCount[nodeType] = #nodes
					lastMap[nodeType] = mapID
				end
			end
		end
	end
end

do -- Compass Management!!
	function Q_C:GetMap()
		return _G["WorldMapFrame"]
	end

	function Q_C:GetViewedMapID()
		local view, world = _G["WorldMapFrame"]:GetMapID() , HBD.GetWorldMapIDFromMapID[_G["WorldMapFrame"]:GetMapID()]


		local zoneName = GetRealZoneText(view)
		if IsInInstance() then
			local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceID, instanceGroupSize, LfgDungeonID = GetInstanceInfo()
			if zoneName == name then
				local info = Quest_CompassDB.instanceLocations and Quest_CompassDB.instanceLocations[zoneNamemm]
				if info then
					return info.position.mapID, HBD.GetWorldMapIDFromMapID[info.position.mapID]
				end
			end
		end

		if WorldMapFrame:IsVisible() then
			return view, world
		else
			return self:GetPlayerMapID()
		end
	end

	function Q_C:GetPlayerMapID()

		if IsInInstance() then
			local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceID, instanceGroupSize, LfgDungeonID = GetInstanceInfo()

			local info = Quest_CompassDB.instanceLocations and Quest_CompassDB.instanceLocations[name]
			if info then
				return info.position.mapID, HBD.GetWorldMapIDFromMapID[info.position.mapID]
			end
		end

		return C_Map.GetBestMapForUnit("player") , HBD.GetWorldMapIDFromMapID[C_Map.GetBestMapForUnit("player")]
	end

	function Q_C:GetCanvas()
		return _G["WorldMapFrame"]:GetCanvas()
	end

	local queue = {
		shared = {},

	}
	Q_C.queue = queue

	local strings = {
		accept = "Left click to accept.",
		ignore = "Right click to ignore.",
		map = "Left Click to show on map",
		announcement = "I am here!", -- Add some variation!
		askhelp = "I could use some help here!", -- Add some variation!
		announce = "Left Click to announce your current location.", -- Add some variation!
		helpme = "Right Click to ask for help!", -- Add some variation!
		headedto = "I am on my way to:", -- Add some variation!
		returning = "I will return here soon:",
		askaboutsuggest = function(t, x, y) return "Is "..t .." ".."at: "..x..", "..y.."?" end,

	}

	local RecentCoords = {}
	local target = "target"
	local tempTooltip = {}
	local TargetInfo = {}
	local curTiming
	local curTimer
	local function Timer(x, y, id)
		if curTimer ~= x..y..id then
			curTiming = GetTime()
		end
		curTimer = x..y..id
		return math.abs(GetTime() - curTiming)
	end

	local rareSessionTracker = {}

	local check = function(name)
		if name then
			for l,  j in pairs(Q_C.travelQueues.target)do
				if (UnitGUID(target) == j[5] ) and (UnitIsDead(target) == true) then
					tremove(Q_C.travelQueues.target, l)
				elseif j[5] == name then
					return true
				end
			end
		end
	end

	function GetHandler(key, singular, plural, navIcon, notifyIcon, announce, notifyTooltip, rightClick, leftClick)--, navTooltip, navClick)
		local f = {}

		f.get = function()
			return unpack((Q_C.travelQueues[key] and Q_C.travelQueues[key][1]) or {})
		end

		f.compass = {
					tooltip = (type(navTooltip) == "function") and navTooltip or function(self, x, y, mapID)
						wipe(tempTooltip)

						if Q_C.travelQueues[key] and #Q_C.travelQueues[key]>0 then
							tinsert(tempTooltip, singular)
							local one = Q_C.travelQueues[key][1]
							local name, x, y, id, creatureName, m,i = unpack(one)

							--print(m, i)

							tinsert(tempTooltip, "   "..name)
							if creatureName then
								--print(unpack(creatureName))
								tinsert(tempTooltip, ""..creatureName)
							end
							if m then
								--tinsert(tempTooltip, ""..m)
							end
							tinsert(tempTooltip, "   "..format("%.2f, %.2f", x * 100 or 0, y * 100 or 0))

							tinsert(tempTooltip, "Left click to view map.")
							tinsert(tempTooltip, "Right click to clear.")

							return tempTooltip
						end
					end,
					icon = function(self, x, y, mapID)
						local texture, useAtlasSize, w, h, style, l, r, t, b, questID

						if type(navIcon) == "function" then
							texture, useAtlasSize, w, h, style, l, r, t, b, questID = navIcon()
						else
							texture, useAtlasSize, w, h, style, l, r, t, b = unpack(navIcon)
						end

						return texture, useAtlasSize, w, h, style, l, r, t, b, questID
					end,
					click = function(self, btn, x, y, mapID)
						if IsShiftKeyDown() then
							local f = Q_C.travelQueues[key][1]
							tremove(Q_C.travelQueues[key], 1)
							tinsert(Q_C.travelQueues[key], f)
						elseif btn == "LeftButton" then
							local one = Q_C.travelQueues[key][1]
							local name, x, y, id = unpack(one)
							OpenWorldMap(id)
							--show map
						else
							--ignore
							tremove(Q_C.travelQueues[key], 1)
						end
					end,
					shouldShow = function(self, playerMapID)
						if ((Q_C.travelQueues[key] and #Q_C.travelQueues[key]) or 0) > 0 then
							local one = Q_C.travelQueues[key][1]
							local name, x, y, id = unpack(one)

							local px, py, pID = Q_C:GetPlayerPosition()

							local range = (f.noRange == nil) and Q_C.range(pID, px, py, id, x, y) or math.huge

							if  range < 15 then
								local reset = Timer(x, y, id)
								if reset >= 9 then
									tremove(Q_C.travelQueues[key], 1)
								end
							end
							return {x, y, id}, key
						end
					end,
				}
		f.notify = {
			tooltip = (type(notifyTooltip) == "function") and notifyTooltip or function(self, x, y, mapID)
				wipe(tempTooltip)

				tinsert(tempTooltip, leftClick)

				if #Q_C.travelQueues[key] >= 1 then
					local two = Q_C.travelQueues[key][2]
					if two then
						if Q_C.travelQueues[key] and #Q_C.travelQueues[key] > 2 then
							tinsert(tempTooltip, (max(0, (Q_C.travelQueues[key] and #Q_C.travelQueues[key] - 1) or 0)) .. " "..string.lower(plural).." waiting.")
						end
						tinsert(tempTooltip, "Next "..singular..":")
						local name, x, y, id = unpack(two)
						tinsert(tempTooltip, "   "..name)
						tinsert(tempTooltip, "   "..format("%.2f, %.2f", x * 100 or 0, y * 100 or 0))

						tinsert(tempTooltip, "Right click to clear next "..string.lower(singular)..".")
					end
				return tempTooltip

				end
			end,
			icon = function(self, x, y, mapID)
				if notifyIcon then
					if type(notifyIcon) == "table" then
						return unpack(notifyIcon)
					elseif type(notifyIcon) == "function" then
						return notifyIcon()
					end
				end
			end,
			click = navClick or function(self, btn, x, y, mapID)
				if IsShiftKeyDown() then

				elseif btn == "LeftButton" then
----					--share to chat
					local one = Q_C.travelQueues[key][1]
					local name, x, y, id, t = unpack(one)
					x = format("%.2f", x*100)
					y = format("%.2f", y*100)
					Q_C:SendToChat(((type(announce) == "function") and announce(t, x, y)) or ((type(announce) == "string")  and announce .." "..x..", "..y) or "")
				elseif btn == "RightButton" then
					if #Q_C.travelQueues[key] > 1 then
						tremove(Q_C.travelQueues[key], 2)
					end
				end
			end,
			shouldShow = function(self, playerMapID)
				if ((Q_C.travelQueues[key] and #Q_C.travelQueues[key]) or 0) > 0 then
					local one = Q_C.travelQueues[key][1]
					local name, x, y, id = unpack(one)



					self.questID = nil

					return {x, y, id}, key
				end
			end,
		}
		return f
	end

	local instanceLocations = {}

	local way = {
		{--Basic
		--[[
			compass = {
				tooltip = function(self, x, y, mapID)
					wipe(tempTooltip)

					if IsAltKeyDown() or (InCombatLockdown() and UnitExists(target)) then
						tinsert(tempTooltip, "Announce target's position")
						tinsert(tempTooltip, "Ask for help with target.")
					else
						local anounce = (type(strings.announce) == "table") and strings.announce() or strings.announce -- future compatible
						local help = (type(strings.helpme) == "table") and strings.helpme() or strings.helpme -- future compatible
						tinsert(tempTooltip, anounce)
						tinsert(tempTooltip, help)
					end
					return tempTooltip
				end,
				icon = function(self, x, y, mapID)

				end,
				click = function(self, btn, x, y, mapID)
					local px, py, pID = Q_C:GetPlayerPosition()

					px = format("%.2f", px*100)
					py = format("%.2f", py*100)

					if IsAltKeyDown() or (InCombatLockdown() and UnitExists(target)) then
						local name = UnitName("target")
						local health = math.floor((UnitHealth("target")/UnitHealthMax("target"))*100)
						local combat = UnitAffectingCombat("target")
						local classification = classification
						local level = UnitLevel("target")
						local guid = UnitGUID("target")

						if btn == "RightButton" then
							Q_C:SendToChat("Could I get some help with "..name .." , ".."Level ".. level.. ", " .. health.."% ".." at "..px..", "..py)
						elseif btn == "LeftButton" then
							--share
							Q_C:SendToChat(name .." , ".."Level ".. level.. ", ".. health.."% ".." at "..px..", "..py.."")
						end
						return
					end

					if btn == "LeftButton" then
						--i am here!
						Q_C:SendToChat((type(strings.announcement) == "table") and strings.announcement() or strings.announcement .." "..px..", "..py)
					else
						Q_C:SendToChat((type(strings.askhelp) == "table") and strings.askhelp() or strings.askhelp .." "..px..", "..py)
						--help me!
					end
				end,
				shouldShow = function(self, playerMapID)
					local px, py, pID = Q_C:GetPlayerPosition()
					local vID = Q_C:GetViewedMapID()

					if IsInInstance() then
						local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceID, instanceGroupSize, LfgDungeonID = GetInstanceInfo()
						local info = Quest_CompassDB.instanceLocations and Quest_CompassDB.instanceLocations[name]
						if info then
							return {info.position.x, info.position.y, info.position.mapID}

						end
					end
					if vID then
						Quest_CompassDB.instanceLocations = Quest_CompassDB.instanceLocations or {}

						if not instanceLocations[vID] then
							instanceLocations[vID] = C_EncounterJournal.GetDungeonEntrancesForMap(vID)
							--wipe(Quest_CompassDB.instanceLocations)
						end

						if #instanceLocations[vID] > 0 then
							for i, b in pairs(instanceLocations[vID]) do
								if  b.name and not Quest_CompassDB.instanceLocations[ b.name] then
									b.position.mapID = vID
									Quest_CompassDB.instanceLocations[ b.name] = Quest_CompassDB.instanceLocations[ b.name] or b
								end
							end
						end
					end
					return {px, py, vID}
				end,
			},
			--]]
			notify = {
				tooltip = function(self, x, y, mapID)
					wipe(tempTooltip)

					if IsAltKeyDown() or (InCombatLockdown() and UnitExists(target)) then
						tinsert(tempTooltip, "Announce target's position")
						tinsert(tempTooltip, "Ask for help with target.")
					else
						local anounce = (type(strings.announce) == "table") and strings.announce() or strings.announce -- future compatible
						local help = (type(strings.helpme) == "table") and strings.helpme() or strings.helpme -- future compatible
						tinsert(tempTooltip, anounce)
						tinsert(tempTooltip, help)
					end
					return tempTooltip
				end,
				icon = function(self, x, y, mapID)

				end,
				click = function(self, btn, x, y, mapID)
					local px, py, pID = Q_C:GetPlayerPosition()

					px = format("%.2f", px*100)
					py = format("%.2f", py*100)

					if IsAltKeyDown() or (InCombatLockdown() and UnitExists(target)) then
						local name = UnitName("target")
						local health = math.floor((UnitHealth("target")/UnitHealthMax("target"))*100)
						local combat = UnitAffectingCombat("target")
						local classification = classification
						local level = UnitLevel("target")
						local guid = UnitGUID("target")

						if btn == "RightButton" then
							Q_C:SendToChat("Could I get some help with "..name .." , ".."Level ".. level.. ", " .. health.."% ".." at "..px..", "..py)
						elseif btn == "LeftButton" then
							--share
							Q_C:SendToChat(name .." , ".."Level ".. level.. ", ".. health.."% ".." at "..px..", "..py.."")
						end
						return
					end

					if btn == "LeftButton" then
						--i am here!
						Q_C:SendToChat((type(strings.announcement) == "table") and strings.announcement() or strings.announcement .." "..px..", "..py)
					else
						Q_C:SendToChat((type(strings.askhelp) == "table") and strings.askhelp() or strings.askhelp .." "..px..", "..py)
						--help me!
					end
				end,
				shouldShow = function(self, playerMapID)
					local px, py, pID = Q_C:GetPlayerPosition()
					local vID = Q_C:GetViewedMapID()

					if IsInInstance() then
						local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceID, instanceGroupSize, LfgDungeonID = GetInstanceInfo()
						local info = Quest_CompassDB.instanceLocations and Quest_CompassDB.instanceLocations[name]
						if info then
							return {info.position.x, info.position.y, info.position.mapID}

						end
					end
					if vID then
						Quest_CompassDB.instanceLocations = Quest_CompassDB.instanceLocations or {}

						if not instanceLocations[vID] then
							instanceLocations[vID] = C_EncounterJournal.GetDungeonEntrancesForMap(vID)
							--wipe(Quest_CompassDB.instanceLocations)
						end

						if #instanceLocations[vID] > 0 then
							for i, b in pairs(instanceLocations[vID]) do
								if  b.name and not Quest_CompassDB.instanceLocations[ b.name] then
									b.position.mapID = vID
									Quest_CompassDB.instanceLocations[ b.name] = Quest_CompassDB.instanceLocations[ b.name] or b
								end
							end
						end
					end
					return {px, py, vID}
				end,
			},
		},
	}

	Q_C.way = way

	local lastDeadPos

	tinsert(way, 1, {--Corpse - you are dead! stop that!
		compass = {
			tooltip = function()
				local _, _, pID = Q_C:GetPlayerPosition()
				local position = C_DeathInfo.GetCorpseMapPosition(pID)
				wipe(tempTooltip)
				x, y = math.floor(position.x*10000)/100, math.floor(position.y*10000)/100

				tinsert(tempTooltip, "You are dead!")
				tinsert(tempTooltip, x..", "..y)
				tinsert(tempTooltip, "Left Click to view World Map.")
				tinsert(tempTooltip, "Right click to ignore.")

				return tempTooltip
			end,
			icon = function(self, x, y, mapID)
				return "Interface\\Worldmap\\Skull_64Grey", nil, 45, 45, "texture"
			end,
			click = function(self, btn, mapID)
				local _, _, pID = Q_C:GetPlayerPosition()
				if btn == "LeftButton" then
					OpenWorldMap(pID)
					--show map
				elseif pID then
					--ignore this death
					--RecentCoords
					local position = C_DeathInfo.GetCorpseMapPosition(pID)
					if position then
						RecentCoords["dead: "..x..", "..y ] = true
						RecentCoords[x..", "..y ] = true
					end
				end
			end,
			shouldShow = function(self, playerMapID)
				local px, py, pID = Q_C:GetPlayerPosition()
				if not pID then
					return
				end
				local position = C_DeathInfo.GetCorpseMapPosition(pID)

				if position then

					local x, y = math.floor(position.x*10000)/100 , math.floor(position.y*10000)/100

					if RecentCoords["dead: "..x..", "..y ] or RecentCoords[x..", "..y ] then
						return nil
					end



					local range = Q_C.range(pID, px, py, pID, position.x, position.y)
					if range >= 15 then
						if lastDeadPos ~= position.x.. position.y.. pID then
							lastDeadPos = position.x.. position.y.. pID
							Q_C.snapTo = 1
						end

						return {position.x, position.y, pID}, "corpse"
					end
				end
			end,
			update = function(self, x, y, mapID)

			end,
		},
		notify = {
			tooltip = function(self, x, y, mapID)
				local _, _, pID = Q_C:GetPlayerPosition()
				if not pID then
					return
				end
				local position = C_DeathInfo.GetCorpseMapPosition(pID)
				if position then
					wipe(tempTooltip)
					tinsert(tempTooltip, "Left click to ask for a res!")
					tinsert(tempTooltip, "Right click to ignore.")
					return tempTooltip
				end
			end,
			icon = function(self, x, y, mapID)
				return .25, .25, 2.5, 1
			end,
			click = function(self, btn, x, y, mapID)
				if btn == "LeftButton" then
					--ask for res
					--self:SendToChat(message)
				else
					--temp ignore
				end
			end,
			shouldShow = function(self, playerMapID)
				local _, _, pID = Q_C:GetPlayerPosition()
				if not pID then
					return
				end
				local position = C_DeathInfo.GetCorpseMapPosition(pID)
				local x, y
				if position then
					return {position.x, position.y, playerMapID}
				end
			end,
			update = function(self, x, y, mapID)

			end,
		},
	})

	tinsert(way, 2, GetHandler("anchors", "Anchor", "Anchors", {Q_C.TexturePath.."anchor", nil, 60, 60, "texture", 0, 1, 0, 1}, {0,0,1,0}, strings.returning, "help", "rightClick", "Left click to announce that you will return."))

	local f = GetHandler("target", "Target", "Targets",
		function(self, x, y, mapID)
			local one = Q_C.travelQueues.target[1]
			local name, x, y, id, t, l = unpack(one)
			local classification = type(l) == "table" and l.classification or l
			local icon =  ( classification == "rare" or classification == "worldboss" ) and "nameplates-icon-elite-gold" or ( classification == "rareelite" ) and "nameplates-icon-elite-silver"
			if icon then
				return icon, nil, 45, 45, "atlas"
			else
				return "Interface\\TARGETINGFRAME\\UI-RaidTargetingIcon_4", nil, 45, 45, "texture"
			end
		end,
		"", {1,1,1,1})
	f.noRange = true
	f.update = function()
		local tar = (UnitExists(target) == true) and UnitGUID(target)

				local name = UnitName("target")
		local classification = UnitClassification(target)-- "worldboss", "rareelite", "elite", "rare", "normal", "trivial", or "minus"

		check(name)

		if tar and not rareSessionTracker[name] then
			--if (IsAltKeyDown() == true)
			if (classification == "worldboss")
			or (classification == "rareelite")
			or (classification == "rare") then
				wipe(TargetInfo)
					TargetInfo.name = UnitName("target")
					TargetInfo.health = math.floor((UnitHealth("target")/UnitHealthMax("target"))*100)
					TargetInfo.combat = UnitAffectingCombat("target")
					TargetInfo.classification = classification
					TargetInfo.level = UnitLevel("target")
					TargetInfo.guid = tar


				local info
				for i, inf in pairs(Quest_CompassDB.rareLocations) do
					if inf and inf.name == name then
						info = inf
						break
					end
				end

				local xx, xy, id = info and info.position.x, info and info.position.y, info and info.position.id
				if xx and yy and id then
					x, y, pID = xx,  yy, id
				end

				local x,y,pID = Q_C:GetPlayerPosition()
				if check(name) ~= true then
					Q_C:StoreRareInfo(TargetInfo.guid, x, y, pID, TargetInfo.name, TargetInfo.classification)
					Q_C:SetWaypoint(x, y, pID, TargetInfo.name, "target", TargetInfo)

					rareSessionTracker[name] =  true
				end
			end
		end
	end

		f.notify.tooltip = function(self, x, y, mapID)
			wipe(tempTooltip)

			tinsert(tempTooltip, "Left click to announce target.")
			tinsert(tempTooltip, "Right click to ask for help with target.")

			if #Q_C.travelQueues.target > 1 then
				local two = Q_C.travelQueues.target[2]
				local name, x, y, id, t, rareTargetInfo = unpack(two)

				local name = rareTargetInfo.name
				local level = rareTargetInfo.level
				local health = rareTargetInfo.health

				if UnitGUID(target) == rareTargetInfo.guid then
					health = rareTargetInfo.health
				end

				tinsert(tempTooltip, "    "..name)
				tinsert(tempTooltip, "    Level: "..level)
				tinsert(tempTooltip, "    Health: "..health.."%")

				tinsert(tempTooltip, "Shift click to clear next target.")

				if Q_C.travelQueues[key] and #Q_C.travelQueues.target > 2 then
					tinsert(tempTooltip, (max(0, (Q_C.travelQueues.target and Q_C.travelQueues.target - 1) or 0)) .. " targets waiting.")
				end
			end
			return tempTooltip
		end

		f.compass.tooltip = function(self, x, y, mapID)
			wipe(tempTooltip)

			local one = Q_C.travelQueues.target[1]
			local name, x, y, id, t, rareTargetInfo = unpack(one)

			local name = rareTargetInfo.name
			local level = rareTargetInfo.level
			local health = rareTargetInfo.health

			if UnitGUID(target) == rareTargetInfo.guid then
				health = math.floor((UnitHealth(target)/UnitHealthMax(target))*100)
			end

			local status = rareTargetInfo.classification
			if (status == "worldboss")
			or (status == "rareelite")
			or (status == "rare") then
				tinsert(tempTooltip, "Rare Enemy")
			else
				tinsert(tempTooltip, "Target")
			end
				tinsert(tempTooltip, "    "..name)
				tinsert(tempTooltip, "    Level: "..level)
				tinsert(tempTooltip, "    Health: "..health.."%")

				tinsert(tempTooltip, format("%.2f, %.2f", x * 100 or 0, y * 100 or 0))
				if #Q_C.travelQueues.target>1 then
					tinsert(tempTooltip, "Shift click to cycle.")
				end

			tinsert(tempTooltip, "")
			tinsert(tempTooltip, "Left click to view map.")
			tinsert(tempTooltip, "Right click to clear.")
			return tempTooltip
		end
		f.notify.click = function(self, btn, x, y, mapID)
			local one = Q_C.travelQueues.target[1]
			local _, x, y, id, _, rareTargetInfo = unpack(one)

			local name = rareTargetInfo.name
			local level = rareTargetInfo.level
			local health = rareTargetInfo.health

			if UnitGUID(target) == rareTargetInfo.guid then
				health = math.floor((UnitHealth(target)/UnitHealthMax(target))*100)
			end

			local status = rareTargetInfo.classification

			x = format("%.2f", x*100)
			y = format("%.2f", y*100)

			if btn == "RightButton" then
				Q_C:SendToChat("Could I get some help with "..name .." , ".."Level ".. level.. ", " .. health.."% ".." at "..x..", "..y)
			elseif btn == "LeftButton" then
				--share
				Q_C:SendToChat("Rare Creature: "..name .." , ".."Level ".. level.. ", ".. health.."% ".." at "..x..", "..y.."")
			end
		end


	tinsert(way, 3, f)

	tinsert(way, 4, GetHandler("playerShared", "Shared", "Shared", {"Interface\\BUTTONS\\UI-StopButton", nil, 45, 45, "texture"}, {.88, .21,1,1}, strings.askaboutsuggest, "help", "rightClick", "Left click to ask if suggestion is at destination."))

	local iconCheat = {
		rare = "nameplates-icon-elite-gold",
		worldboss = "nameplates-icon-elite-gold",
		rareelite = "nameplates-icon-elite-silver",
	}

	tinsert(way, 5, GetHandler("creatureYell", "Announced Rare", "Announced Rares", function()
		local one = Q_C.travelQueues.creatureYell[1]
		if not one then return end
		local name, x, y, id, t, k, classification = unpack(one)

		return iconCheat[classification], nil, 45, 45, "atlas"
	end, {}, strings.askaboutsuggest, "help", "rightClick", "Left click to ask if creature is at destination."))

	tinsert(way, 6, GetHandler("flightMaster", "Taxi", "Taxis", {"Interface\\TAXIFRAME\\taxiassets", nil, 30, 30, "texture", 0, .5, .055, .21}, {.5, .31, .17, 1}, strings.headedto, "help", "rightClick", "Left click to announce that you are on your way to destination."))

	tinsert(way, 7, GetHandler("rare", "Saved Rare", "Saved Rares", function()
		local one = Q_C.travelQueues.rare[1]
		local name, x, y, id, t, k, classification = unpack(one)

		return iconCheat[classification], nil, 45, 45, "atlas"
	end, {}, strings.askaboutsuggest, "help", "rightClick", "Left click to ask if creature is at destination."))

	tinsert(way, 8, GetHandler("userSelected", "Destination", "Destinations", {"Interface\\Worldmap\\UI-World-Icon", nil, 60, 60, "texture"}, {.25, .72, .33, 1}, strings.headedto, "help", "rightClick", "Left click to announce that you are on your way to destination."))

	tinsert(way, 9, GetHandler("suggestions", "Suggestion", "Suggestions", {Q_C.TexturePath.."suggestion", nil, 60, 60, "texture", 0, 1, 0, 1}, {1,0,0,1}, strings.askaboutsuggest, "help", "rightClick", "Left click to ask if suggestion is at destination."))

	--10: tracked Quest and nearest standard quest.
	local f = {compass = {}, notify = {}}
	f.compass.click = function(self, btn)

		local _,_,_, questID = f.get()

		if IsAltKeyDown() then
			if btn == "LeftButton" then
				--temp ignore this session
				Q_C.sessionIgnore[questID] = true
			elseif btn == "RightButton" then
				--restore temp
				Q_C.sessionIgnore = {}
			end
		elseif IsControlKeyDown() then
			if btn == "LeftButton" then
				--perm ignore
				Q_C:GetSets().ignored = Q_C:GetSets().ignored or {}
				Q_C:GetSets().ignored[questID] = true
			elseif btn == "RightButton" then
				--restore perm
				Q_C:GetSets().ignored = {}
			end
		elseif btn == "LeftButton" then
			QuestMapFrame_OpenToQuestDetails(questID)
		elseif btn == "RightButton" then
			--untrack

				if (questID == C_QuestSession.GetSuperTrackedQuest()) or (questID == C_SuperTrack.GetSuperTrackedQuestID()) then
					QuestObjectiveTracker_UntrackQuest(nil, questID)
					BonusObjectiveTracker_UntrackWorldQuest(questID)
					C_QuestSession.SetQuestIsSuperTracked(questID, false)
				else
					C_SuperTrack.SetSuperTrackedQuestID(questID)
					C_QuestSession.SetQuestIsSuperTracked(questID, true)
				end

		end
	end
 
	local function AddQuestRewardsToTooltip(tooltip, questID, style)
		style = style or TOOLTIP_QUEST_REWARDS_STYLE_DEFAULT;

		if ( GetQuestLogRewardXP(questID) > 0 or GetNumQuestLogRewardCurrencies(questID) > 0 or GetNumQuestLogRewards(questID) > 0 or
			GetQuestLogRewardMoney(questID) > 0 or GetQuestLogRewardArtifactXP(questID) > 0 or GetQuestLogRewardHonor(questID) > 0 or
			GetNumQuestLogRewardSpells(questID) > 0) then
			if tooltip.ItemTooltip then
				tooltip.ItemTooltip:Hide();
			end

			GameTooltip_AddBlankLinesToTooltip(tooltip, style.prefixBlankLineCount);
			if style.headerText and style.headerColor then
				GameTooltip_AddColoredLine(tooltip, style.headerText, style.headerColor, style.wrapHeaderText);
			end
			GameTooltip_AddBlankLinesToTooltip(tooltip, style.postHeaderBlankLineCount);

			local hasAnySingleLineRewards, showRetrievingData = QuestUtils_AddQuestRewardsToTooltip(tooltip, questID, style);

			if hasAnySingleLineRewards and tooltip.ItemTooltip and tooltip.ItemTooltip:IsShown() then
				GameTooltip_AddBlankLinesToTooltip(tooltip, 1);
				if showRetrievingData then
					GameTooltip_AddColoredLine(tooltip, RETRIEVING_DATA, RED_FONT_COLOR);
				end
			end
		end
	end

	f.compass.tooltip = function(self)
		local x, y, mapID, questID = f.get(self)

		if (not questID) then
			return
		end
		wipe(tempTooltip)
		local f = tempTooltip

		local titleText

		self.questID = questID

		local logIndex = questID and C_QuestLog.GetLogIndexForQuestID(questID)

		local title, factionID, capped, factionName = Q_C.getFactionInfo(questID)

		--print(title, factionID, capped, factionName)

		local info = questID and C_QuestLog.GetQuestTagInfo(questID)
			-- .quality
			-- .isElite
			-- .worldQuestType
			-- .tagID
			-- .displayExpiration
			-- .tagName

		--print(info.tagName)

		local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName

		local color = WORLD_QUEST_QUALITY_COLORS[rarity]

		if color then
			titleText = {title, color.r, color.g, color.b}
		else
			titleText = title
		end

		tinsert(f, titleText or "")

		local numObjectives = questID and C_QuestLog.GetNumQuestObjectives(questID) or 0

		local questLogIndex = C_QuestLog.GetLogIndexForQuestID(questID);
		local completionText = GetQuestLogCompletionText(questLogIndex);
		if completionText then
			local color = HIGHLIGHT_FONT_COLOR
			tinsert(f, {"    "..QUEST_DASH .. completionText, color.r, color.g, color.b, true})
		end

		local wayPointText = C_QuestLog.GetNextWaypointText(questID)

		if wayPointText then
			local color = HIGHLIGHT_FONT_COLOR
			tinsert(f, {"    "..QUEST_DASH .. wayPointText, color.r, color.g, color.b, true})
		else
			for objectiveIndex = 1, numObjectives do
				local objectiveText, objectiveType, finished = GetQuestObjectiveInfo(questID, objectiveIndex, false)
				if ( objectiveText and #objectiveText > 0 ) then
					local color = finished and GRAY_FONT_COLOR or HIGHLIGHT_FONT_COLOR
					tinsert(f, {"    "..QUEST_DASH .. objectiveText, color.r, color.g, color.b, true})
				end
			end
		end

		tinsert(f, "     ")

		local playerMapID = Q_C:GetViewedMapID()

		local mapID

		if  QuestUtils_IsQuestWorldQuest(questID) == true then
			mapID = C_TaskQuest.GetQuestZoneID(questID) or GetQuestUiMapID(questID) or playerMapID
		else
			mapID = C_QuestLog.GetNextWaypoint(questID)
		end

		local info = mapID and C_Map.GetMapInfo(mapID)

		local WaypointZone = info and info.name

		if WaypointZone then
			if mapID ~= Q_C:GetPlayerMapID() then
				tinsert(f, 2, {QUEST_DASH .. WaypointZone..((factionName and " | "..factionName) or ""),  1, 0, 0, true})
			else
				tinsert(f, 2, {QUEST_DASH .. WaypointZone..((factionName and " | "..factionName) or ""),  1, 1, 1, true})
			end
		end

		tinsert(f, function() WorldMap_AddQuestTimeToTooltip(questID) end)

		tinsert(f, function() QuestUtils_AddQuestTypeToTooltip(GameTooltip, questID, NORMAL_FONT_COLOR) end)

		local percent = C_TaskQuest.GetQuestProgressBarInfo(questID)
		if ( percent ) then
			tinsert(f, function() GameTooltip_ShowProgressBar(GameTooltip, 0, 100, percent, PERCENTAGE_STRING:format(percent)) end)
		end

		tinsert(f, "     ")

		if IsAltKeyDown() then
			tinsert(f, "Left Click to temporarily ignore.")
			tinsert(f, "Right Click to restore temporary ignores.")
		elseif IsControlKeyDown() then
			tinsert(f, "Left Click to permanently ignore.")
			tinsert(f, "Right Click to restore permanent ignores.")
		else
			tinsert(f, "Left Click to view map.")
			if questID == (C_QuestSession.GetSuperTrackedQuest() or C_SuperTrack.GetSuperTrackedQuestID()) or (isTask == true) then
				tinsert(f, "Right Click to untrack.")
			else
				tinsert(f, "Right Click to track.")
			end
			tinsert(f, "Hold a modifier key for more options.")
		end

		tinsert(f, function(self)
			AddQuestRewardsToTooltip(GameTooltip, questID)
		end)

		return f
	end

	f.compass.icon = function()

		local x, y, mapID, questID = f.get()

		return _, _, _, _, _, _, _, _, _, questID
	end

	f.compass.shouldShow = function(self, ...)
		local x, y, mapID, questID = f.get(self)
		if questID then

			if Q_C.lastWorldQuestID ~= questID then
		--		Q_C.wayStep = 10
			end
			Q_C.lastWorldQuestID = questID
			return {x, y, mapID}, "quest", questID
		end
	end

	f.notify.tooltip = function(self)

		wipe(tempTooltip)
		tinsert(tempTooltip, "Tracked Quests")

		tinsert(tempTooltip, "Shift Click to share link.")
		tinsert(tempTooltip, "Left Click to announce that you are on this quest.")
		tinsert(tempTooltip, "Right Click to ask for help with this quest.")

		return tempTooltip
	end

	f.notify.icon = {1, .88, .21, 1}

	f.notify.click = function(self, btn)
		local x, y, mapID, questID = f.get(self)

		if IsShiftKeyDown() then
			local link = GetQuestLink(questID)
			if link and ChatEdit_GetActiveWindow() then
				ChatEdit_InsertLink(link)
			end
		elseif btn == "LeftButton" then
			--announce
			local link = GetQuestLink(questID)
			if link then
				Q_C:SendToChat("I am working on " ..link)
			end
		elseif btn == "RightButton" then
			--help
			local link = GetQuestLink(questID)
			if link then
						x = format("%.2f", x*100)
						y = format("%.2f", y*100)
				Q_C:SendToChat("I could use some help with " ..link..". I am at: "..x..", "..y)
			end
		end
	end

	f.notify.shouldShow = function(...)
		return f.compass.shouldShow(...)
	end

	f.get = function()
		--get the id for world or standard quests
		local questID = (Q_C:GetSets().worldQuests.show == true) and C_QuestSession.GetSuperTrackedQuest() or (Q_C:GetSets().types.quests == true) and C_SuperTrack.GetSuperTrackedQuestID()

		questID = (questID and ( questID > 0)) and questID or nil

		--local questID

		local world = (Q_C:GetSets().worldQuests.show == true) and C_QuestSession.GetSuperTrackedQuest()
		local standard = (Q_C:GetSets().types.quests == true) and C_SuperTrack.GetSuperTrackedQuestID()

		do
			local worldComplete = world and C_QuestLog.IsQuestFlaggedCompleted(world) or nil
			local standardComplete  = standard and C_QuestLog.IsQuestFlaggedCompleted(standard) or nil

			if worldComplete == true then
			--	QuestObjectiveTracker_UntrackQuest(nil, questID)
					BonusObjectiveTracker_UntrackWorldQuest(questID)
					questID = standard
			end
			if standardComplete == true then
				QuestObjectiveTracker_UntrackQuest(nil, questID)
				--BonusObjectiveTracker_UntrackWorldQuest(questID)
				questID = nil
			end
		end



		if (type(questID) ~= "number") or Q_C:ShouldIgnore(questID) then
			questID = nil
		end

		local mapID = Q_C:GetPlayerMapID()


		if (Q_C:GetSets().types.nearestQuest == true) and ((questID == 0) or (questID == nil)) then
			local id = Questra:GetNearestQuestID(true, mapID, Q_C.shouldIgnore)
			if id then
				questID = id
			end
		end

		if not questID then
			return
		end

		local x, y, mapID = Questra:GetQuestLocation(questID)


		if (not mapID) or (not (questID and (x or y))) then
			return 0, 0, mapID
		end

		Q_C.questID = questID

		return x, y, mapID, questID
	end
	tinsert(way, 10, f)

	do
		--10: Closest World Quest
		local f = {compass = {}, notify = {}}
		f.compass.click = function(self, btn)

			local _,_,_, questID = f.get()

			if IsAltKeyDown() then
				if btn == "LeftButton" then
					--temp ignore this session
					Q_C.sessionIgnore[questID] = true
				elseif btn == "RightButton" then
					--restore temp
					Q_C.sessionIgnore = {}
				end
			elseif IsControlKeyDown() then
				if btn == "LeftButton" then
					--perm ignore
					Q_C:GetSets().ignored = Q_C:GetSets().ignored or {}
					Q_C:GetSets().ignored[questID] = true
				elseif btn == "RightButton" then
					--restore perm
					Q_C:GetSets().ignored = {}
				end
			elseif btn == "LeftButton" then
				QuestMapFrame_OpenToQuestDetails(questID)
			elseif btn == "RightButton" then
				--track or untrack

				if (questID == C_QuestSession.GetSuperTrackedQuest()) or (questID == C_SuperTrack.GetSuperTrackedQuestID()) then
					QuestObjectiveTracker_UntrackQuest(nil, questID)
					BonusObjectiveTracker_UntrackWorldQuest(questID)
					C_QuestSession.SetQuestIsSuperTracked(questID, false)
				else
					C_SuperTrack.SetSuperTrackedQuestID(questID)
					C_QuestSession.SetQuestIsSuperTracked(questID, true)
					Q_C.snapTo = 10
				end

			end
		end

		local function AddQuestRewardsToTooltip(tooltip, questID, style)
			style = style or TOOLTIP_QUEST_REWARDS_STYLE_DEFAULT;

			if ( GetQuestLogRewardXP(questID) > 0 or GetNumQuestLogRewardCurrencies(questID) > 0 or GetNumQuestLogRewards(questID) > 0 or
				GetQuestLogRewardMoney(questID) > 0 or GetQuestLogRewardArtifactXP(questID) > 0 or GetQuestLogRewardHonor(questID) > 0 or
				GetNumQuestLogRewardSpells(questID) > 0) then
				if tooltip.ItemTooltip then
					tooltip.ItemTooltip:Hide();
				end

				GameTooltip_AddBlankLinesToTooltip(tooltip, style.prefixBlankLineCount);
				if style.headerText and style.headerColor then
					GameTooltip_AddColoredLine(tooltip, style.headerText, style.headerColor, style.wrapHeaderText);
				end
				GameTooltip_AddBlankLinesToTooltip(tooltip, style.postHeaderBlankLineCount);

				local hasAnySingleLineRewards, showRetrievingData = QuestUtils_AddQuestRewardsToTooltip(tooltip, questID, style);

				if hasAnySingleLineRewards and tooltip.ItemTooltip and tooltip.ItemTooltip:IsShown() then
					GameTooltip_AddBlankLinesToTooltip(tooltip, 1);
					if showRetrievingData then
						GameTooltip_AddColoredLine(tooltip, RETRIEVING_DATA, RED_FONT_COLOR);
					end
				end
			end
		end

		f.compass.tooltip = function(self)
			local x, y, mapID, questID = f.get(self)

			if (not questID) or (questID and ( not HaveQuestData(questID) )) then
				return
			end
			wipe(tempTooltip)
			local f = tempTooltip

			local titleText

			self.questID = questID
			local title, factionID, capped, factionName = Q_C.getFactionInfo(questID)
			local info = questID and C_QuestLog.GetQuestTagInfo(questID)
				-- .quality
				-- .isElite
				-- .worldQuestType
				-- .tagID
				-- .displayExpiration
				-- .tagName

			local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName

			local color = WORLD_QUEST_QUALITY_COLORS[rarity]

			if color then
				titleText = {title, color.r, color.g, color.b}
			else
				titleText = title
			end

			tinsert(f, titleText or "")



		local playerMapID = Q_C:GetViewedMapID()

		local mapID

		if  QuestUtils_IsQuestWorldQuest(questID) == true then
			mapID = C_TaskQuest.GetQuestZoneID(questID) or GetQuestUiMapID(questID) or playerMapID
		else
			mapID = C_QuestLog.GetNextWaypoint(questID)
		end

		local info = mapID and C_Map.GetMapInfo(mapID)

		local WaypointZone = info and info.name

		if WaypointZone then
			if mapID ~= Q_C:GetPlayerMapID() then
				tinsert(f, 2, {QUEST_DASH .. WaypointZone..((factionName and " | "..factionName) or ""),  1, 0, 0, true})
			else
				tinsert(f, 2, {QUEST_DASH .. WaypointZone..((factionName and " | "..factionName) or ""),  1, 1, 1, true})
			end
		end

			tinsert(f, "     ")

			local numObjectives = C_QuestLog.GetNumQuestObjectives(questID) or 0

			local questLogIndex = C_QuestLog.GetLogIndexForQuestID(questID);
			local completionText = GetQuestLogCompletionText(questLogIndex);
			if completionText then
				local color = HIGHLIGHT_FONT_COLOR
				tinsert(f, {"    "..QUEST_DASH .. completionText, color.r, color.g, color.b, true})
			end


			tinsert(f, function() QuestUtils_AddQuestTypeToTooltip(GameTooltip, questID, NORMAL_FONT_COLOR) end)






			local wayPointText = C_QuestLog.GetNextWaypointText(questID)

			local percent = nil

			local x, y = 0, 0

			if wayPointText then
				local color = HIGHLIGHT_FONT_COLOR

				if string.match(wayPointText, "%d+/%d+") then
					local text = string.match(wayPointText, "%d+/%d+")

					local a, b = strsplit("/", text)

					local a, b = tonumber(a), tonumber(b)

					x=x+a
					y=y+b

				end

				tinsert(f, {"    "..QUEST_DASH .. wayPointText, color.r, color.g, color.b, true})
			else
				for objectiveIndex = 1, numObjectives do
					local wayPointText, objectiveType, finished = GetQuestObjectiveInfo(questID, objectiveIndex, false)
					if ( wayPointText and #wayPointText > 0 ) then
						local color = finished and GRAY_FONT_COLOR or HIGHLIGHT_FONT_COLOR

						if string.match(wayPointText, "%d+/%d+") then
							local text = string.match(wayPointText, "%d+/%d+")

							local a, b = strsplit("/", text)

							local a, b = tonumber(a), tonumber(b)

							x=x+a
							y=y+b

						end

						tinsert(f, {"    "..QUEST_DASH .. wayPointText, color.r, color.g, color.b, true})
					end
				end
			end

			local percent = C_TaskQuest.GetQuestProgressBarInfo(questID) or ((x ~= y) and (y ~= 1)) and (x/y) * 100
			if ( percent ) then
				tinsert(f, function() GameTooltip_ShowProgressBar(GameTooltip, 0, 100, percent, PERCENTAGE_STRING:format(percent)) end)
			end
			tinsert(f, "     ")

			if IsAltKeyDown() then
				tinsert(f, "Left Click to temporarily ignore.")
				tinsert(f, "Right Click to restore temporary ignores.")
			elseif IsControlKeyDown() then
				tinsert(f, "Left Click to permanently ignore.")
				tinsert(f, "Right Click to restore permanent ignores.")
			else
				tinsert(f, "Left Click to view map.")

				if questID == (C_QuestSession.GetSuperTrackedQuest() or C_SuperTrack.GetSuperTrackedQuestID()) or (isTask == true) then
					tinsert(f, "Right Click to untrack.")
				else
					tinsert(f, "Right Click to track.")
				end
				tinsert(f, "Hold a modifier key for more options.")
			end

			tinsert(f, function(self)
				AddQuestRewardsToTooltip(GameTooltip, questID)
			end)

			return f
		end

		f.compass.icon = function()

			local x, y, mapID, questID = f.get()

			return _, _, _, _, _, _, _, _, _, questID
		end

		f.compass.shouldShow = function(self)
			local x, y, mapID, questID = f.get(self)
			if questID then

				if Q_C.lastWorldQuestID ~= questID then
			--		Q_C.wayStep = 10
				end
				Q_C.lastWorldQuestID = questID
				return {x, y, mapID}, "quest", questID
			end
		end

		f.notify.tooltip = function(self)

			wipe(tempTooltip)
			tinsert(tempTooltip, "World Quests")

				tinsert(tempTooltip, "Shift Click to share link.")
				tinsert(tempTooltip, "Left Click to announce that you are on this quest.")
				tinsert(tempTooltip, "Right Click to ask for help with this quest.")

			return tempTooltip
		end

		f.notify.icon = {.72, .5, .25,1}

		f.notify.click = function(self, btn)
			local x, y, mapID, questID = f.get(self)

			if IsShiftKeyDown() then
				local link = GetQuestLink(questID)
				if link and ChatEdit_GetActiveWindow() then
					ChatEdit_InsertLink(link)
				end
			elseif btn == "LeftButton" then
				--announce
				local link = GetQuestLink(questID)
				if link then
					Q_C:SendToChat("I am working on " ..link)
				end
			elseif btn == "RightButton" then
				--help
				local link = GetQuestLink(questID)
				if link then
							x = format("%.2f", x*100)
							y = format("%.2f", y*100)
					Q_C:SendToChat("I could use some help with " ..link..". I am at: "..x..", "..y)
				end
			end
		end

		f.notify.shouldShow = function(...)
			return f.compass.shouldShow(...)
		end

		f.get = function()
			--get the id for world or standard quests
			local questID

			local mapID = Q_C:GetPlayerMapID()

			if ((questID == 0) or (questID == nil)) then
				local id = Questra:GetNearestWorldQuestID(Q_C:GetSets().worldQuests.currentZone, mapID, shouldIgnore)

				if id then
					questID = id
				end
			end

			if not questID then
				return
			end

			local x, y, mapID = Questra:GetQuestLocation(questID)

			if (not mapID) or (not (questID and (x or y))) then
				return 0, 0, mapID
			end

			Q_C.questID = questID

			return x, y, mapID, questID
		end
		tinsert(way, 11, f)
	end

	tinsert(way, 5, GetHandler("portal", "Portal", "Portals", {Q_C.TexturePath.."notifySwirl", nil, 60, 60, "texture", .25, .75, .25, .75}, {}, strings.askaboutsuggest, "help", "rightClick", "Left click to ask if suggestion is at destination."))

	function Q_C.getFactionInfo(questID)
		if not questID then return end
		local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)

		if factionID == nil then
				local logIndex = C_QuestLog.GetLogIndexForQuestID(questID)

				title = C_QuestLog.GetTitleForLogIndex(logIndex)
				--shadowlands made extensive changes. I can't seem to find faction info for non world quests now.
			--title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)
		end

		return title, factionID, capped, factionID and GetFactionInfoByID(factionID)
	end
end

local old = C_TaskQuest.GetQuestProgressBarInfo
function C_TaskQuest.GetQuestProgressBarInfo(questID)
	if not questID then return end
	if (C_QuestLog.IsQuestTask(questID)) and old(questID) then
		return old(questID)
	end
	local wayPointText = C_QuestLog.GetNextWaypointText(questID)
	local x, y = 0, 0
	if wayPointText then
		if string.match(wayPointText, "%d+/%d+") then
			local text = string.match(wayPointText, "%d+/%d+")
			local a, b = strsplit("/", text)
			x = x + tonumber(a)
			y = y + tonumber(b)
		end
	else
		local numObjectives = C_QuestLog.GetNumQuestObjectives(questID) or 0
		for objectiveIndex = 1, numObjectives do
			local wayPointText = GetQuestObjectiveInfo(questID, objectiveIndex, false)
			if ( wayPointText and #wayPointText > 0 ) then
				if string.match(wayPointText, "%d+/%d+") then
					local text = string.match(wayPointText, "%d+/%d+")
					local a, b = strsplit("/", text)
					x = x + tonumber(a)
					y = y + tonumber(b)
				end
			end
		end
	end
	local value = ((x ~= y) and (y ~= 1)) and (x/y) * 100
	return ((value~=100) or (value~=100)) and value
end


local minutes, seconds, hours, dispHours, dispMinutes, dispSeconds

function convertTime(allSeconds)
	minutes = floor(allSeconds/60)
	seconds = allSeconds - (minutes * 60)
	hours = math.floor(minutes/60)

	local dispHours = (hours == 0) and "" or hours..":"

	dispMinutes = (hours ~= 0) and minutes - (hours * 60) or minutes
	dispMinutes = (dispMinutes == 0) and "" or (dispMinutes < 10) and "0"..dispMinutes or dispMinutes..":"

	dispSeconds = (seconds < 10) and "0"..seconds or seconds
	return dispHours..dispMinutes..dispSeconds
end

do -- navigation display updates
	local green, white = {r = .45, g = .8, b = 0}, {1, 1, 1}

	function Q_C:SetCompassDisplay(x, y, mapID, destX, destY, destMapID, info, isEvent) --info = {x, y, mapID, {r, g, b, a}, iconTexture, etc...}
		if ((destX == 0) and (destY == 0)) or ((x == 0) and (y == 0)) or ((x == nil) or (y == nil)) then return self:HideAll() end

		local viewedMapID, viewedWorldID = self:GetViewedMapID()
		local playerMapID, playerWorldID = self:GetPlayerMapID()

		--[[
			things are rigged to record dungeon entrance locations,
			so that when a player enters a dungeon, this system spoofs
			their location as the dungeon's exterior entrance.
		--]]

		local sameMap = destMapID == playerMapID or nil
		local sameWorld = viewedWorldID == playerWorldID or nil

		local worldPlayerX, worldPlayerY
		local worldDestX, worldDestY

		local viewedPlayerX, viewedPlayerY
		local viewedDestX, viewedDestY

		if sameWorld then
			worldPlayerX, worldPlayerY = Questra:TranslateCoords(x,     y,     mapID,     playerWorldID, true)
			worldDestX,   worldDestY =   Questra:TranslateCoords(destX, destY, destMapID, playerWorldID, true)

			-- origin and destination are on same map
			local viewingRandom = ((viewedMapID ~= playerMapID) and (viewedMapID ~= destMapID)) or nil
			local viewingOrigin = (viewedMapID == playerMapID)
			local viewingDest =   (viewedMapID == destMapID)

			if viewingRandom then
				viewedPlayerX, viewedPlayerY = Questra:TranslateCoords(worldPlayerX, worldPlayerY, playerWorldID, viewedMapID, true)
				viewedDestX,   viewedDestY   = Questra:TranslateCoords(destX,        destY,        destMapID,     viewedMapID, true)
			elseif viewingOrigin then
				viewedPlayerX, viewedPlayerY = x, y, mapID
				viewedDestX,   viewedDestY   = Questra:TranslateCoords(destX,        destY,        destMapID,     mapID, true)
			elseif viewingDest then
				viewedPlayerX, viewedPlayerY = Questra:TranslateCoords(worldPlayerX, worldPlayerY, playerWorldID, destMapID, true)
				viewedDestX,   viewedDestY   = destX, destY, destMapID
			end
		else
			--origin and destination are not on same world...
				--to do: reactivate portal recommendations!
		end

		if not self.Get then return end

		local _, _, _, questID = self.Get()

		if viewedDestX and viewedDestY and worldPlayerX and worldPlayerY then-- and (distance ~= 0) then
			--local text = tonumber(Questra.editor:GetText()) or mult

			--local distance = math.floor(math.sqrt(math.pow((worldPlayerX - worldDestX), 2) + math.pow((worldPlayerY - worldDestY), 2)) * mult)/100

			local distance = Q_C.range(mapID, x, y, destMapID, destX, destY, true)

	
			if  SuperTrackedFrame and SuperTrackedFrame.Icon and SuperTrackedFrame.Icon:IsVisible()then
			
				local scale = .5
				
				
				local scale = distance / 1000
			
			
				SuperTrackedFrame.Icon:SetScale(max(.1, 1.6 - (scale * 1.5)))
			end
			if  SuperTrackedFrame and SuperTrackedFrame.DistanceText and SuperTrackedFrame.DistanceText:IsVisible()then

				SuperTrackedFrame.DistanceText:Hide()
			
			
			end
			
			
			


			--distance = C_Navigation.GetDistance() or distance

			-- if questID then
				-- distance = C_QuestLog.GetDistanceSqToQuest(questID)
				-- if distance == 0 then
					-- distance = Q_C.range(mapID, x, y, destMapID, destX, destY)
				-- end

			-- end
			--debug code
				-- local di = math.floor(math.sqrt(math.pow((worldPlayerX - worldDestX), 2) + math.pow((worldPlayerY - worldDestY), 2)) * text)/100

				-- print(di)

				-- if di == 45 then
					-- self.sets.newDistADJ = self.sets.newDistADJ or {}
					-- self.sets.newDistADJ[self:GetPlayerMapID()] = text
				-- end

			self:ShowAll()
			local isInArea
			if questID then
				isInArea = GetTaskInfo(questID)
				local percent =  C_TaskQuest.GetQuestProgressBarInfo(questID)
				self.percent:SetText(percent and math.floor(percent).."%" or "")

				if ( QuestUtils_ShouldDisplayExpirationWarning(questID) ) then
					local allSeconds = C_TaskQuest.GetQuestTimeLeftSeconds(questID)
					local info = questID and C_QuestLog.GetQuestTagInfo(questID)
						-- .quality
						-- .isElite
						-- .worldQuestType
						-- .tagID
						-- .displayExpiration
						-- .tagName

					local index, professionIndex, rarity, displayTimeLeft = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration




					if allSeconds then-- <= 1200 then

						self.timeLeft:SetText(convertTime(allSeconds))
						self.timeLeftbg:Show()

						local color = (allSeconds > 1200) and green or (allSeconds > 300) and OBJECTIVE_TRACKER_COLOR["Header"] or (allSeconds >= 300) and OBJECTIVE_TRACKER_COLOR["Failed"] or white

						self.timeLeft:SetTextColor(color.r, color.g, color.b)
					else
						self.timeLeft:SetText("")
						self.timeLeftbg:Hide()
					end
				else
					self.timeLeft:SetText("")
					self.timeLeftbg:Hide()
				end
			else
				self.timeLeft:SetText("")
				self.timeLeftbg:Hide()
			end
			
			local inrange = C_Navigation.GetTargetState() == Enum.NavigationState.Invalid or nil

				
			--if C_Navigation.WasClampedToScreen() and (self:GetSets().display.showDist == true) and not inrange then --and isInArea ~= true then
			if (self:GetSets().display.showDist == true) and isInArea ~= true then --and isInArea ~= true then
				self.distText:SetText(IN_GAME_NAVIGATION_RANGE:format(Round(distance)))
			else
				self.distText:SetText("")
			end

			if IsInInstance() ~= true then
				self:SetArrow(distance, destX, destY, destMapID, isInArea == true, questID)
			else
				self.arrow:Hide()
			end
			local xDist = viewedDestX - viewedPlayerX

			local angle = (90 - acos(xDist/(math.sqrt(math.pow((xDist), 2) + math.pow((viewedDestY - viewedPlayerY), 2))))) * math.pi /180

			self:SetLine(self:GetCanvas(), angle, viewedPlayerX, viewedPlayerY, viewedDestX, viewedDestY)

			--set pin texture according to waypoint type
			if (not InCombatLockdown()) and (not self.pin:IsShown()) then
				self.pin:Show()
			end

			if not isEvent then return end

			local questType, rarity, tradeIndex, complete = self.waypointType, 1, nil, nil

			self:UpdatePinTextures(questType, rarity or 1, tradeIndex, complete)
			self:ShowItemButton(questID)
			self:ShowGroupButton(questID)
		else

			if self.waypointLine then
				self.waypointLine:Hide()
			end
			self.arrow:Hide()
			self.distText:SetText("")
			return
		end
	end

	function Q_C:SetArrow(distance, destX, destY, destMapID, hideArrow, questID)
	--	local hideArrow = nil
			local navFrame = C_Navigation.GetFrame()

			local blizArrow = SuperTrackedFrame.Arrow
			local icon = SuperTrackedFrame.Icon
	
		if (hideArrow == true) then
			self.arrow:Hide()
				if blizArrow then blizArrow:SetAlpha(1) end
				--if icon then icon:SetAlpha(1) end
		else
		

			--if  distance > 999 then
				
				self.arrow:Show()
				if blizArrow then blizArrow:SetAlpha(0) end
				--if icon then icon:SetAlpha(0) end
			--else
				
				
				--if blizArrow then blizArrow:SetAlpha(1) end
				--if icon then icon:SetAlpha(1) end
				--self.arrow:Hide()
			--end
		
			local angle, oAngle = Questra:GetDirectionToQuest(questID, destX, destY, destMapID)
			self.arrow:SetRotation(angle)
			
			SuperTrackedFrame:SetEllipticalRadii(400, 400)
			
		end
	end

	function Q_C:SetLine(canvas, angle, viewed_pX, viewed_pY, viewed_wX, viewed_wY)
		if self:GetSets().display.waypointLine == true then

			if self.waypointLine then
				do
					--using these cheaters help to reduce intense maths.
					self.playerLocationHandler:SetPoint("TopLeft", WorldMapFrame)
					self.playerLocationHandler:SetPoint("BottomRight", canvas, viewed_pX, viewed_pY)
					self.waypointLocationHandler:SetPoint("TopLeft", WorldMapFrame)
					self.waypointLocationHandler:SetPoint("BottomRight", canvas, viewed_wX, viewed_wY)

					local wayW, wayH = self.waypointLocationHandler:GetSize()
					local playW, playH = self.playerLocationHandler:GetSize()

					if math.sqrt(math.pow((playW - wayW), 2) + math.pow((playH - wayH), 2)) < .03 then
						self.waypointLine:Hide()
						self.waypointMarker:Hide()
						return
					else
						self.waypointLine:Show()
						self.waypointMarker:Show()
					end
				end

				local cWidth, cHeight = canvas:GetSize()
				local scale = canvas:GetScale()

				self.waypointLine:SetThickness(3/scale)

				local dist = 10 / scale

				local pAngle = -(math.deg(angle ) - 90) * (math.pi/180) --in degrees

				local wAngle =  -(math.deg(angle ) - 90) * (math.pi/180)

				 if viewed_pY < viewed_wY then
					pAngle = -wAngle
					wAngle = pAngle
				end

				local pX = (((cWidth ) * (viewed_pX)) + 2.5) + math.cos(pAngle) * dist
				local pY = (cHeight-((cHeight) * (viewed_pY))+1.25) + math.sin(pAngle )* dist

				local _wX = viewed_wX * cWidth
				local _wY = cHeight - (viewed_wY * cHeight)

				local wX = (_wX) - math.cos(wAngle) * dist
				local wY = _wY - math.sin(wAngle )* dist

				self.waypointLine:SetStartPoint("BottomLeft", canvas, pX, pY)
				self.waypointLine:SetEndPoint("BottomLeft", canvas, wX, wY)

				--if (self.waypointType == "userSelected") or (self.waypointType == "shared") then
					self.waypointMarker:Show()
					self.waypointMarker:SetPoint("Center", canvas, "BottomLeft", _wX, _wY)
					self.waypointMarker:SetSize(15 / canvas:GetScale(), 15 / canvas:GetScale())
				--else
				--	self.waypointMarker:Hide()
				--end
			end
		elseif self.waypointLine then
			self.waypointMarker:Hide()
			self.waypointLine:Hide()
		end
	end

	function Q_C:HideAll()
		if self.waypointLine then
			self.waypointLine:Hide()
			self.waypointMarker:Hide()
		end
		self.distText:SetText("")
		self.arrow:Hide()
		self.pin:Hide()
		self.timeLeftbg:Hide()
	end

	function Q_C:ShowAll()
		if InCombatLockdown() or issecure() then
			return
		end

		if self.waypointLine then
			self.waypointLine:Show()
			self.waypointMarker:Show()
		end
		self.arrow:Show()
		if (not InCombatLockdown()) and (not self.pin:IsShown()) then
			self.pin:Show()
			self.timeLeftbg:Show()
		end
	end

	local iconDisplayInfo
	iconDisplayInfo = {
		[Enum.QuestTagType.PvP] = function() return "worldquest-icon-pvp-ffa", true, nil, nil, "atlas" end,

		[Enum.QuestTagType.PetBattle] = function() return "worldquest-icon-petbattle", true, nil, nil, "atlas" end,

		[Enum.QuestTagType.Profession] = function(tradeskillLineID) return WORLD_QUEST_ICONS_BY_PROFESSION[tradeskillLineID], true, nil, nil, "atlas" end,

		[Enum.QuestTagType.Dungeon] = function() return 2909778, true, 80, 80, "texture" end,

		[Enum.QuestTagType.Raid] = function() return "worldquest-icon-raid", true, nil, nil, "atlas" end,

		[Enum.QuestTagType.Invasion] = function() return "worldquest-icon-burninglegion", true, nil, nil, "atlas" end,

		[Enum.QuestTagType.Threat] = function() return "worldquest-icon-nzoth", true, nil, nil, "atlas" end,

		[Enum.QuestTagType.Islands] = function() return "poi-islands-table", true, nil, nil, "atlas" end,

		[Enum.QuestTagType.FactionAssault] = function()
			local faction = UnitFactionGroup("player")
			if faction == "Horde" then
				return "QuestPortraitIcon-Horde-small", true, 60, 60, "atlas"
			elseif faction == "Alliance" then
				return "QuestPortraitIcon-Alliance-small", true, 45, 45, "atlas"
			end

			return iconDisplayInfo[faction]
		end,

		["bang"] = function() return "worldquest-questmarker-questbang", nil, 18, 45, "atlas" end,

		["question"] = function() return "worldquest-questmarker-questionmark", nil, 25, 45, "atlas" end,
	}

	local complete = {}

	local function IsComplete(questID)
		return ((C_QuestLog.IsComplete(questID) == true)
		or (C_QuestLog.IsQuestFlaggedCompleted(questID) == true)
		or (IsQuestFlaggedCompleted(questID) == true)
		or (C_QuestLog.GetLogIndexForQuestID(questID) and select(6, {GetQuestLogTitle(C_QuestLog.GetLogIndexForQuestID(questID))}) == true)
		or (GetQuestsCompleted(complete)[questID] == true)) and true or nil
	end

	Q_C.factions = {
		[1883] = {name = "Dreamweavers", icon = 1394953,},
		[2373] = {name = "The Unshackled", icon = 2909044,},
		[2158] = {name = "Voldunai", icon = 2203920,},
		[2164] = {name = "Champions of Azeroth", icon = 2203913,},
		[2045] = {name = "Armies of Legionfall", icon = 1708507,},
		[2170] = {name = "Argussian Reach",},
		[2391] = {name = "Rustbolt Resistance", icon = 2909316,},
		[1900] = {name = "Court of Farondis", icon = 1394952,},
		[2155] = {name = "7th Legion", icon = 2203912,},
		[2157] = {name = "The Honorbound", icon = 2203914,},
		[1948] = {name = "Valarjar", icon = 1394957,},
		[2163] = {name = "Tortollan Seekers", icon = 2203919,},
		[2165] = {name = "Army of the Light",},
		[2415] = {name = "Rajani",},
		[2417] = {name = "Uldum Accord",},
		[1894] = {name = "The Wardens", icon = 1394958,},
		[1859] = {name = "The Nightfallen", icon = 1394956,},
		[1828] = {name = "Highmountain Tribe", icon = 1394954,},
		[2103] = {name = "Zandalari Empire", icon = 2203921,},
		[2156] = {name = "Talanji's Expedition", icon = 2203918,},
		[2135] = {name = "Chromie",},
	}

	do --Texture Finder
		-- local f = CreateFrame("Frame",nil, UIParent)
		-- f:SetPoint("Top", 0, -100)
		-- f:SetSize(1000,500)

		-- local textures = {}

		-- local index = 25500
		-- local index = 29093167
		-- for j = 1,5 do
			-- for i = 1, 10 do
				-- local texture = f:CreateTexture(nil, "ARTWORK")
				-- texture:SetSize(100,100)
				-- texture:SetPoint("TopLeft", 110*(i-1), -110*(j-1))
				-- local text = f:CreateFontString(nil, "ARTWORK", "GameFontNormal")
				-- text:SetPoint("Top", texture, "Bottom")
				-- texture.text = text
					-- texture:SetTexture(index + #textures)
					-- texture.text:SetText(index + #textures)
				-- tinsert(textures, texture)
			-- end
		-- end

		-- f:SetScript("OnMouseWheel", function(_, scroll)
			-- index = index + (scroll * 50)

			-- for i, texture in pairs(textures)do
				-- texture:SetTexture(index + (i-1))
				-- texture.text:SetText(index + (i-1))

			-- end
		-- end)
	end


local once

	function Q_C:UpdatePinTextures()
		local texture, useAtlasSize, w, h, style, l, r, t, b

		if self.navigation.icon then
			texture, useAtlasSize, w, h, style, l, r, t, b, questID = self.navigation.icon(self)
		end

		local complete, questType, skipTexture
		local rarity --= 1
		if questID then
			local info = questID and C_QuestLog.GetQuestTagInfo(questID)
				-- .quality
				-- .isElite
				-- .worldQuestType
				-- .tagID
				-- .displayExpiration
				-- .tagName

			local index, professionIndex, rarity, displayTimeLeft = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration

			local complete =  C_QuestLog.IsComplete(questID)

			local display = (complete and iconDisplayInfo["question"] or iconDisplayInfo[professionIndex] or iconDisplayInfo["bang"])

			texture, useAtlasSize, w, h, style, l, r, t, b = display(professionIndex)

			if Quest_CompassDB.worldQuests.useFaction then
				local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)

				if not factionID then
					titleam, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)
				end

				if self.factions[factionID] and self.factions[factionID].icon then
					SetPortraitToTexture(self.pin.Faction, self.factions[factionID].icon)
					self.pin.Faction:SetTexCoord(l or 0, r or 1, t or 0, b or 1)
					self.pin.Faction:SetSize(40,40)
					self.pin.Texture:SetTexture("")
					skipTexture = true
				else
					self.pin.Faction:SetTexture("")
				end
			else

				self.pin.Faction:SetTexture("")
			end

		else
			self.pin.Faction:SetTexture("")
		end

		local normal = rarity == 1 and "Interface/WorldMap/UI-QuestPoi-NumberIcons" or rarity == 2 and "worldquest-questmarker-rare" or "worldquest-questmarker-epic"
		local pushed = rarity == 1 and "Interface/WorldMap/UI-QuestPoi-NumberIcons" or rarity == 2 and "worldquest-questmarker-rare-down" or "worldquest-questmarker-epic-down"

		local normalCoords = rarity == 1 and {218/256, 198/256, 122/256, 103/256} or {0, 1, 0, 1}
		local pushedCoords = rarity == 1 and {198/256, 218/256, 103/256, 122/256} or {0, 1, 0, 1}

		if rarity == 1 then
			self.pin.normalTexture:SetTexture(normal)
			self.pin.pushedTexture:SetTexture(pushed)
		else
			self.pin:SetNormalAtlas(normal)
			self.pin:SetPushedAtlas(pushed)
		end


		self.pin.normalTexture:Show()
		self.pin.normalTexture:SetTexCoord(unpack(normalCoords))
		self.pin.pushedTexture:SetTexCoord(unpack(pushedCoords))

		if not skipTexture then
			if style == "texture" then
				self.pin.Texture:SetTexture(texture)
				if self.waypointMarker then
					self.waypointMarker:SetTexture(texture)

				end
			elseif style == "portrait" then
				SetPortraitToTexture(self.pin.Texture, texture)
				if self.waypointMarker then
					SetPortraitToTexture(self.pin.Texture, self.waypointMarker)
				end
			else
				self.pin.Texture:SetAtlas(texture, useAtlasSize)
				if self.waypointMarker then
					self.waypointMarker:SetAtlas(texture, useAtlasSize)
					self.waypointMarker:SetTexCoord(l or 0, r or 1, t or 0, b or 1)
				end
			end
		end
		local W, H = (60-(w or 40))/60, (60-(h or 40))/60
		local pW, pH = self.pin.normalTexture:GetSize()
		local x, y = (pW* W)/2, (pH * H)/2

		self.pin.Texture:ClearAllPoints()
		self.pin.Texture:SetPoint("TopLeft", self.pin.normalTexture, x,-y)
		self.pin.Texture:SetPoint("BottomRight", self.pin.normalTexture, -x,y)

		self.pin.Texture:SetTexCoord(l or 0, r or 1, t or 0, b or 1)

		if self.waypointMarker then
			self.waypointMarker:SetTexCoord(l or 0, r or 1, t or 0, b or 1)
		end

		self.arrow:SetDrawLayer("ARTWORK", 7)
		self.pin.Texture:SetDrawLayer("ARTWORK", 1)
		self.pin.border:SetDrawLayer("OVERLAY", 5)
		if (not InCombatLockdown()) and (not self.pin:IsShown()) then
			self.pin:Show()
			if self.waypointMarker and not questID then
				self.waypointMarker:Show()
			elseif self.waypointMarker then
				self.waypointMarker:Hide()
			end
		end
		return
	end



	local oldold = C_LFGList.CanCreateQuestGroup
	local function CanCreateQuestGroup(questID, ...)
		local o = oldold(questID, ...)
		if o then
			return o
		end
							local info = questID and C_QuestLog.GetQuestTagInfo(questID)
						-- .quality
						-- .isElite
						-- .worldQuestType
						-- .tagID
						-- .displayExpiration
						-- .tagName

					local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName

		if worldQuestType == 6 then
			return true, true
		end
	end

	local op = {
		Left = "Right",
		Right = "Left",
		Bottom = "Top",
		Top = "Bottom",
	}

	local categories = {}
	local function GetQueueInfo(index)
		local lfgDungeonIDs = C_LFGInfo.GetAllEntriesForCategory(LE_LFG_CATEGORY_LFD)

		local total = 0
		for i = 1, 7 do
			local ids = C_LFGInfo.GetAllEntriesForCategory(i)
			categories[i] = categories[i] or {}
			wipe(categories[i])
			tinsert(categories[i], ids)
			total = total + #ids
		end

		return id and categories[id] or categories, id and #categories[id] or total
	end

	local function QuickJoinToastMixin_OnEnter(button, questID, queued)
		if not questID then
			return --print(button, questID)
		end
		local canMake = C_LFGList.CanCreateQuestGroup(questID)

		--local ids, numIDs = GetQueueInfo(index)

		local looking = queued

		if  looking or canMake and looking then
			QueueStatusMinimapButton:GetScript("OnEnter")(button)


			local x, y = button:GetCenter()
			local w, h = UIParent:GetSize()
			local half_w, half_h = w/2, h/2
			local ver, hor = "Bottom", "Left"

			if not (y and x and half_h and half_w) then return end

			if y >= half_h then
				ver = "Top"
			end
			if x >= half_w then
				hor = "Right"
			end
			QueueStatusFrame:ClearAllPoints()
			QueueStatusFrame:SetPoint(ver..hor, button, op[ver]..op[hor])

			local eScale = QueueStatusFrame:GetEffectiveScale()
			if eScale ~= 1 then
				local scale = 1
				self.scale = scale;  -- Saved in case it needs to be re-calculated when the parent's scale changes.
				local parent = QueueStatusFrame:GetParent();
				if ( parent ) then
					scale = scale / parent:GetEffectiveScale()
				end
				QueueStatusFrame:SetScale(scale);
			end
			QueueStatusFrame:SetFrameStrata("TOOLTIP")
			QueueStatusFrame:SetFrameLevel(100)
		else
			GameTooltip:SetOwner(button, "ANCHOR_RIGHT");
			GameTooltip:SetText(button.tooltip or "")
			GameTooltip:AddLine(button.tooltip2 or "")
			GameTooltip:Show()
		end
	end

	local function GetInstanceIDByName(zoneID, name)
		local dungeonEntrances = C_EncounterJournal.GetDungeonEntrancesForMap(zoneID)
		for i, b in pairs(dungeonEntrances) do
			if b.name == name then
				return b.journalInstanceID
			end
		end
	end

local ls
-- local hooky
-- function EyeTemplate_OnUpdate(self, elapsed)
	-- local textureInfo = LFG_EYE_TEXTURES[self.queueType or "default"];
	-- AnimateTexCoords(self.texture, textureInfo.width, textureInfo.height, textureInfo.iconSize, textureInfo.iconSize, textureInfo.frames, elapsed, textureInfo.delay)
	-- if hooky then
		-- AnimateTexCoords(hooky.texture, textureInfo.width, textureInfo.height, textureInfo.iconSize, textureInfo.iconSize, textureInfo.frames, elapsed, textureInfo.delay)
	-- end
-- end



local function shouldShowAndOrAnimate()
	local self = QueueStatusFrame
	local animateEye;
	local nextEntry = 1;

	--Try each LFG type
	for i=1, NUM_LE_LFG_CATEGORYS do
		local mode, submode = GetLFGMode(i);
		if ( mode and submode ~= "noteleport" ) then
			nextEntry = nextEntry + 1;
			if ( mode == "queued" ) then
				animateEye = true;
			end
		end
	end

	--Try LFGList entries
	local isActive = C_LFGList.HasActiveEntryInfo();
	if ( isActive ) then
		nextEntry = nextEntry + 1;
		animateEye = true;
	end

	--Try LFGList applications
	local apps = C_LFGList.GetApplications();
	for i=1, #apps do
		local _, appStatus = C_LFGList.GetApplicationInfo(apps[i]);
		if ( appStatus == "applied" or appStatus == "invited" ) then
			nextEntry = nextEntry + 1;
			if ( appStatus == "applied" ) then
				animateEye = true;
			end
		end
	end

	local inProgress, _, _, _, _, isBattleground = GetLFGRoleUpdate();
	--Try PvP Role Check
	if ( inProgress and isBattleground ) then
		nextEntry = nextEntry + 1;
	end

	local readyCheckInProgress, readyCheckIsBattleground = GetLFGReadyCheckUpdate();

	-- Try PvP Ready Check
	if ( readyCheckInProgress and readyCheckIsBattleground ) then
		nextEntry = nextEntry + 1;
	end

	--Try all PvP queues
	for i=1, GetMaxBattlefieldID() do
		local status, mapName, teamSize, registeredMatch, suspend = GetBattlefieldStatus(i);
		if ( status and status ~= "none" ) then
			nextEntry = nextEntry + 1;
			if ( status == "queued" and not suspend ) then
				animateEye = true;
			end
		end
	end

	--Try all World PvP queues
	for i=1, MAX_WORLD_PVP_QUEUES do
		local status, mapName, queueID = GetWorldPVPQueueStatus(i);
		if ( status and status ~= "none" ) then
			nextEntry = nextEntry + 1;
			if ( status == "queued" ) then
				animateEye = true;
			end
		end
	end

	--World PvP areas we're currently in
	if ( CanHearthAndResurrectFromArea() ) then
		nextEntry = nextEntry + 1;
	end

	--Pet Battle PvP Queue
	local pbStatus = C_PetBattles.GetPVPMatchmakingInfo();
	if ( pbStatus ) then
		nextEntry = nextEntry + 1;
		if ( pbStatus == "queued" ) then
			animateEye = true;
		end
	end

	local showButton, animate
	--Update the minimap icon
	if ( nextEntry > 1 ) then
		showButton = true
		--QueueStatusMinimapButton:Show();
	else
		--QueueStatusMinimapButton:Hide();
	end
	if ( animateEye ) then
		animate = true
		--EyeTemplate_StartAnimating(QueueStatusMinimapButton.Eye);
	else
		--EyeTemplate_StopAnimating(QueueStatusMinimapButton.Eye);
	end

	if not animate then
		--All LFG types
		local queued
		for i=1, NUM_LE_LFG_CATEGORYS do
			local mode, submode = GetLFGMode(i);
			local entry = QueueStatusFrame_GetEntry(QueueStatusFrame, i)
			if mode then
				showButton = true
				animate = true
				break
			end
		end
	end
	
	return showButton, animate
end



	local accept
	local dungeonTitle = ""
	local _text = ""
	_G.StaticPopupDialogs.ConfirmDungeonEntry = {
		text = _text,
		button1 = ACCEPT,
		button2 = CANCEL,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		--hasEditBox = true,
		OnAccept = function(self, data)
			if accept then
				accept()
			end
		end,
		OnCancel = function(self, data)
			
		end,

		--enterClicksFirstButton = true,
		--OnCancel = function() show() end, -- Need to wrap it so we don't pass |self| as an error argument to show().
		preferredIndex = STATICPOPUP_NUMDIALOGS,
	}


local function GetDisplayNameFromCategory(category)
	if (category == LE_LFG_CATEGORY_BATTLEFIELD) then
		local brawlInfo;
		if (C_PvP.IsInBrawl()) then
			brawlInfo = C_PvP.GetActiveBrawlInfo();
		else
			brawlInfo = C_PvP.GetAvailableBrawlInfo();
		end
		if (brawlInfo and brawlInfo.canQueue and brawlInfo.name) then
			return brawlInfo.name;
		end
	end

	if (category == LE_LFG_CATEGORY_SCENARIO) then
		local scenarioIDs = C_LFGInfo.GetAllEntriesForCategory(category)
		for i, scenID in ipairs(scenarioIDs) do
			if (not C_LFGInfo.HideNameFromUI(scenID)) then
				local instanceName = GetLFGDungeonInfo(scenID);
				if(instanceName) then
					return instanceName;
				end
			end
		end
	end

	return LFG_CATEGORY_NAMES[category];
end

	local on, off = {1,1,1,1}, {1,0,0,1}
	local canCreateGroup = {}
	function Q_C:ShowGroupButton(questID)
		local queued = shouldShowAndOrAnimate()
		

		
		canCreateGroup[questID or 0] = questID and canCreateGroup[questID] or questID and CanCreateQuestGroup(questID) or false
		local state = UnitInAnyGroup("player") == true and true or canCreateGroup[questID]
		if state and not self.groupButton then
			self.groupButton = CreateFrame("BUTTON", "Q_C-GroupFinder", self, "Q_C-EyeHighlightAnim");
			self.groupButton:SetHitRectInsets(3, 25 , 30, 30)
			self.groupButton:ClearAllPoints()
			self.groupButton:SetFrameStrata("LOW")
			self.groupButton:SetPoint("Top", self.pin.border, 0, -3)
			self.groupButton:SetPoint("Bottom", self.pin.border)
			self.groupButton:SetPoint("Right", self.pin.border, "Center")
			self.groupButton:SetPoint("Left", self.pin.border)

			local w, h = self.pin:GetSize()

			self.groupButton:SetSize(w/2,h)
			self.groupButton:SetNormalTexture(self.TexturePath.."item")
			self.groupButton:GetNormalTexture():SetTexCoord(1,0,1,0)
			self.groupButton:SetPushedTexture(self.TexturePath.."item-pushed")
			self.groupButton:GetPushedTexture():SetTexCoord(1,0,1,0)
			--self.groupButton:SetHighlightTexture(self.TexturePath.."lfg-highlight")

			self.groupButton.icon = self.groupButton:CreateTexture(nil, "OVERLAY")
			self.groupButton.Eye = self.groupButton.icon
			self.groupButton.icon:SetPoint("Right", self.groupButton, "Center", 10,0)
			self.groupButton.icon:SetSize(35, 35)
			self.groupButton.icon:SetTexture("Interface\\LFGFrame\\LFG-Eye")

			self.groupButton.Highlight:SetPoint("Center", self.groupButton.icon)

			local textureInfo = LFG_EYE_TEXTURES["default"]
			self.groupButton.icon:SetTexCoord(0, textureInfo.iconSize / textureInfo.width, 0, textureInfo.iconSize / textureInfo.height);

			self.groupButton.texture=self.groupButton.icon
			--EyeTemplate_OnUpdate(self.groupButton, 1)
			self.groupButton:RegisterForClicks("AnyUp")
			self.groupButton:SetScript("OnClick", function(self, btn, ...)

				self.DropDown = self.DropDown or QueueStatusMinimapButton.DropDown
				if not InCombatLockdown() then
					if shouldShowAndOrAnimate() then
						QueueStatusMinimapButton_OnClick(self, btn, ...)
						local p1, p2 = Q_C.SelectProperSide(self)
						DropDownList1:ClearAllPoints()
						DropDownList1:SetPoint(p2, self, p1)
					elseif btn == "RightButton" then
						return C_PartyInfo.LeaveParty()
					elseif UnitInAnyGroup() then
						return C_PartyInfo.LeaveParty()
					else
						if PVEFrame:IsShown() == true then
						HideUIPanel(PVEFrame)
							--PVEFrame:Hide()
						else
							local info = self.questID and C_QuestLog.GetQuestTagInfo(self.questID)
							if info and WORLD_QUEST_TYPE_DUNGEON_TYPES[info.worldQuestType] then
								local x,y, zoneID = Questra:GetQuestLocation(self.questID, true)
								local title, factionID, capped, factionName = Q_C.getFactionInfo(self.questID)
								local name = string.split(":", title )
								local dungeonID = GetInstanceIDByName(zoneID, name)

								_G.StaticPopupDialogs.ConfirmDungeonEntry.text = "Are you sure you want to queue for "..name.."?"
								accept = function()
									if dungeonID then
										LFDFrame_DisplayDungeonByID(dungeonID);
										PVEFrame_ShowFrame("GroupFinderFrame", LFDParentFrame);
										do--can i auto-queue? lets find out!
											local list = {}
											local count = 0
											local last
											for i = 1, NUM_LFD_CHOICE_BUTTONS do
												local button = _G["LFDQueueFrameSpecificListButton"..i];
												if GetLFGDungeonInfo(button.id) == name then
													count = count + 1
													if count == 2 and last then
														tremove(list, 1)
														LFGDungeonList_SetDungeonEnabled(last, false)
													end
													tinsert(list, button.id)
													LFGDungeonList_SetDungeonEnabled(button.id, true)
													last = button.id
												end
											end
											if #list > 0 then
												LFDQueueFrame_Join()
											end
											HideUIPanel(PVEFrame)
										end--i can auto-queue! i fully expect Blizzard to kill this one day.
									end
								end
								StaticPopup_Show("ConfirmDungeonEntry")
							else
								local isFromGreenEyeButton = true;
								LFGListUtil_FindQuestGroup(self.questID, isFromGreenEyeButton)
							end
						end
					end
				end
			end)

			local high = self.groupButton:CreateTexture(nil, "HIGHLIGHT")
			high:SetTexture(self.TexturePath.."lfg-highlight")


			high:SetBlendMode("ADD")
			high:ClearAllPoints()

			high:SetAlpha(.75)
			high:SetPoint("Left", 1, 0)
			local w,  h = self.groupButton:GetSize()
			high:SetSize(w, h)
			self.groupButton:HookScript("OnMouseDown", function()
				self.groupButton.icon:SetSize(25, 45)
				self.groupButton.icon:SetPoint("Right", self.groupButton, "Center", 7,0)
				high:SetPoint("Left", 5, 0)
				high:SetAlpha(.25)
			end)

			self.groupButton:HookScript("OnMouseUp", function()
				self.groupButton.icon:SetSize(35, 35)
				self.groupButton.icon:SetPoint("Right", self.groupButton, "Center", 10,0)
				high:SetPoint("Left", 1, 0)
				high:SetAlpha(.75)
			end)

			self.groupButton:SetScript("OnEnter", function()
				QuickJoinToastMixin_OnEnter(self.groupButton, self.groupButton.questID, shouldShowAndOrAnimate())
			end)

			self.groupButton:SetScript("OnLeave", function()
				GameTooltip:Hide()
				QueueStatusFrame:Hide()
			end)
			
			hooky = self.groupButton

			
		end


		local showGroup, shouldAnimate = shouldShowAndOrAnimate()
		
		if self.groupButton then

			
			if queued then					
				EyeTemplate_StartAnimating(self.groupButton)
			else
				EyeTemplate_StopAnimating(self.groupButton)
			end
			self.groupButton.glowLocks = QueueStatusMinimapButton.glowLocks
			QueueStatusMinimapButton_UpdateGlow(self.groupButton)
		end
		
		local textureInfo = LFG_EYE_TEXTURES["default"]
		if self.groupButton and not InCombatLockdown() then
			self.groupButton.questID = questID
			local shouldShow = ((canCreateGroup[questID] == true) or (UnitInAnyGroup("player") == true) or (queued)) and true or nil
			self.groupButton.tooltip, self.groupButton.tooltip2  = (((canCreateGroup[questID] == true) and (not inGroup)) and (queued)) and "You are looking for a group." --and you need to be in a group for this quest.
			or ((canCreateGroup[questID] == true) and (not inGroup) and (self.groupButton.icon:SetTexCoord(0, textureInfo.iconSize / textureInfo.width, 0, textureInfo.iconSize / textureInfo.height) == nil)) and "This task suggests a group." --and you aren't in one yet.
			or ((UnitInAnyGroup("player") == true) and (self.groupButton.icon:SetTexCoord(0, textureInfo.iconSize / textureInfo.width, 0, textureInfo.iconSize / textureInfo.height) == nil)) and "You are in a group, but this task does not require one", --some quests won't provide credit if you are in the wrong type of group...
			(((canCreateGroup[questID] == true) and (not inGroup)) and (queued)) and "Be patient..."
			or ((canCreateGroup[questID] == true) and (not inGroup)) and "Left click to find one."
			or (UnitInAnyGroup("player") == true) and "Right click to leave the group."
			
			self.groupButton:GetNormalTexture():SetVertexColor(unpack(((canCreateGroup[questID] == true) and (not inGroup)) and on or off))
			self.groupButton:GetPushedTexture():SetVertexColor(unpack(((canCreateGroup[questID] == true) and (not inGroup)) and on or off))

			shouldShow = shouldShow or showGroup

			local d = (((UnitInAnyGroup("player") == true) or shouldShow) and not self.groupButton:IsShown()) and self.groupButton:Show() or (self.groupButton:IsShown() and not shouldShow) and self.groupButton:Hide() --why not?
		
		
			if self.groupButton:IsShown() then
				QueueStatusMinimapButton:Hide()
			end
		
			return d
		end
		queued = nil
	end

	function Q_C:ShowItemButton(questID)
		if questID and C_QuestLog.GetLogIndexForQuestID(questID) and GetQuestLogSpecialItemInfo(C_QuestLog.GetLogIndexForQuestID(questID)) and not self.itemButton then
			self.itemButton = CreateFrame("Button", addonName.."_ItemButton", self.coordText, "Quest_CompassItemButtonTemplate");
			self.itemButton:ClearAllPoints()

			self.itemButton:SetFrameStrata("BACKGROUND")
			self.itemButton:SetPoint("Top", self.pin, 0, 25)
			self.itemButton:SetPoint("Bottom", self.pin, 0, -25)
			self.itemButton:SetPoint("Left", self.pin, "Center")
			self.itemButton:SetPoint("Right", self.pin, 22.5, 0)
			self.itemButton:SetParent(self.frame)
			self.itemButton:SetHitRectInsets(0, 1, 25, 25)

			self.itemButton.icon:SetDrawLayer("ARTWORK")
			self.itemButton:GetNormalTexture():SetDrawLayer("BACKGROUND")
			self.itemButton:GetPushedTexture():SetDrawLayer("BACKGROUND")

			self.itemButton:EnableMouse(true)
			self.itemButton.Cooldown:SetAllPoints(self.itemButton.icon)

			self.itemButton:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetItemByID(self:GetID())
			end)

			local high = self.itemButton:GetHighlightTexture()
			high:SetBlendMode("ADD")
			high:ClearAllPoints()
			self.itemButton.icon:ClearAllPoints()
			self.itemButton.icon:SetPoint("Center", self.itemButton, "Right", -18,0)
			self.itemButton.HotKey:SetPoint("Center", 14, -15.5)

			high:SetAlpha(.75)
			high:ClearAllPoints()
			high:SetPoint("Right", -1, 0)
			local w,  h = self.itemButton:GetSize()
			high:SetSize(w, h)
			self.itemButton:HookScript("OnMouseDown", function()
				self.itemButton.icon:SetSize(15, 30)
				self.itemButton.icon:SetPoint("Center", self.itemButton, "Right", -20,0)
				high:SetPoint("Right", -5, 0)
				self.itemButton.HotKey:SetPoint("Center", 12, -16)

				high:SetAlpha(.25)
			end)

			self.itemButton:HookScript("OnMouseUp", function()
				self.itemButton.icon:SetSize(25, 25)
				self.itemButton.icon:SetPoint("Center", self.itemButton, "Right", -18,0)
				self.itemButton.HotKey:SetPoint("Center", 14, -15.5)
				high:SetPoint("Right", -1, 0)
				high:SetAlpha(.75)
			end)
		end

		if questID then

			local questLogIndex = C_QuestLog.GetLogIndexForQuestID(questID)
			if questLogIndex then
				local link, item, charges, showItemWhenComplete = GetQuestLogSpecialItemInfo(questLogIndex)
				if link then
					local itemID = GetItemInfoInstant(link)

					if IsItemInRange(link, "target") == true then
						self.itemButton.HotKey:SetVertexColor(1,1,1,1)
					else
						self.itemButton.HotKey:SetVertexColor(1,0,0,1)
					end

					local start, duration, enable = GetItemCooldown(itemID);
					if ( start ) then
						CooldownFrame_Set(self.itemButton.Cooldown, start, duration, enable);
						if ( duration > 0 and enable == 0 ) then
							SetItemButtonTextureVertexColor(self.itemButton, 0.4, 0.4, 0.4);
						else
							SetItemButtonTextureVertexColor(self.itemButton, 1, 1, 1);
						end
					end

					if self.itemButton.lastItem ~= item then
						local itemID = GetItemInfoInstant(link)
						self.itemButton.charges = charges;
						self.itemButton.rangeTimer = -1;
						SetItemButtonTexture(self.itemButton, item);
						SetItemButtonCount(self.itemButton, charges);

						if not InCombatLockdown() then
							self.itemButton:SetAttribute('type', 'macro')
							self.itemButton:SetAttribute('macrotext', '/use ' .. GetItemInfo(itemID))
							self.itemButton:SetID(itemID)
						end
						SetPortraitToTexture(self.itemButton.icon, self.itemButton.icon:GetTexture())

						self.itemButton.lastItem = item
					end
					if not InCombatLockdown() and not self.itemButton:IsShown() then
						self.itemButton:Show()
					end
				end
			end
		else
			if self.itemButton and not InCombatLockdown() and self.itemButton:IsShown() then
				self.itemButton:Hide()
			end
		end
	end
end

do -- settings management
	Q_C.defaults = {
		rareLocations = {},
		point = "Center",
		ignoreQuestTypes = {},
		scale = 0.7500000000000001,
		layer = 4,
		ignored = {},
		worldQuests = {
			show = true,
			prioritizeTracked = true,
			useFaction = true,
			ignored = {},
			nearest = true,
		},
		anchorDist = 10,
		x = 0,
		version = 60,
		distance = 1,
		display = {
			anchorDist = 10,
			savedLocations = true,
			rareLocations = true,
			flightPoints = true,
			showDist = true,
			waypointLine = true,
		},
		savedLocations = {
		},
		y = -123.75,
		types = {
			corpse = true,
			nearestQuest = true,
			flight = true,
			quests = true,
			rares = true,
			added = true,
		},
		portals = {
			enable = true,
		},
	}

	Q_C.options = {

			{
				name = "Waypoint Types",
				style = "Menu",
				key = "types",
				options = {
					{
						name = "Corpse Location",
						key = "corpse",
						style = "Toggle",
					},
					{
						name = "Quest Location",
						key = "quests",
						style = "Toggle",
					},
					{
						name = "Nearest Quest",
						key = "nearestQuest",
						style = "Toggle",
					},
					{
						name = "Flight Master Locations",
						key = "flight",
						style = "Toggle",
					},
					{
						name = "Rare Creature Locations",
						key = "rareLocations",
						style = "Toggle",
					},
					{
						name = "Rare Creatures Share",
						key = "rares",
						style = "Toggle",
					},
					{
						name = "Flight Master Locations",
						key = "flight",
						style = "Toggle",
					},
					{
						name = "Saved Locations",
						key = "savedLocations",
						style = "Toggle",
					},
					{
						name = "Added by other addons",
						key = "added",
						style = "Toggle",
					},
					{
						name = "Prioritize Added",
						key = "priorityAdded",
						style = "Toggle",
					},

				},
			},

			{
				name = "World Quests",
				key = "worldQuests",
				style = "Menu",
				options = {

					{
						name = "Show Tracked",
						key = "show",
						style = "Toggle",
					},
					{
						name = "Prioritize Tracked",
						key = "prioritizeTracked",
						style = "Toggle",
					},
					{
						name = "Show Faction Icon when Possible",
						key = "useFaction",
						style = "Toggle",
					},
					{
						name = "Show Nearest",
						key = "nearest",
						style = "Toggle",
					},
					{
						name = "Current Zone Only",
						key = "currentZone",
						style = "Toggle",
					},
					{
						name = "Ignore",
						key = "ignored",
						style = "Menu",
						options = {
							"Battle Pet",
							"Dungeon",
							"PvP",
							"Profession",
							"Raid",
							"Invasion"
						},
					},

				},
			},

			{
				name = "Display",
				key = "display",
				style = "Menu",
				options = {
					{
						name = "Map Line",
						key = "waypointLine",
						style = "Toggle",
					},
					{
						name = "Flip",
						key = "flip",
						style = "Toggle",
					},
					{
						name = "show Distance",
						key = "showDist",
						style = "Toggle",
					},
				},
			},

			{
				name = "Distance Sensitivity",
				key = "distance",
				style = "Slider",
				low = 0,
				high = 100,
				displayAdjust = 1,
			},

			{
				name = "Release Anchor Lock Distance",
				key = "anchorDist",
				style = "Slider",
				low = 0,
				high = 30,
				displayAdjust = .01,
			},
			-- {
				-- name = "Frame layer",
				-- key = "layer",
				-- style = "Slider",
				-- low = 1,
				-- high = 8,
				-- cap = true,
			-- },
			{
				name = "Scale",
				key = "scale",
				style = "Slider",
				low = 25,
				high = 250,
				displayAdjust = 100,
				display = true,
			},

			{
				name = "Portal Recommendations",
				key = "portals",
				style = "Menu",
				options = {
					{
						name = "Enable",
						key = "enable",
						style = "Toggle",
					},
					{
						name = "Flip",
						key = "flip",
						style = "Toggle",
					},
					{
						name = "Reset Ignored",
						key = "reset",
						style = "Button",
						func = function()

						end,
						checked = function()end,
					},
				},
			},

		}

	local function GetOptionDetails(keyString)
		if not keyString then
			return Q_C.options
		end
		local keys = {string.split("-", keyString)}
		local optioninfo
		for index = 1, #keys do

			local key, optionTable = keys[index], optioninfo and optioninfo.options or Q_C.options

			for i, info in pairs(optionTable) do
				if info.key == key then
					optioninfo = info
				end
			end
		end
		return optioninfo
	end

	local optionsTable = {}

	function Q_C:GetSets(set, value)
		Quest_CompassDB = Quest_CompassDB or CopyTable(self.defaults)
		if set then
			if not Quest_CompassDB[set] then
				Quest_CompassDB[set] = value
			end
			return Quest_CompassDB[set]
		end
		return Quest_CompassDB
	end

	function Q_C:CreateDropDownMenu()
		if not self.dropdown then
			local dropdown = CreateFrame("Frame", "Quest_CompassOptionsMenu", self.coordText, "UIDropDownMenuTemplate")
			self.dropdown = dropdown
			UIDropDownMenu_Initialize( dropdown, function(_, level, ...) return self:InitializeMenu(level, ...) end, "MENU")
		end
		return self.dropdown
	end

	function Q_C:InitializeMenu(level, ...)
		if level == 1 then
			for i, inf in pairs(self:GetMenuOptions(level)) do
				if inf.dispCondition() then
					local info = UIDropDownMenu_CreateInfo()
					for j, b in pairs(inf) do
						info[j] = b
					end
					UIDropDownMenu_AddButton(info)
				end
			end
		else
			local inf = self:GetMenuOptions(level, UIDROPDOWNMENU_MENU_VALUE, ...)
			for i, inf in pairs(inf or {}) do
				local info = UIDropDownMenu_CreateInfo()
				for j, b in pairs(inf) do
					info[j] = b
				end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end

	function Q_C:GetMenuOptions(level, menuValue)
		if level == 1 then
			if not optionsTable[menuValue or level] then
				optionsTable[menuValue or level] = {}
				for key, info in pairs(GetOptionDetails(menuValue)) do
					local inf = {}
					inf.text = info.name
					if info.style == "Button" then
						inf.notCheckable = true
						inf.hasArrow = nil
						if not info.func then
							inf.func = function() return self:ToggleSetting(info.key) end
						else
							inf.func = function() return info.func(self) end
						end
						if not info.checked then
							inf.checked = function() return self:GetSetting(info.key) end
						else
							inf.checked = function() return info.checked(self) end
						end
					elseif info.style == "Toggle" then
						if not info.func then
							inf.func = function() return self:ToggleSetting(info.key) end
						else
							inf.func = function() return info.func(self) end
						end
						if not info.checked then
							inf.checked = function() return self:GetSetting(info.key) end
						else
							inf.checked = function() return info.checked(self) end
						end
					elseif info.style == "Slider" then
						inf.displayMode = "MENU"
						inf.hasArrow = 1
						inf.value = info.key
						inf.notCheckable = true
					elseif info.style == "Menu" then
						inf.displayMode = "MENU"
						inf.hasArrow = 1
						inf.value = info.key
						inf.notCheckable = true
						if info.subType == "Toggle" then
							inf.notCheckable = false
							if not info.func then
								inf.func = function() return self:ToggleSetting(info.keySetting) end
							else
								inf.func = function() return info.func(self) end
							end
							if not info.checked then
								inf.checked = function() return self:GetSetting(info.keySetting) end
							else
								inf.checked = function() return info.checked(self) end
							end
						end
					end
					inf.keepShownOnClick = true
					inf.dispCondition = function()
						if info.display then
						--	if not (placeHolder) then
								return true
						--	end
						else
							return true
						end
					end,
					tinsert(optionsTable[menuValue or level], inf)
				end
			end

			return optionsTable[menuValue or level], inf
		end
		if level and (level ~= 1) then
			if not optionsTable[menuValue or level] then
				optionsTable[menuValue or level] = {}
				local optionInfo = GetOptionDetails(menuValue)

				if optionInfo.style == "Slider" then
					do
						local keys
						if optionInfo.cap then
							keys = {-1, "x", 1}
						else
							keys = {-25,-10,-5, -1, -.5, -.1, "x", .1, .5, 1,5,10,25}
						end

						for i, number in pairs(keys) do
							local inf = {}
							if number == "x" then
								local adjust = self:GetAdjust(menuValue) or 1
								inf.text = self:GetSetting(menuValue) * adjust
								inf.func = function()
									if IsShiftKeyDown() then
										self:ResetSetting(menuValue)
										_G["DropDownList2Button"..i]:SetText(self:GetSetting(menuValue) * adjust)
									end
								end
							else
								if number > 0 then
									inf.text = "+"..number
								else
									inf.text = number
								end
								inf.func = function()
									local adjust = self:GetAdjust(menuValue) or 1
									local high, low = self:GetRange(menuValue)

									local set = self:GetSetting(menuValue)

									local value = (set * adjust) + number

									local text = 0

									if value > high or value < low then
										if value > high then
											self:UpdateSetting(menuValue, high/adjust)
											text = high
										elseif value < low then
											self:UpdateSetting(menuValue, low/adjust)
											text = low
										end
									else

										self:AdjustSetting(menuValue, number/adjust)
										text = value
									end

									if _G["DropDownList2Button7"]:IsShown() then
										_G["DropDownList2Button7"]:SetText(text)
									else
										_G["DropDownList2Button2"]:SetText(text)
									end
									self:Layout()
								end
							end

							inf.notCheckable = true
							inf.keepShownOnClick = true
							tinsert(optionsTable[menuValue or level], inf)
						end
					end
				elseif optionInfo.style == "Menu" then

					local opts = GetOptionDetails(menuValue).options

					for _, b in pairs(opts) do
						local inf = {}
					local args = {string.split("-", menuValue)}

						inf.text = type(b) == "table" and b.name or b
						inf.value = menuValue.."-"..(type(b) == "table" and b.key or b)

					if b.style == "Button" then
						inf.notCheckable = true
						inf.hasArrow = nil
						if not b.func then
							inf.func = function() return self:ToggleSetting(args) end
						else
							inf.func = function() return b.func(self) end
						end
						if not b.checked then
							inf.checked = function() return self:GetSetting(args) end
						else
							inf.checked = function() return b.checked(self) end
						end
					elseif (b.style == "Toggle") or type(b) == "string" then
							-- if type(b) == "table" then
								-- inf.func = function() return self:ToggleSetting(b.key) end
								-- inf.checked = function() return self:GetSetting(b.key) end
							-- else
							if type(b) == "table" then
								tinsert(args, b.key)
							else
								tinsert(args, b)
							end
								inf.func = function() return self:ToggleSetting(args) end
								inf.checked = function() return self:GetSetting(args) end
							-- end
						else
							inf.displayMode = "MENU"
							inf.hasArrow = 1
							inf.notCheckable = true
						end
						inf.arg1 = menuValue
						inf.keepShownOnClick = true
						tinsert(optionsTable[menuValue or level], inf)
					end
				end
			end
			return optionsTable[menuValue or level]
		end
	end

	function Q_C:ToggleSetting(keys)
		if (type(keys) ~= "table") or (#keys < 2) then
			local key = type(keys) == "table" and keys[1] or keys
			self:GetSets()[key] = not self:GetSets()[key]
			if self:GetSets()[key] == false then
				self:GetSets()[key] = nil
			end
		end
		local settingsTable = self:GetSets()
		local lastKey = keys[#keys]

		for index = 1, #keys - 1 do
			local key = keys[index]
			for i, info in pairs(settingsTable) do
				if i == key then
					if type(info) == "table" then
						if (index == #keys - 1) or info[lastKey] then
							info[lastKey] = not info[lastKey]
							if info[lastKey] == false then
								info[lastKey] = nil
							end
						else
							settingsTable = info
						end
					end
				end
			end
		end
		self:Layout()
	end

	function Q_C:UpdateSetting(keys, value)
		if type(keys) ~= "table" then
			self:GetSets()[keys] = value
		end
		local optioninfo
		for index = 1, #keys do
			local key, optionTable = keys[index], optioninfo or self:GetSets()
			for i, info in pairs(optionTable) do
				if i == key then
					if type(info) == "table" then
						if optioninfo[#keys] then
							optioninfo[#keys] = value
						else
							optioninfo = info
						end
					end
				end
			end
		end
		self:Layout()
	end

	function Q_C:GetAdjust(key)
		for i, info in pairs(self.options) do
			if info.key == key then
				return info.displayAdjust or 1
			end
		end
	end

	function Q_C:GetRange(key)
		for i, info in pairs(self.options) do
			if info.key == key then
				return info.high, info.low
			end
		end
	end

	function Q_C:ResetSetting(setting)
		self:GetSets()[setting] = self.defaults[setting]
		self:Layout()
	end

	function Q_C:AdjustSetting(keys, value)
		if type(keys) ~= "table" then
			self:GetSets()[keys] = self:GetSets()[keys] + value
		end
		local optioninfo
		for index = 1, #keys do
			local key, optionTable = keys[index], optioninfo or self:GetSets()
			for i, info in pairs(optionTable) do
				if i == key then
					if type(info) == "table" then
						if optioninfo[#keys] then
							optioninfo[#keys] = optioninfo[#keys] + value
						else
							optioninfo = info
						end
					end
				end
			end
		end
		self:Layout()
	end

	function Q_C:GetSetting(keys)
		if type(keys) ~= "table" then
			return self:GetSets()[keys]
		end
		local optioninfo
		for index = 1, #keys do
			local key, optionTable = keys[index], optioninfo or self:GetSets()
			for i, info in pairs(optionTable) do
				if i == key then
					if type(info) == "table" then
						optioninfo = info
					else
						return info
					end
				end
			end
		end
	end
end

do -- slash commands
	local function finder(cmd)
		local coords
		if strmatch(cmd, "%d+%p%d+%s+%d+%p%d+") then
			return "%d+%p%d+%s+%d+%p%d+"
		end
		if strmatch(cmd,"%d+%p%d+%p+%s+%d+%p%d+") then
			return "%d+%p%d+%p+%s+%d+%p%d+"
		end
		if strmatch(cmd, "%d+%p+%s+%d+") then
			return "%d+%p+%s+%d+"
		end
		if strmatch(cmd, "%d+%s+%d+") then
			return "%d+%s+%d+"
		end
		if strmatch(cmd, "%d+%p+%d+") then
			return "%d+%p+%d+"
		end
	end

	SLASH_QUESTCOMPASS1 = "/qway"
	SLASH_QUESTCOMPASSWAY1 = "/way"

	SlashCmdList["QUESTCOMPASS"] = function(Cmd, ...)
		if string.find(Cmd, "reset", 1) then
			Quest_CompassDB = CopyTable(Q_C.defaults)
			Q_C:Layout()
		elseif string.find(Cmd, "save", 1) then
			Q_C:SavePrompt()
		elseif string.find(Cmd, "import", 1) then
			if TomTomWaypointsM and TomTomWaypointsM.profiles then
				local info = {}
				for i, profile in pairs(TomTomWaypointsM.profiles) do
					for mapID, inf in pairs(profile) do
						for mapID, inf in pairs(inf) do
							tinsert(info, {position = {x = inf[2], y = inf[3], mapID = inf[1]}, name = inf.title})
						end
					end
				end

				for i, b in pairs(info) do
					for q,z in pairs(Quest_CompassAltDB) do
						--ignore and remove duplicates before importing
						if (math.floor(z.x*100) == math.floor(b.x*100)) and (math.floor(z.y*100) == math.floor(b.y*100)) and (z.name == b.name) and (z.mapID == b.mapID) then
							tremove(info, i)
						end
					end
				end

				for i, b in pairs(info) do
					tinsert(Quest_CompassAltDB, b)
				end
				print("Quest_Compass:", #info, "Waypoints have been imported from TomTom!")
			end
		else
			Q_C:OnMouseDown("RightButton")
		end
	end

	SlashCmdList["QUESTCOMPASSWAY"] = function(Cmd)
		local cmd, _match = Cmd, finder(Cmd)
		local coords = strmatch(cmd, _match)
		local x = coords and  strmatch(coords, "%d+%p%d+") or strmatch(coords, "%d+")
		local y = coords and string.trim(string.gsub(string.gsub(coords, x, ""), ",", ""))
		local startPos, endPos = string.find(Cmd, _match)

		local z = strsub(Cmd, 1, startPos-1)
		local desc = strsub(Cmd, endPos+1)

		local zone, zoneID

		for name, mapId in pairs(HBD.NameToMapId) do
			local first,last = string.split(":", z)
			if first and strmatch(first, name) then
				zoneID = mapId
				zone = name
				break --exact match
			elseif strmatch(name, z) then
				zoneID = mapId
				zone = name
			end
		end

		tinsert(Q_C.travelQueues.userSelected, {zoneID, x/100, y/100, desc, zone})
	end
end

do --questra!
	Questra = Questra or {}

	local function GetPlayerMapID()
		return Q_C:GetPlayerMapID()
	end

	local function GetViewedMapID()
		return Q_C:GetViewedMapID()
	end

	-- local edit = CreateFrame("EditBox", addonName.."EditBoxHelper", UIParent, "InputBoxTemplate")
	-- Questra.editor = edit
	-- edit:SetFontObject("GameFontHighlight")
	-- edit:SetSize(100, 100)
	-- edit:SetPoint("Left", 30, 0)
	-- edit:SetAutoFocus(false)
	-- edit:SetMultiLine("true")
	-- edit:SetScript("OnMouseDown", function(n, btn) PrintViewed(btn) end)
	local lastViewed
	function PrintViewed(btn)
		--for debug purposes
		--if IsShiftKeyDown() then
			local viewed = GetViewedMapID()
			if btn == "RightButton" then return print(viewed) end
		--	if lastViewed ~= viewed then

				-- local x, y, id = Q_C:GetPlayerPosition()

				-- edit:SetText("destination = {\n".."					id = "..id..",\n".."					x = "..x..",\n".."					y = "..y..",\n				},")

				-- local b = GetQuestBountyInfoForMapID(viewed)
				-- for i, bounties in pairs(b) do
					-- for i, bounty in pairs(bounties) do
						-- --for i, bounty in pairs(b.icon) do
							-- print(i, bounty)
						-- --end
					-- end
				-- end
				lastViewed = viewed
		--	end
		--end
	end

	function Questra:GetQuestLocation(questId, noLocalize)
		if not questId then
			return
		end

		local playerMapID = GetViewedMapID()

		if  QuestUtils_IsQuestWorldQuest(questId) == true then
			local mapID = C_TaskQuest.GetQuestZoneID(questId) or GetQuestUiMapID(questId) or playerMapID
			local x, y = C_TaskQuest.GetQuestLocation(questId, mapID)

			if noLocalize then
				return x, y, mapID
			end

			local _x, _y = HBD:TranslateZoneCoordinates(x, y, mapID, HBD.GetWorldMapIDFromMapID[mapID], true)
			local _x, _y = HBD:TranslateZoneCoordinates(_x, _y, HBD.GetWorldMapIDFromMapID[mapID], playerMapID, true)

			return _x or x , _y or y, playerMapID or mapID
		else
			local mapID, x, y = C_QuestLog.GetNextWaypoint(questId)
			if not (x and y) then
				mapID = GetQuestUiMapID(questId) or playerMapID
				local mapQuests = C_QuestLog.GetQuestsOnMap(mapID)
				if mapQuests then
					for i, info in ipairs(mapQuests) do
						if questId == (info.questId or info.questID) then
							if mapID == GetPlayerMapID() then
								return info.x, info.y, mapID
							end

							if noLocalize then
								return info.x, info.y, mapID
							end

							local _x, _y = HBD:TranslateZoneCoordinates(info.x, info.y, mapID, HBD.GetWorldMapIDFromMapID[mapID], true)
							local _x, _y = HBD:TranslateZoneCoordinates(_x, _y, HBD.GetWorldMapIDFromMapID[mapID], playerMapID, true)
							return _x or x, _y or y, playerMapID or mapID
						end
					end
				end
			end
			return x, y, mapID
		end
	end

	local questMapInfo = {}
	function Questra:GetQuestsForMap(uiMapID)
		wipe(questMapInfo)
		local uiMapID = uiMapID or GetViewedMapID()

		local quests = GetPlayerMapID() and C_QuestLog.GetQuestsOnMap(GetPlayerMapID())
		if quests and #quests > 0 then
				for i, info in ipairs(quests) do
					tinsert(questMapInfo, info)
				end
			return
		end

		local mapQuests = C_QuestLog.GetQuestsOnMap(uiMapID)
		if mapQuests then
			for i, info in ipairs(mapQuests) do
				tinsert(questMapInfo, info)
			end
		end
		return questMapInfo
	end

	local FactionQuests = {}

	function Questra:HasFactionQuests(factionID)
		if FactionQuests[factionID] then
			return FactionQuests[factionID]
		else
			return false
		end

	end

	local mapCount = {}

	local worldQuestMapInfo = {}
	local info = {}
	function Questra:GetWorldQuestsForMap(uiMapID)
		local uiMapID = uiMapID or GetViewedMapID()

		local taskInfo
		if (uiMapID) then
			taskInfo = C_TaskQuest.GetQuestsForPlayerByMapID(uiMapID)
		end

		mapCount[uiMapID] = mapCount[uiMapID] or {}

		if taskInfo and #taskInfo ~= mapCount[uiMapID] then
			FactionQuests = {}
		end

		mapCount[uiMapID] = #taskInfo

		if taskInfo then
			for i, info in ipairs(taskInfo) do
				local ID = quest.questID or quest.questId
				if QuestUtils_IsQuestWorldQuest(questID) and HaveQuestData(questID) then
					local info = questID and C_QuestLog.GetQuestTagInfo(questID)
						-- .quality
						-- .isElite
						-- .worldQuestType
						-- .tagID
						-- .displayExpiration
						-- .tagName

					local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName

					local info = {}

					local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)

					if not factionID then
						title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)
					end

					if factionID then
						FactionQuests[factionID] = FactionQuests[factionID] or {}
						FactionQuests[factionID][questID] = true
					end

					info.x, info.y, info.mapID = self:GetQuestLocation(questID)

					info.questID = questID
					info.tagID = tagID
					info.tagName = tagName
					info.worldQuestType = worldQuestType
					info.rarity = rarity
					info.isElite = isElite
					info.tradeskillLineIndex = tradeskillLineIndex
					info.tagID = displayTimeLeft

					tinsert(worldQuestMapInfo, info)
				end
			end
		end
		return worldQuestMapInfo
	end

	local blankFunc = function() end

	local worldQuestTypes = {
		[Enum.QuestTagType.PetBattle] = "Battle Pet",
		[Enum.QuestTagType.Dungeon] = "Dungeon",
		[Enum.QuestTagType.PvP] = "PvP",
		[Enum.QuestTagType.Profession] = "Profession",
		[Enum.QuestTagType.Raid] = "Raid",
		[Enum.QuestTagType.Invasion] = "Invasion",
		--[Enum.QuestTagType.FactionAssault]= "Faction",
		--[Enum.QuestTagType.Islands]= "Island",
		--[Enum.QuestTagType.Threat]= "N'Zoth",
	}

	function Q_C:ShouldIgnore(questID)


		local info = questID and C_QuestLog.GetQuestTagInfo(questID)
			-- .quality
			-- .isElite
			-- .worldQuestType
			-- .tagID
			-- .displayExpiration
			-- .tagName

		if not info then
			return
		end

		local index, questType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName
		if not questID then return end
		local name, _, _, _, _, watched = GetWatchedFactionInfo()
		local hasWatched = Questra:HasFactionQuests(watched)




		local tracked = Q_C:GetSets().priorityFaction or 1
		local hasTracked = Questra:HasFactionQuests(tracked)

		local _, factionID = C_TaskQuest.GetQuestInfoByQuestID(questID)
		if not factionID then
			_, factionID = C_TaskQuest.GetQuestInfoByQuestID(questID)
		end

		if factionID then
			if tracked and hasTracked and tracked ~= factionID then
				return true
			end
			if watched and hasWatched and watched ~= factionID then
				return true
			end
		end
		Q_C.sessionIgnore = Q_C.sessionIgnore or {}

				local isTracked = (questID == C_QuestSession.GetSuperTrackedQuest()) or (questID == C_SuperTrack.GetSuperTrackedQuestID())


		if (((Q_C:GetSets().ignored and Q_C:GetSets().ignored[questID]) or Q_C.sessionIgnore[questID]) or
			(Q_C:GetSets().worldQuests.ignored[worldQuestTypes[questType]])) and not isTracked then
			return true
		end
	end

	local function shouldIgnore(...)
		return Q_C:ShouldIgnore(...)
	end

	local once

	function Questra:GetNearestQuestID(uiMapID, fallBackID)

		local numEntries, numQuests = C_QuestLog.GetNumQuestLogEntries()
		local distance = math.huge
		local questID


if not once then
	once = true
		-- If nothing with POI data is being tracked expand search to quest log
		if not closestQuestID then
			for questLogIndex = 1, C_QuestLog.GetNumQuestLogEntries() do

				local info = C_QuestLog.GetInfo(questLogIndex);

			if info.questID == (C_QuestSession.GetSuperTrackedQuest() or C_SuperTrack.GetSuperTrackedQuestID()) or (info.isTask == true) then
				info.questID = nil
			end

			if info.questID then
				local dist = self:GetDistanceToQuest(info.questID)
				if dist < distance and (shouldIgnore(info.questID) == nil) then
					questID = info.questID
					distance = dist
				end
			end

			end
		end

end



		-- for questLogIndex = 1, numQuests do
			-- local title, level, suggestedGroup, isHeader, isCollapsed, isComplete, frequency, questLogIndex, startEvent, questID isOnMap, hasLocalPOI, isTask, isBounty, isStory, isHidden, isScaling = GetQuestLogTitle(questLogIndex)


		-- end
		return questID
	end

	function Questra:GetNearestWorldQuestID(MapID, fallBackID)
		local uiMapID

		local CurrentZoneQuests = GetPlayerMapID() and C_TaskQuest.GetQuestsForPlayerByMapID(GetPlayerMapID()) or {}

		if CurrentZoneQuests and #CurrentZoneQuests > 0 then

			local distance, questId = math.huge

			if CurrentZoneQuests and #CurrentZoneQuests ~= mapCount[GetPlayerMapID()] then
				FactionQuests = {}
			end
			mapCount[GetPlayerMapID()] = #CurrentZoneQuests

			for i, quest in ipairs(CurrentZoneQuests) do
				local ID = quest.questID or quest.questId
				if ID then
					local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(ID)

					if not factionID then
						title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(ID)
					end

					if factionID then
						FactionQuests[factionID] = FactionQuests[factionID] or true
					end
					if QuestUtils_IsQuestWorldQuest(ID) and HaveQuestData(ID) then
						local dist = self:GetDistanceToQuest(ID)
						local info = ID and C_QuestLog.GetQuestTagInfo(ID)
							-- .quality
							-- .isElite
							-- .worldQuestType
							-- .tagID
							-- .displayExpiration
							-- .tagName

						local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName
						if dist == math.huge then

						end

						if ((dist and dist <= distance) and (shouldIgnore(ID, worldQuestType) == nil)) and (not (ID == C_QuestSession.GetSuperTrackedQuest()) or (ID == C_SuperTrack.GetSuperTrackedQuestID())) then
							distance = dist
							questId = ID
						end
					end
				end
			end
			if distance ~= math.huge and (Q_C:ShouldIgnore(questId) == nil) then
				return questId
			end
		end

		if MapID == true then
			uiMapID = Q_C.digForZone(GetPlayerMapID())
		else

			uiMapID = HBD.GetWorldMapIDFromMapID[ GetPlayerMapID()] or GetPlayerMapID()
		end

		local quests = uiMapID and C_TaskQuest.GetQuestsForPlayerByMapID(uiMapID) or {}

		local distance, questId = math.huge

		if quests and #quests ~= mapCount[uiMapID] then
			FactionQuests = {}
		end
		if uiMapID then
			mapCount[uiMapID] = #quests
		end
		for i, quest in ipairs(quests) do
			local ID = quest.questID or quest.questId
			if ID == (C_QuestSession.GetSuperTrackedQuest() or C_SuperTrack.GetSuperTrackedQuestID()) then
				ID = nil
			end

			if ID then
					local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(ID)

					if not factionID then
						title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(ID)
					end

					if factionID then
						FactionQuests[factionID] = FactionQuests[factionID] or true
					end
				if QuestUtils_IsQuestWorldQuest(ID) and HaveQuestData(ID) then
					local dist = self:GetDistanceToQuest(ID)
					local info = ID and C_QuestLog.GetQuestTagInfo(ID)
						-- .quality
						-- .isElite
						-- .worldQuestType
						-- .tagID
						-- .displayExpiration
						-- .tagName

					local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName
								if dist == math.huge then

					end
					if (dist and dist <= distance) and (Q_C:ShouldIgnore(ID) == nil) then

						distance = dist
						questId = ID
					end
				end
			end
		end
		if distance ~= math.huge then
			return questId
		end
	end

	function Questra:TranlsateCoords(sourceX, sourceY, sourceID, destinationID)
		if sourceID == destinationID then
			return sourceX, sourceY, sourceID
		end

		local worldID = HBD.GetWorldMapIDFromMapID[sourceID]
		local destX, destY = sourceX, sourceY

		if worldID and worldID ~= sourceID then
			--try to convert zone coords to world coords
			local wX, wY = HBD:TranslateZoneCoordinates(sourceX, sourceY, sourceID, worldID, true)
			if wX and wY then
				destX, destY, worldID = wX, wY, worldID
			else
				--if conversion fails, default to zone coords
				worldID = sourceID
			end
		else
			destX, destY, worldID = sourceX, sourceY, sourceID
		end

		if destinationID then
			if destinationID ~= sourceID then
				--if a destination is provided, try to convert to the destination
				local dX, dY = HBD:TranslateZoneCoordinates(destX, destY, worldID, destinationID, true)
				if dX and dY then
					destX, destY, worldID = dX, dY, destinationID
				end
			else
				--destination same as source, cancel conversion
				destX, destY, worldID = sourceX, sourceY, sourceID
			end
		end

		return destX, destY, worldID
	end

	function Questra:GetDistanceToQuest(questId)
		local _pX, _pY, playerMapID = HBD:GetPlayerZonePosition(true)
		_pX, _pY = _pX or 0, _pY or 0

		local x, y, uiMapID = self:GetQuestLocation(questId)

		local pX, pY = Questra:TranlsateCoords(_pX, _pY, playerMapID, GetViewedMapID())
		local _x, _y = Questra:TranlsateCoords(x, y, uiMapID, GetViewedMapID())

		if HBD.GetWorldMapIDFromMapID[playerMapID] == HBD.GetWorldMapIDFromMapID[uiMapID] then
			if (not (_x and _y)) or (not (pX and pY)) then
				if x and y and _pX and _pY then
					return math.sqrt(math.pow((_pX - x), 2) + math.pow((_pY - y), 2))
				end
			end
		end

		if (not (_x and _y)) or (not (pX and pY)) then
			return math.huge
		end

		return math.sqrt(math.pow((pX - _x), 2) + math.pow((pY - _y), 2))
	end

	function Questra:GetDistanceBetween(x, y, uiMapID, oX, oY, oUiMapID)

		if (not (x and y)) or (not (oX and oY)) then
			return math.huge
		end

		return math.sqrt(math.pow((oX - x), 2) + math.pow((oY - y), 2))
	end

	function Questra:GetDistanceToLocation(x, y, uiMapID)
		local _pX, _pY, playerMapID = HBD:GetPlayerZonePosition(true)
		_pX, _pY = _pX or 0, _pY or 0

		local _pX, _pY, playerMapID = Questra:TranlsateCoords(_pX, _pY, playerMapID)
		local _x, _y, uiMapID = Questra:TranlsateCoords(x, y, HBD.GetWorldMapIDFromMapID[uiMapID])

		if (not (_x and _y)) or (not (_pX and _pY)) then
			return math.huge
		end

		return math.sqrt(math.pow((_pX - _x), 2) + math.pow((_pY - _y), 2))
	end

	function Questra:GetDirectionToQuest(questId, qx, qy, uiMapID)
		if questId then
			--qx, qy, uiMapID = self:GetQuestLocation(questId)
		end

		local _pX, _pY, playerMapID = Q_C:GetPlayerPosition()
		local x, y, uiMapID = Questra:TranlsateCoords(qx, qy, uiMapID, playerMapID)

		local xDist = x - _pX
		local yDist = y - _pY

		local dist = math.sqrt(math.pow((xDist), 2) + math.pow((yDist), 2))
		
		local angle = 90 - acos(xDist/dist)

		if angle < 0 then
			angle = 360 + angle
		end
		if y > _pY then
			angle = 270 - math.abs(270 - angle)
			if x > _pX then
				angle = 180 - angle
			end
		end

		angle = angle * math.pi /180

		local arrowAngle = ((GetPlayerFacing() or 0) + angle)

		return -arrowAngle, -arrowAngle
	end

	local questInfo = {}

	function Questra:GetQuestInfo(questId)
		wipe(questInfo)

		questInfo.x, questInfo.y, questInfo.mapID = self:GetQuestLocation(questId)
		questInfo.title, questInfo.factionID, questInfo.capped = C_TaskQuest.GetQuestInfoByQuestID(questId) or C_TaskQuest.GetQuestInfoByQuestID(questId)


		local info = questID and C_QuestLog.GetQuestTagInfo(questId)
			-- .quality
			-- .isElite
			-- .worldQuestType
			-- .tagID
			-- .displayExpiration
			-- .tagName


		questInfo = info
		questInfo.distance = self:GetDistanceToQuest(questId)

		return questInfo
	end

	function Questra:TranslateCoords(sourceX, sourceY, sourceID, destinationID, bool)
		if sourceID == destinationID then
			return sourceX, sourceY, sourceID
		else
			return HBD:TranslateZoneCoordinates(sourceX, sourceY, sourceID, destinationID, bool)
		end

	end

end

local function GetQuestType(questID)
	C_QuestLog.IsQuestTask(questID)

	C_QuestLog.IsQuestDisabledForSession(questID)
	C_QuestLog.IsQuestFlaggedCompleted(questID)
	C_QuestLog.IsQuestReplayable(questID)
	C_QuestLog.IsQuestReplayedRecently(questID)

	IsQuestInvasion(questID)

	IsQuestBounty(questID)
	IsQuestCriteriaForBounty(questID, bountyQuestID)

	IsQuestFlaggedCompleted(questID)

		local info = questID and C_QuestLog.GetQuestTagInfo(questID)
			-- .quality
			-- .isElite
			-- .worldQuestType
			-- .tagID
			-- .displayExpiration
			-- .tagName

		local tagID, worldQuestType, rarity, displayTimeLeft, isElite, tagName = info and info.tagID, info and info.worldQuestType, info and info.quality, info and info.displayExpiration, info and info.isElite, info and info.tagName
	-- Solo quests — the most common kind.

	-- Group quests — suggested for more than 1 player.

	-- Dungeon quests — objective of the quest is inside an instance.

	-- Heroic quests — objective of the quest is inside a Heroic-mode version of an instance.

	-- Legendary quests — usually the main story-line, such as the War Campaign

	-- Raid quests — objective of the quest is inside a raid instance or involves a raid boss.

	-- PvP quests — quests that require you to complete them while flagged for PvP.

	-- Repeatable quests — can be soloable or for groups with the primary intention to boost reputation with a faction.

	-- Daily quests — special repeatable type of solo quest introduced in The Burning Crusade for those at the level cap to primarily generate money, but changed in to be used more as an RRQ also.

	-- Seasonal quests — available only for limited periods of time and often related to events or holidays.

	-- Bonus Objectives — objectives for entering certain areas.
end

	function WorldMapFrame:SetOverlayFrameLocation(frame, location)
		frame:ClearAllPoints()
		frame:SetPoint("BottomLeft", WorldMapFrame, "BottomLeft", 268, -75)
		frame:SetFrameStrata("FULLSCREEN")
		frame:SetScale(.9)
	end
	
	
	
	
	
	
	
	
	function HookMapButtons()
		local kids = WorldMapFrame.ScrollContainer:GetChildren()
		for i, b in pairs(kids) do
			print(i, b)
		end
		
		--print(#WorldMapFrame.ScrollContainer.Child:GetChildren())
	end
	
	
